package ru.crp.lvms.cluster.model;

public class ClusterProperties {
    private NodePropertyInfo nodes;
    private NodePropertyInfo cores;
    private NodePropertyInfo memory;

    public static ClusterProperties createEmptyProperties() {
        return new ClusterProperties(NodePropertyInfo.createEmptyInfo(),
                NodePropertyInfo.createEmptyInfo(),
                NodePropertyInfo.createEmptyInfo());
    }

    public ClusterProperties(NodePropertyInfo nodes, NodePropertyInfo cores, NodePropertyInfo memory) {
        this.nodes = nodes;
        this.cores = cores;
        this.memory = memory;
    }

    public NodePropertyInfo getNodes() {
        return nodes;
    }

    public NodePropertyInfo getCores() {
        return cores;
    }

    public NodePropertyInfo getMemory() {
        return memory;
    }

    public void addNode(NodeForClusterInfoEntity node, String solverName) {
        if (!node.isEnabled()) {
            addDisabled(node);
        } else if (!node.isAlive() || !node.getEnabledSolvers().contains(solverName)) {
            addUnavailable(node);
        } else if (node.isBusy()) {
            addAcquired(node);
        } else {
            addFree(node);
        }
    }


    public void addNode(NodeForClusterInfoEntity node) {
        if (!node.isEnabled()) {
            addDisabled(node);
        } else if (!node.isAlive()) {
            addUnavailable(node);
        } else if (node.isBusy()) {
            addAcquired(node);
        } else {
            addFree(node);
        }
    }

    private void addDisabled(NodeForClusterInfoEntity node) {
        nodes.addDisabled(1);
        cores.addDisabled(node.getNumOfCores());
        memory.addDisabled(node.getMemory());
    }

    private void addFree(NodeForClusterInfoEntity node) {
        nodes.addFree(1);
        cores.addFree(node.getNumOfCores());
        memory.addFree(node.getMemory());
    }

    private void addAcquired(NodeForClusterInfoEntity node) {
        nodes.addAcquired(1);
        cores.addAcquired(node.getNumOfCores());
        memory.addAcquired(node.getMemory());
    }

    private void addUnavailable(NodeForClusterInfoEntity node) {
        nodes.addUnavailable(1);
        cores.addUnavailable(node.getNumOfCores());
        memory.addUnavailable(node.getMemory());
    }
}
