package ru.crp.lvms.cluster.model;

import org.apache.commons.lang3.time.FastDateFormat;
import ru.crp.lvms.utils.Utils;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class NodeStatistics {
    private class StatisticsEntry {
        public String name;
        public String value;

        public StatisticsEntry(String name, String value) {
            this.name = name;
            this.value = value;
        }
    }

    private static final String TASKS_COUNT_KEY = "Tasks count";
    private static final String AVERAGE_TIME_KEY = "Average time (min)";
    private static final String MINIMUM_TIME_KEY = "Minimum time (min)";
    private static final String MAXIMUM_TIME_KEY = "Maximum time (min)";
    private static final String LAST_USAGE_KEY = "Last usage";
    private static final String MEMORY_USAGE_KEY = "Average memory usage";

    private static final String NO_VALUE_LABEL = "-";

    private static final FastDateFormat dateFormat = FastDateFormat.getInstance("yyy-MM-dd-hh-mm-ss");

    private List<StatisticsEntry> properties;
    private int nodeId;

    public NodeStatistics(int nodeId) {
        this.nodeId = nodeId;
        properties = new ArrayList<>();
    }

    public int getNodeId() {
        return nodeId;
    }

    public List<StatisticsEntry> getProperties() {
        return properties;
    }

    public void setTasksCount(Integer tasksCount) {
        properties.add(new StatisticsEntry(TASKS_COUNT_KEY, tasksCount.toString()));
    }

    public void setAverageTime(@Nullable Integer averageTime) {
        properties.add(new StatisticsEntry(AVERAGE_TIME_KEY, averageTime == null ? NO_VALUE_LABEL : averageTime.toString()));
    }

    public void setMinimumTime(@Nullable Integer minimumTime) {
        properties.add(new StatisticsEntry(MINIMUM_TIME_KEY, minimumTime == null ? NO_VALUE_LABEL : minimumTime.toString()));
    }

    public void setMaximumTime(@Nullable Integer maximumTime) {
        properties.add(new StatisticsEntry(MAXIMUM_TIME_KEY, maximumTime == null ? NO_VALUE_LABEL : maximumTime.toString()));
    }

    public void setLastUsage(@Nullable Date date) {
        properties.add(new StatisticsEntry(LAST_USAGE_KEY, date == null ? NO_VALUE_LABEL : dateFormat.format(date)));
    }

    public void setMemoryUsage(@Nullable Long memoryInMegabytes) {
        properties.add(new StatisticsEntry(MEMORY_USAGE_KEY, memoryInMegabytes == null ?
                NO_VALUE_LABEL : Utils.humanReadableByteCount(memoryInMegabytes * 1024 * 1024)));
    }
}
