package ru.crp.lvms.managers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.crp.cmlb.model.entities.base.Named;
import ru.crp.lvms.managers.misc.BaseManagerSuggest;
import ru.crp.lvms.model.dao.BaseSpaceDao;
import ru.crp.lvms.model.dao.MaterialDao;
import ru.crp.lvms.model.dao.ParameterValuesDao;
import ru.crp.lvms.model.entities.FileEntity;
import ru.crp.lvms.model.entities.MaterialEntity;
import ru.crp.lvms.model.entities.UserEntity;
import ru.crp.lvms.optimizations.base.managers.OptimizationManager;
import ru.crp.lvms.utils.MessageUtils;
import ru.crp.lvms.utils.ValidationHelper;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.List;

import static ru.crp.lvms.model.AccessRights.update;
import static ru.crp.lvms.utils.MessageUtils.COMPOSITE_MATERIAL_HAS_MATERIAL_FILES;
import static ru.crp.lvms.utils.MessageUtils.MATERIAL_USED_IN_COMPOSITE_MATERIALS;


@Service
public class MaterialManager extends BaseManagerSuggest<MaterialEntity> {
    @Autowired
    private CompositeMaterialManager compositeMaterialManager;

    @Autowired
    private MaterialDao materialDao;

    @Autowired
    private ParameterValuesDao parameterValuesDao;

    @Autowired
    private FileManager fileManager;

    @Autowired
    private ValidationHelper validationHelper;

    @Autowired
    private OptimizationManager optimizationManager;


    @Override
    protected BaseSpaceDao<MaterialEntity> aliveDao() {
        return materialDao;
    }


    public void extendMaterial(MaterialEntity material, UserEntity currentUser) {
        material.setMaterialFiles(fileManager.getMaterialFiles(false, material.getId(), currentUser));
        material.setParameterValues(parameterValuesDao.findByMaterialId(material.getId()));
    }


    @Override
    public void delete(MaterialEntity entity, @Nullable UserEntity currentUser) {
        checkDelete(entity, currentUser);
        parameterValuesDao.deleteAllParametersForMaterial(entity.getId());
        aliveDao().delete(entity.getId());
    }


    @Override
    public void checkDelete(MaterialEntity entity, @Nullable UserEntity currentUser) {
        validationHelper.checkDependencies(entity.getId(), MATERIAL_USED_IN_COMPOSITE_MATERIALS,
                compositeMaterialManager::getCompositeMaterialsUsingMaterial, currentUser);
        validationHelper.checkDependencies(entity.getId(), COMPOSITE_MATERIAL_HAS_MATERIAL_FILES,
                this::getMaterialFiles, currentUser);
        super.checkDelete(entity, currentUser);
    }


    private List<? extends Named> getMaterialFiles(Integer materialId, UserEntity currentUser) {
        return fileManager.getMaterialFiles(false, materialId, currentUser);
    }


    @Nonnull
    public FileEntity deleteMaterialFile(@Nullable Integer materialId, int materialFileId,
                                         UserEntity currentUser) {
        if (materialId != null) {
            checkEntityExistsAndAccesible(materialId, update, currentUser);
        }
        validationHelper.checkDependencies(materialFileId, MessageUtils.FILE_IS_USED_IN_OPTIMIZATION,
                this::getConnectedOptimisations, currentUser);
        return fileManager.deleteFile(materialFileId, currentUser);
    }


    private List<? extends Named> getConnectedOptimisations(Integer materialFileId, UserEntity unused) {
        return optimizationManager.getOptimizationsThatUseFile(materialFileId, unused);
    }
}
