package ru.crp.lvms.managers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.stereotype.Service;
import ru.crp.lvms.exceptions.EntityDeleteException;
import ru.crp.lvms.managers.misc.BaseManagerSuggest;
import ru.crp.lvms.model.dao.BaseSpaceDao;
import ru.crp.lvms.model.dao.UserDao;
import ru.crp.lvms.model.entities.UserEntity;
import ru.crp.lvms.utils.ValidationHelper;

import javax.annotation.Nullable;

import static ru.crp.lvms.utils.MessageUtils.*;

@Service
public class UserManager extends BaseManagerSuggest<UserEntity> {

    @Autowired
    private UserDao userDao;

    @Autowired
    private MessageSource msgSrc;

    @Autowired
    private ValidationHelper validationHelper;

    @Autowired
    private TechnologyManager technologyManager;

    @Autowired
    private MaterialManager materialManager;

    @Autowired
    private CompositeMaterialManager compositeMaterialManager;

    @Autowired
    private FileManager fileManager;


    @Override
    protected BaseSpaceDao<UserEntity> aliveDao() {
        return userDao;
    }

    @Override
    public void checkDelete(UserEntity entity, @Nullable UserEntity currentUser) {
        if (currentUser == null) {
            throw new AccessDeniedException(msgSrc.getMessage(USER_DELETE_ITSELF, null, null));
        }
        if (entity.getId() == currentUser.getId()) {
            throw new EntityDeleteException(msgSrc.getMessage(USER_DELETE_ITSELF, null, null));
        }
        validationHelper.checkDependencies(entity.getId(), USER_HAS_TECHNOLOGIES,
                technologyManager::findByOwnerId, currentUser);

        validationHelper.checkDependencies(entity.getId(), USER_HAS_MATERIALS,
                materialManager::findByOwnerId, currentUser);

        validationHelper.checkDependencies(entity.getId(), USER_HAS_COMPOSITE_MATERIALS,
                compositeMaterialManager::findByOwnerId, currentUser);

        validationHelper.checkDependencies(entity.getId(), USER_HAS_FILES,
                fileManager::findByOwnerId, currentUser);

        super.checkDelete(entity, currentUser);
    }
}
