package ru.crp.lvms.managers.tasks.resultsHandlers;

import org.apache.commons.io.FilenameUtils;
import ru.crp.cmls.calculations.solvers.xml.config.ResultType;
import ru.crp.lvms.exceptions.solving.ResultProcessingException;
import ru.crp.lvms.managers.tasks.ResultsParent;
import ru.crp.lvms.model.entities.ResultEntity;
import ru.crp.lvms.model.entities.misc.CreatedBy;

import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;

import static java.util.Collections.singletonList;
import static java.util.stream.Collectors.toList;

public class ValuesResultHandler extends KeyResultHandler {

    public ValuesResultHandler(Path pathToStorage) {
        super(pathToStorage);
    }

    @Override
    public boolean canHandle(ResultType resultType, Path pathToFile) {
        return resultType == ResultType.values && Files.isRegularFile(pathToFile);
    }

    @Override
    protected List<ResultEntity> doHandleResultsFile(Path pathToFile, ResultsParent resultsParent,
                                                     CreatedBy createdBy) {
        if (isTrueValueResults(pathToFile)) {
            try {
                return Files.lines(pathToFile, Charset.forName("CP1251"))
                        .map(str -> {
                            String[] valdata = str.split(",");
                            ResultEntity newValue = initCommonResultProperties(pathToFile, resultsParent, createdBy);
                            newValue.setName(valdata[0]);
                            newValue.setType(ResultType.values);
                            newValue.setValue(valdata[1]+" "+valdata[2]);
                            if (valdata.length > 3) {
                                newValue.setTargetValue(valdata[3] + " " + valdata[4] + " " + valdata[2]);
                            }
                            return newValue;
                        })
                        .collect(toList());
            }
            catch (IOException ex) {
            throw new ResultProcessingException(ex);
        }
        }
        ResultEntity newDatabase = initCommonResultProperties(pathToFile, resultsParent, createdBy);
        if (createdBy == CreatedBy.Auto) {
            extractDescription(pathToFile, newDatabase);
        }

        newDatabase.setName(pathToFile.getFileName().toString());
        newDatabase.setType(ResultType.values);

        return singletonList(newDatabase);
    }

    public static boolean isTrueValueResults(Path valuesFile) {
        String name = FilenameUtils.getName(valuesFile.toString());
        return "values.txt".equalsIgnoreCase(name);
    }
}
