package ru.crp.lvms.model.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Repository;
import ru.crp.lvms.managers.ParametersManager;
import ru.crp.lvms.model.entities.effective.characteristics.ECEntity;
import ru.crp.lvms.model.entities.effective.characteristics.ECEntityTable;
import ru.crp.lvms.model.entities.effective.characteristics.ECParamEntity;
import ru.crp.lvms.model.entities.technology.TechnologyEntityTable;
import ru.crp.lvms.model.entities.technology.TechnologySimulationsEntity;
import ru.crp.lvms.model.entities.technology.TechnologySimulationsParamEntity;
import ru.crp.lvms.tasks.fileGenerators.TSParameters;
import ru.crp.lvms.tasks.fileGenerators.ThermalSimulationParameters;

import java.math.BigDecimal;
import java.util.List;

@Repository
@Primary
public class TechnologySimulationsDao extends BaseSpaceDao<TechnologySimulationsEntity> {

    private static final int MIN_RADIUS = 1;
    private static final int NUM_LEVELS = 2;
    private static final int VERTICAL_STEP = 3;
    private static final int LINE_FEED_RATE = 4;
    private static final int LINE_EXTRUSION_RATE = 5;
    private static final int ARC_FEED_RATE = 6;
    private static final int ARC_EXTRUSION_RATE = 7;
    private static final int PRINT_RECT_MIN_X = 8;
    private static final int PRINT_RECT_MIN_Y = 9;
    private static final int PRINT_RECT_MAX_X = 10;
    private static final int PRINT_RECT_MAX_Y = 11;
    private static final int USEFUL_RECT_MIN_X = 12;
    private static final int USEFUL_RECT_MIN_Y = 13;
    private static final int USEFUL_RECT_MAX_X = 14;
    private static final int USEFUL_RECT_MAX_Y = 15;

    //  @Autowired
  //  private TechnologySimulationsDao technologySimulationsDao;

    public List<TechnologySimulationsEntity> getByTechnologyId(Integer technologySimulationsId) {
        return find("WHERE technology_simulations_id = ? ORDER BY param_id", technologySimulationsId);
    }


    public List<Integer> getAllParamIds() {
        return jdbcTemplate().queryForList("SELECT id FROM technology_simulations_parameters", Integer.class);
    }


    private String getTechnologySimulationsParameters(Integer id, Integer paramId) {
        List<String> resp = jdbcTemplate().query("SELECT param_value " +
                        "FROM   technology_simulations_parameters_view AS tspv " +
                        "WHERE  tspv.technology_simulations_id = ? " +
                        "AND    tspv.param_id = ?;",
                new Object[]{id, paramId},
                (rs, rowNum) -> rs.getString("param_value")
        );
        return resp.isEmpty() ? null : resp.get(0);
    }

    public TSParameters getTechnologySimulationsParameters(Integer technologySimulationsId) {
        TSParameters parameters = new TSParameters();

        parameters.setMinRadius(getTechnologySimulationsParameters(technologySimulationsId, MIN_RADIUS));
        parameters.setNumLevels(getTechnologySimulationsParameters(technologySimulationsId, NUM_LEVELS));
        parameters.setVerticalStep(getTechnologySimulationsParameters(technologySimulationsId, VERTICAL_STEP));
        parameters.setLineFeedRate(getTechnologySimulationsParameters(technologySimulationsId, LINE_FEED_RATE));
        parameters.setLineExtrusionRate(getTechnologySimulationsParameters(technologySimulationsId, LINE_EXTRUSION_RATE));
        parameters.setArcFeedRate(getTechnologySimulationsParameters(technologySimulationsId, ARC_FEED_RATE));
        parameters.setArcExtrusionRate(getTechnologySimulationsParameters(technologySimulationsId, ARC_EXTRUSION_RATE));
        parameters.setPrintRectMinX(getTechnologySimulationsParameters(technologySimulationsId, PRINT_RECT_MIN_X));
        parameters.setPrintRectMinY(getTechnologySimulationsParameters(technologySimulationsId, PRINT_RECT_MIN_Y));
        parameters.setPrintRectMaxX(getTechnologySimulationsParameters(technologySimulationsId, PRINT_RECT_MAX_X));
        parameters.setPrintRectMaxY(getTechnologySimulationsParameters(technologySimulationsId, PRINT_RECT_MAX_Y));
        parameters.setUsefulRectMinX(getTechnologySimulationsParameters(technologySimulationsId, USEFUL_RECT_MIN_X));
        parameters.setUsefulRectMinY(getTechnologySimulationsParameters(technologySimulationsId, USEFUL_RECT_MIN_Y));
        parameters.setUsefulRectMaxX(getTechnologySimulationsParameters(technologySimulationsId, USEFUL_RECT_MAX_X));
        parameters.setUsefulRectMaxY(getTechnologySimulationsParameters(technologySimulationsId, USEFUL_RECT_MAX_Y));

        return parameters;
    }

}
