package ru.crp.lvms.model.entities;

import ru.crp.cmlb.model.entities.Column;
import ru.crp.cmlb.model.entities.Suggest;
import ru.crp.cmlb.model.entities.Table;
import ru.crp.cmlb.model.entities.base.Named;
import ru.crp.lvms.model.entities.misc.BaseEntityTable;
import ru.crp.lvms.model.entities.misc.StatusFlag;
import ru.crp.lvms.validators.EntityExists;
import ru.crp.lvms.validators.FileExists;
import ru.crp.lvms.web.dto.columns.GridColumn;
import ru.crp.lvms.web.dto.columns.GridColumnType;

import javax.annotation.Nullable;
import javax.validation.constraints.NotNull;

@Table("files")
public class FileEntityTable extends BaseEntityTable implements Named {
    @Suggest()
    @GridColumn(name = "name", label = "Название", order = 0, cell = GridColumnType.uri)
    @Column("name")
    @NotNull(message = "Имя файла должно быть задано")
    private String name;

    @EntityExists(entityClass = FileEntity.class, message = "Родительский файл не найден")
    @Column("reference_id")
    private Integer referenceId;

    @Column("pid")
    @Nullable
    private Integer pid;

    @GridColumn(name = "description", label = "Описание", order = 8)
    @Column("description")
    private String description;

    @Column("flag")
    private StatusFlag flag = StatusFlag.none;

    private String flagDisplayName = StatusFlag.none.getDisplayName();

    @Column("path_to_file")
    @FileExists
    @NotNull(message = "Путь к файлу должен быть задан")
    private String pathToFile;

    @Column(value = "file_type_id")
    private int fileTypeId;

    public Integer getReferenceId() {
        return referenceId;
    }

    public void setReferenceId(@Nullable Integer referenceId) {
        this.referenceId = referenceId;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public String getPathToFile() {
        return pathToFile;
    }

    public void setPathToFile(String pathToFile) {
        this.pathToFile = pathToFile;
    }

    public StatusFlag getFlag() {
        return flag;
    }

    public void setFlag(@Nullable StatusFlag flag) {
        this.flag = flag;
        if (flag == null) {
            this.flagDisplayName = null;
        } else {
            this.flagDisplayName = flag.getDisplayName();
        }
    }

    @Nullable
    public Integer getPid() {
        return pid;
    }

    public void setPid(@Nullable Integer pid) {
        this.pid = pid;
    }

    @Override
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public int getFileTypeId() {
        return fileTypeId;
    }

    public void setFileTypeId(int fileTypeId) {
        this.fileTypeId = fileTypeId;
    }

    public String getFlagDisplayName() {
        return flagDisplayName;
    }
}
