package ru.crp.lvms.model.entities.nodes;

import org.apache.commons.lang3.builder.EqualsBuilder;
import org.hibernate.validator.constraints.NotBlank;
import org.hibernate.validator.constraints.URL;
import ru.crp.cmlb.model.entities.Column;
import ru.crp.cmlb.model.entities.Table;
import ru.crp.lvms.model.entities.misc.AuditableImpl;
import ru.crp.lvms.web.dto.columns.GridColumn;
import ru.crp.lvms.web.dto.columns.GridColumnType;

import java.util.ArrayList;
import java.util.List;

import static java.util.Collections.emptyList;

@Table("nodes_alive")
public class NodeEntity extends AuditableImpl implements Cloneable {

    @NotBlank
    @GridColumn(name = "name", label = "Название", order = 0, cell = GridColumnType.uri)
    @Column("name")
    private String name;

    @URL
    @GridColumn(name = "url", label = "Адрес", order = 1)
    @Column("url")
    private String url;

    @GridColumn(name = "description", label = "Описание", order = 2)
    @Column("description")
    private String description;

    @GridColumn(name = "numOfCores", label = "Количество ядер", cell = GridColumnType.integer, order = 3)
    @Column("num_cores")
    private int numOfCores;

    @GridColumn(name = "memory", label = "Доступная память (MB)", cell = GridColumnType.integer, order = 4)
    @Column("memory")
    private long memory;

    @GridColumn(name = "isAlive", label = "Alive", order = 5)
    @Column("alive")
    private boolean isAlive;

    @Column("enabled")
    private boolean isEnabled;

    @GridColumn(name = "owner", label = "Owner", cell = GridColumnType.uri, order = 102)
    @Column(value = "owner", onlySelect = true)
    private String owner;

    @Column("cluster_id")
    private int clusterId;

    // name should be synchronized with property name in CustomBeanPropertyRowMapper
    @Column(value = "available_solvers", onlySelect = true)
    private List<String> availableSolvers = emptyList();

    public static boolean equalsIgnoringModificationDate(NodeEntity first, NodeEntity second) {
        return EqualsBuilder.reflectionEquals(first, second, "modificationDate");
    }

    @Override
    public Object clone() throws CloneNotSupportedException {
        NodeEntity clone = (NodeEntity) super.clone();
        if (availableSolvers != null) {
            clone.availableSolvers = new ArrayList<>(availableSolvers);
        }
        return clone;
    }

    public int getNumOfCores() {
        return numOfCores;
    }

    public void setNumOfCores(int numOfCores) {
        this.numOfCores = numOfCores;
    }

    public long getMemory() {
        return memory;
    }

    public void setMemory(long memory) {
        this.memory = memory;
    }

    public String getUrl() {
        return url;
    }

    public void setUrl(String url) {
        this.url = url;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public boolean getIsAlive() {
        return isAlive;
    }

    public void setIsAlive(boolean isAlive) {
        this.isAlive = isAlive;
    }

    public boolean getIsEnabled() {
        return isEnabled;
    }

    public void setIsEnabled(boolean enabled) {
        isEnabled = enabled;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getOwner() {
        return owner;
    }

    public void setOwner(String owner) {
        this.owner = owner;
    }

    public List<String> getAvailableSolvers() {
        return availableSolvers;
    }

    public void setAvailableSolvers(List<String> availableSolvers) {
        if (availableSolvers == null) {
            this.availableSolvers = emptyList();
        } else {
            this.availableSolvers = availableSolvers;
        }
    }

    public int getClusterId() {
        return clusterId;
    }

    public void setClusterId(int clusterId) {
        this.clusterId = clusterId;
    }
}
