package ru.crp.lvms.model.entities.simulations;

import ru.crp.cmlb.model.entities.CalculationStatus;

import javax.annotation.Nonnull;
import java.util.EnumSet;
import java.util.Set;

public enum SolvingStatus {
    Created(1),
    Processing(2),
    Failed(3),
    Completed(4),
    Queued(5);

    private static final Set<SolvingStatus> RUNNABLE_STATUSES = EnumSet.of(Created, Failed, Completed);

    private int id;
    private String displayName = "";

    SolvingStatus(int id) {
        this.id = id;
    }

    public String getDisplayName() {
        return displayName;
    }

    public void setDisplayName(String displayName) {
        this.displayName = displayName;
    }

    public int getId() {
        return id;
    }

    @Nonnull
    public static SolvingStatus valueOf(int id) {
        for (SolvingStatus status : values()) {
            if (status.getId() == id) {
                return status;
            }
        }
        throw new IllegalArgumentException("Enum value for id " + id + " is not found");
    }

    public static int fromCalculationStatus(CalculationStatus calculationStatus) {
        switch (calculationStatus) {
            case Draft:
                return Created.getId();
            case Waiting:
                return Queued.getId();
            case Sending:
            case Starting:
            case Solving:
            case CalculationFinished:
            case ImportingResults:
            case Terminating:
                return Processing.getId();
            case Terminated:
            case Canceled:
                return Created.getId();
            case Failed:
                return Failed.getId();
            case Finished:
                return Completed.getId();
            default:
                throw new IllegalArgumentException("Unexpected enum value: " + calculationStatus);
        }
    }

    public boolean isRunnable() {
        return RUNNABLE_STATUSES.contains(this);
    }
}
