package ru.crp.lvms.model.mappers;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.BeanWrapper;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import ru.crp.cmlb.model.editors.CollectionAsStringPropertyEditor;
import ru.crp.lvms.model.entities.misc.ItemType;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collector;

public class CustomBeanPropertyRowMapper<T> extends BeanPropertyRowMapper {

    private ObjectMapper jacksonMapper;

    public CustomBeanPropertyRowMapper(ObjectMapper jacksonMapper, Class<T> beanClass) {
        super(beanClass);
        this.jacksonMapper = jacksonMapper;
    }

    @Override
    protected void initBeanWrapper(BeanWrapper bw) {
        bw.registerCustomEditor(ArrayList.class, "path", new PathPropertyEditor(jacksonMapper));

        bw.registerCustomEditor(List.class, "availableSolvers", new PgArrayPropertyEditor());

        bw.registerCustomEditor(ItemType[].class, new CollectionAsStringPropertyEditor<>(
                        ItemType::valueOf,
                        arrayCollector(new ItemType[]{}))
        );
    }

    private <E> Collector<E, ArrayList<E>, E[]> arrayCollector(E[] type) {
        return Collector.<E, ArrayList<E>, E[]>of(
                ArrayList<E>::new,
                List<E>::add,
                (left, right) -> {
                    left.addAll(right);
                    return left;
                },
                list -> list.toArray(type));
    }
}
