package ru.crp.lvms.model.mappers;

import org.postgresql.jdbc4.Jdbc4Array;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nullable;
import java.beans.PropertyEditorSupport;
import java.sql.SQLException;
import java.util.ArrayList;

import static java.util.Arrays.asList;

public class PgArrayPropertyEditor extends PropertyEditorSupport {
    public static final Logger LOGGER = LoggerFactory.getLogger(PgArrayPropertyEditor.class);


    @Override
    public void setValue(@Nullable Object value) {
        if (value == null) {
            super.setValue(value);
            return;
        }
        if (!(value instanceof Jdbc4Array)) {
            LOGGER.error("Unexpected array class: " + value.getClass());
            super.setValue(null);
            return;
        }
        Jdbc4Array pgArray = (Jdbc4Array) value;
        try {
            String[] array = (String[]) pgArray.getArray();
            super.setValue(new ArrayList<>(asList(array)));
        } catch (SQLException ex) {
            LOGGER.error("Exception while converting pg array to list", ex);
            super.setValue(null);
        }
    }
}
