package ru.crp.lvms.optimizations.base.dao;

import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import ru.crp.lvms.model.dao.BaseSpaceDao;
import ru.crp.lvms.optimizations.base.entities.OptimizationTypeSolverCombination;

import javax.annotation.Nullable;
import java.util.List;
import java.util.Objects;
import java.util.stream.Stream;

import static java.util.stream.Collectors.joining;

@Repository
public class OptimizationTypeSolverCombinationDao extends BaseSpaceDao<OptimizationTypeSolverCombination> {

    public List<OptimizationTypeSolverCombination> findAvailableCombinations() {
        return find("WHERE available = ?", true);
    }

    public List<OptimizationTypeSolverCombination> findAvailableCombinationsForOptimizationType(Integer optimizationTypeId) {
        return find("WHERE optimization_type_id = ? AND available = ?", optimizationTypeId, true);
    }

    public List<OptimizationTypeSolverCombination> findAvailableForCao(Integer caoId) {
        return find("WHERE cao_id = ? AND available = ?", caoId, true);
    }


    @Transactional(propagation = Propagation.REQUIRED, readOnly = true)
    public boolean isCombinationAvailable(@Nullable Integer optimizationTypeId, @Nullable Integer solverCaoId, @Nullable Integer solverCaeId, @Nullable Integer solverCadId) {
        Object[] args = Stream.of(optimizationTypeId, solverCaoId, solverCaeId, solverCadId)
                .filter(Objects::nonNull)
                .toArray();

        String condition = Stream.of(createEqualsCondition(OptimizationTypeSolverCombination.OPTIMIZATION_TYPE_ID_COLUMN_NAME, optimizationTypeId),
                createEqualsCondition(OptimizationTypeSolverCombination.CAO_ID_COLUMN_NAME, solverCaoId),
                createEqualsCondition(OptimizationTypeSolverCombination.CAE_ID_COLUMN_NAME, solverCaeId),
                createEqualsCondition(OptimizationTypeSolverCombination.CAD_ID_COLUMN_NAME, solverCadId))
                .collect(joining(" AND ", " AND ", ")"));

        return getJdbcTemplate().queryForObject("SELECT EXISTS " +
                "(SELECT id FROM optimization_type_solver_combinations " +
                "WHERE " + OptimizationTypeSolverCombination.AVAILABLE_COLUMN_NAME + " = TRUE " + condition, Boolean.class, args);
    }

    private String createEqualsCondition(String columnName, @Nullable Integer value) {
        if (value == null) {
            return String.format("%s IS NULL", columnName);
        } else {
            return String.format("%s = ?", columnName);
        }
    }
}
