package ru.crp.lvms.optimizations.base.entities;

import com.fasterxml.jackson.annotation.JsonIgnore;
import ru.crp.cmlb.model.entities.Column;
import ru.crp.cmlb.model.entities.Table;
import ru.crp.cmls.calculations.solvers.xml.config.ResultType;
import ru.crp.lvms.managers.image.ImageFileContainer;
import ru.crp.lvms.model.entities.FileEntity;
import ru.crp.lvms.model.entities.ResultEntity;
import ru.crp.lvms.model.entities.misc.CreatedBy;
import ru.crp.lvms.model.entities.misc.ItemType;
import ru.crp.lvms.model.entities.misc.PathEntry;
import ru.crp.lvms.model.entities.simulations.SolvingStatus;
import ru.crp.lvms.model.entities.tasks.TaskParent;
import ru.crp.lvms.validators.SolversCombinationAvailable;
import ru.crp.lvms.web.dto.columns.GridColumn;
import ru.crp.lvms.web.dto.columns.GridColumnType;

import java.util.List;
import java.util.Map;

@Table("optimizations_join")
@SolversCombinationAvailable
public class OptimizationEntity extends OptimizationEntityTable implements ImageFileContainer, TaskParent, Cloneable {
    @GridColumn(name = "optimizationType", label = "Тип оптимизации", order = 1)
    @Column(value = "optimization_type", onlySelect = true)
    private String optimizationType;

    @GridColumn(name = "name", label = "Базовая", order = 2, cell = GridColumnType.uri)
    @Column(value = "reference", onlySelect = true)
    private String reference;

    @GridColumn(name = "solvingStatus", label = "Статус расчета", order = 3)
    @Column(value = "status", onlySelect = true)
    private String solvingStatus;

    private String postprocessingStatus;

    @GridColumn(name = "owner", label = "Автор", cell = GridColumnType.uri, order = 102)
    @Column(value = "owner", onlySelect = true)
    private String owner;

    @Column(value = "cao_name", onlySelect = true)
    private String caoName;

    @Column(value = "cae_name", onlySelect = true)
    private String caeName;

    @Column(value = "cad_name", onlySelect = true)
    private String cadName;

    private Map<ResultType, List<ResultEntity>> results;
    private Map<CreatedBy, List<OptimizationReportEntity>> reports;
    private List<FileEntity> files;

    @Override
    public void setSolvingStatusId(Integer solvingStatusId) {
        super.setSolvingStatusId(solvingStatusId);
        if (solvingStatusId == null) {
            this.solvingStatus = null;
        } else {
            this.solvingStatus = SolvingStatus.valueOf(solvingStatusId).getDisplayName();
        }
    }

    public String getOwner() {
        return owner;
    }

    public void setOwner(String owner) {
        this.owner = owner;
    }

    public String getSolvingStatus() {
        return solvingStatus;
    }

    public String getPostprocessingStatus() {
        return postprocessingStatus;
    }

    public String getOptimizationType() {
        return optimizationType;
    }

    public void setOptimizationType(String optimizationType) {
        this.optimizationType = optimizationType;
    }

    public String getReference() {
        return reference;
    }

    public void setReference(String reference) {
        this.reference = reference;
    }

    public Map<ResultType, List<ResultEntity>> getResults() {
        return results;
    }

    public void setResults(Map<ResultType, List<ResultEntity>> results) {
        this.results = results;
    }

    public Map<CreatedBy, List<OptimizationReportEntity>> getReports() {
        return reports;
    }

    public void setReports(Map<CreatedBy, List<OptimizationReportEntity>> reports) {
        this.reports = reports;
    }

    public List<FileEntity> getFiles() {
        return files;
    }

    public void setFiles(List<FileEntity> files) {
        this.files = files;
    }

    public String getCaoName() {
        return caoName;
    }

    public void setCaoName(String caoName) {
        this.caoName = caoName;
    }

    public String getCaeName() {
        return caeName;
    }

    public void setCaeName(String caeName) {
        this.caeName = caeName;
    }

    @JsonIgnore
    @Override
    public SolvingStatus getStatus() {
        return SolvingStatus.valueOf(getSolvingStatusId());
    }

    @JsonIgnore
    public Integer getParentCEId() {
        if (getPath() == null) {
            return null;
        }
        return getPath().stream()
                .filter(pathEntry -> pathEntry.getType() == ItemType.ConstructionElement)
                .map(PathEntry::getId)
                .findFirst().orElse(null);
    }

    public String getCadName() {
        return cadName;
    }

    public void setCadName(String cadName) {
        this.cadName = cadName;
    }

    @Override
    public OptimizationEntity clone() throws CloneNotSupportedException {
        return (OptimizationEntity) super.clone();
    }
}
