package ru.crp.lvms.optimizations.base.managers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.crp.lvms.managers.misc.BaseManagerSuggest;
import ru.crp.lvms.model.dao.BaseSpaceDao;
import ru.crp.lvms.optimizations.base.dao.OptimizationTypeDao;
import ru.crp.lvms.optimizations.base.dao.OptimizationTypeSolverCombinationDao;
import ru.crp.lvms.optimizations.base.entities.*;
import ru.crp.lvms.security.PortalUser;

import java.util.List;
import java.util.Map;
import java.util.function.BiFunction;
import java.util.function.Function;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.toList;

@Service
public class OptimizationTypeManager extends BaseManagerSuggest<OptimizationType> {

    @Autowired
    private OptimizationTypeDao optimizationTypeDao;

    @Autowired
    private OptimizationTypeSolverCombinationDao availableSolversDao;

    @Override
    protected BaseSpaceDao<OptimizationType> aliveDao() {
        return optimizationTypeDao;
    }

    private interface AvailableCombinationsMapper
            extends BiFunction<Integer, List<OptimizationTypeSolverCombination>, SolverWithCompatibles> { }

    public List<SolverWithCompatibles> getCaes(Integer optimizationTypeId, PortalUser user) {
        return getSolvers(optimizationTypeId,
                OptimizationTypeSolverCombination::getCaeId,
                CaeWithCompatibles::new);
    }

    public List<SolverWithCompatibles> getCaos(Integer optimizationTypeId, PortalUser user) {
        return getSolvers(optimizationTypeId,
                OptimizationTypeSolverCombination::getCaoId,
                CaoWithCompatibles::new);
    }

    public List<SolverWithCompatibles> getCads(Integer optimizationTypeId, PortalUser user) {
        return getSolvers(optimizationTypeId,
                OptimizationTypeSolverCombination::getCadId,
                CadWithCompatibles::new);
    }

    private List<SolverWithCompatibles> getSolvers(Integer optimizationTypeId,
                                    Function<OptimizationTypeSolverCombination, Integer> idExtractor,
                                    AvailableCombinationsMapper creator) {
        List<OptimizationTypeSolverCombination> combinations =
                availableSolversDao.findAvailableCombinationsForOptimizationType(optimizationTypeId);

        Map<Integer, List<OptimizationTypeSolverCombination>> collect = combinations.stream()
                .filter(otsc -> idExtractor.apply(otsc) != null)
                .collect(Collectors.groupingBy(idExtractor));

        return collect.entrySet().stream()
                .map(entry -> creator.apply(entry.getKey(), entry.getValue()))
                .collect(toList());
    }
}
