package ru.crp.lvms.optimizations.optistruct.dao;

import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;
import ru.crp.lvms.model.dao.BaseSpaceDao;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructCollector;
import ru.crp.lvms.optimizations.optistruct.enums.OptistructCollectorType;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;

@Repository
@Transactional
public class OptistructCollectorDao extends BaseSpaceDao<OptistructCollector> {

    public List<OptistructCollector> findAllForOptimizationByType(Integer optimizationId, @Nullable OptistructCollectorType type) {
        if (type == null) {
            return find("WHERE optimization_id = ?", optimizationId);
        } else {
            return find("WHERE optimization_id = ? AND data_type = ?", optimizationId, type.name());
        }
    }

    public void deleteAllForOptimization(Integer optimizationId) {
        jdbcTemplate().update("DELETE FROM " + getTableName() + " WHERE optimization_id = ?", optimizationId);
    }

    public void addCollectorsForOptimization(Integer optimizationId, List<OptistructCollector> newSteps) {
        List<Object[]> paramsInsert = new ArrayList<>();
        for (OptistructCollector step : newSteps) {
            paramsInsert.add(new Object[]{
                    optimizationId,
                    step.getScriptId(),
                    step.getDataType(),
                    step.getName()
            });
        }
        if (!paramsInsert.isEmpty()) {
            getJdbcTemplate().batchUpdate(
                    "INSERT INTO " + getTableName() +
                            "(optimization_id," +
                            " script_id," +
                            " data_type," +
                            " name) " +
                            "VALUES (?, ?, ?, ?)",
                    paramsInsert);
        }
    }
}
