package ru.crp.lvms.optimizations.optistruct.managers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import ru.crp.lvms.managers.misc.BaseManager;
import ru.crp.lvms.model.entities.UserEntity;
import ru.crp.lvms.optimizations.base.entities.tree.DependenciesDescription;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNode;
import ru.crp.lvms.optimizations.optistruct.dao.OptistructOptimizationTechConstraintDao;
import ru.crp.lvms.optimizations.optistruct.dao.OptistructTechConstraintDao;
import ru.crp.lvms.optimizations.optistruct.dao.OptistructTechnologicalConstraintTreeDao;
import ru.crp.lvms.optimizations.optistruct.dao.OptistructTechnologicalConstraintTreeValueDao;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructOptimizationTechnologicalConstraint;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.PostConstruct;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import static com.google.common.primitives.Ints.asList;
import static freemarker.template.utility.Collections12.singletonList;
import static ru.crp.lvms.optimizations.base.entities.tree.ConditionType.ANY_SELECTED;
import static ru.crp.lvms.optimizations.base.entities.tree.ConditionType.NONE_SELECTED;

@Service
public class OptistructOptimizationTechnologicalConstraintManager extends BaseManager<OptistructOptimizationTechnologicalConstraint> {

    private Map<Integer, DependenciesDescription> dependenciesConfig = new HashMap<>();

    @Autowired
    private OptistructOptimizationTechConstraintDao optistructOptimizationTechConstraintDao;

    @Autowired
    private OptistructTechnologicalConstraintTreeValueDao optistructTechnologicalConstraintTreeValueDao;

    @Autowired
    private OptistructTechConstraintDao optistructTechConstraintDao;

    @Autowired
    private OptistructTechnologicalConstraintTreeDao optistructTechnologicalConstraintTreeDao;

    @PostConstruct
    public void init() {
        // TC_ONE_PLANE_COORDS
        dependenciesConfig.put(706, new DependenciesDescription(asList(703, 704, 705), NONE_SELECTED));
        dependenciesConfig.put(703, new DependenciesDescription(asList(706), NONE_SELECTED));
        dependenciesConfig.put(704, new DependenciesDescription(asList(706), NONE_SELECTED));
        dependenciesConfig.put(705, new DependenciesDescription(asList(706), NONE_SELECTED));

        // TC_ONE_PLANE_NORMAL
        dependenciesConfig.put(711, new DependenciesDescription(asList(708, 709, 710), NONE_SELECTED));
        dependenciesConfig.put(708, new DependenciesDescription(asList(711), NONE_SELECTED));
        dependenciesConfig.put(709, new DependenciesDescription(asList(711), NONE_SELECTED));
        dependenciesConfig.put(710, new DependenciesDescription(asList(711), NONE_SELECTED));

        // TC_TWO_PLANES_COORDS
        dependenciesConfig.put(717, new DependenciesDescription(asList(714, 715, 716), NONE_SELECTED));
        dependenciesConfig.put(714, new DependenciesDescription(asList(717), NONE_SELECTED));
        dependenciesConfig.put(715, new DependenciesDescription(asList(717), NONE_SELECTED));
        dependenciesConfig.put(716, new DependenciesDescription(asList(717), NONE_SELECTED));

        // TC_TWO_PLANES_COORDS_NORMAL_1
        dependenciesConfig.put(722, new DependenciesDescription(asList(719, 720, 721), NONE_SELECTED));
        dependenciesConfig.put(719, new DependenciesDescription(asList(722), NONE_SELECTED));
        dependenciesConfig.put(720, new DependenciesDescription(asList(722), NONE_SELECTED));
        dependenciesConfig.put(721, new DependenciesDescription(asList(722), NONE_SELECTED));

        // TC_TWO_PLANES_COORDS_NORMAL_2
        dependenciesConfig.put(727, new DependenciesDescription(asList(724, 725, 726), NONE_SELECTED));
        dependenciesConfig.put(724, new DependenciesDescription(asList(727), NONE_SELECTED));
        dependenciesConfig.put(725, new DependenciesDescription(asList(727), NONE_SELECTED));
        dependenciesConfig.put(726, new DependenciesDescription(asList(727), NONE_SELECTED));

        // TC_THREE_PLANES_COORDS
        dependenciesConfig.put(733, new DependenciesDescription(asList(730, 731, 732), NONE_SELECTED));
        dependenciesConfig.put(730, new DependenciesDescription(asList(733), NONE_SELECTED));
        dependenciesConfig.put(731, new DependenciesDescription(asList(733), NONE_SELECTED));
        dependenciesConfig.put(732, new DependenciesDescription(asList(733), NONE_SELECTED));

        // TC_THREE_PLANES_COORDS_NORMAL_1
        dependenciesConfig.put(738, new DependenciesDescription(asList(735, 736, 737), NONE_SELECTED));
        dependenciesConfig.put(735, new DependenciesDescription(asList(738), NONE_SELECTED));
        dependenciesConfig.put(736, new DependenciesDescription(asList(738), NONE_SELECTED));
        dependenciesConfig.put(737, new DependenciesDescription(asList(738), NONE_SELECTED));

        // TC_THREE_PLANES_COORDS_NORMAL_3
        dependenciesConfig.put(743, new DependenciesDescription(asList(740, 741, 742), NONE_SELECTED));
        dependenciesConfig.put(740, new DependenciesDescription(asList(743), NONE_SELECTED));
        dependenciesConfig.put(741, new DependenciesDescription(asList(743), NONE_SELECTED));
        dependenciesConfig.put(742, new DependenciesDescription(asList(743), NONE_SELECTED));

        // CYCLIC_COORDS
        dependenciesConfig.put(749, new DependenciesDescription(asList(746, 747, 748), NONE_SELECTED));
        dependenciesConfig.put(746, new DependenciesDescription(asList(749), NONE_SELECTED));
        dependenciesConfig.put(747, new DependenciesDescription(asList(749), NONE_SELECTED));
        dependenciesConfig.put(748, new DependenciesDescription(asList(749), NONE_SELECTED));

        // CYCLIC_COORDS_DIR
        dependenciesConfig.put(754, new DependenciesDescription(asList(751, 752, 753), NONE_SELECTED));
        dependenciesConfig.put(751, new DependenciesDescription(asList(754), NONE_SELECTED));
        dependenciesConfig.put(752, new DependenciesDescription(asList(754), NONE_SELECTED));
        dependenciesConfig.put(753, new DependenciesDescription(asList(754), NONE_SELECTED));

        // CYCLIC_AND_ONE_PLANE_COORDS
        dependenciesConfig.put(760, new DependenciesDescription(asList(757, 758, 759), NONE_SELECTED));
        dependenciesConfig.put(757, new DependenciesDescription(asList(760), NONE_SELECTED));
        dependenciesConfig.put(758, new DependenciesDescription(asList(760), NONE_SELECTED));
        dependenciesConfig.put(759, new DependenciesDescription(asList(760), NONE_SELECTED));

        // CYCLIC_AND_ONE_PLANE_COORDS_DIR
        dependenciesConfig.put(765, new DependenciesDescription(asList(762, 763, 764), NONE_SELECTED));
        dependenciesConfig.put(762, new DependenciesDescription(asList(765), NONE_SELECTED));
        dependenciesConfig.put(763, new DependenciesDescription(asList(765), NONE_SELECTED));
        dependenciesConfig.put(764, new DependenciesDescription(asList(765), NONE_SELECTED));

        // CYCLIC_AND_ONE_PLANE_COORDS_NORMAL
        dependenciesConfig.put(771, new DependenciesDescription(asList(768, 769, 770), NONE_SELECTED));
        dependenciesConfig.put(768, new DependenciesDescription(asList(771), NONE_SELECTED));
        dependenciesConfig.put(769, new DependenciesDescription(asList(771), NONE_SELECTED));
        dependenciesConfig.put(770, new DependenciesDescription(asList(771), NONE_SELECTED));

        // TC_CAST_SOLID_COORDS_1
        dependenciesConfig.put(806, new DependenciesDescription(asList(803, 804, 805), NONE_SELECTED));
        dependenciesConfig.put(803, new DependenciesDescription(asList(806), NONE_SELECTED));
        dependenciesConfig.put(804, new DependenciesDescription(asList(806), NONE_SELECTED));
        dependenciesConfig.put(805, new DependenciesDescription(asList(806), NONE_SELECTED));

        // TC_CAST_SOLID_COORDS_2
        dependenciesConfig.put(832, new DependenciesDescription(asList(829, 830, 831), NONE_SELECTED));
        dependenciesConfig.put(829, new DependenciesDescription(asList(832), NONE_SELECTED));
        dependenciesConfig.put(830, new DependenciesDescription(asList(832), NONE_SELECTED));
        dependenciesConfig.put(831, new DependenciesDescription(asList(832), NONE_SELECTED));

        // TC_CAST_SECTIONAL_COORDS
        dependenciesConfig.put(819, new DependenciesDescription(asList(816, 817, 818), NONE_SELECTED));
        dependenciesConfig.put(816, new DependenciesDescription(asList(819), NONE_SELECTED));
        dependenciesConfig.put(817, new DependenciesDescription(asList(819), NONE_SELECTED));
        dependenciesConfig.put(818, new DependenciesDescription(asList(819), NONE_SELECTED));

        // TC_CAST_SECTIONAL_COORDS_2
        dependenciesConfig.put(824, new DependenciesDescription(asList(821, 822, 823), NONE_SELECTED));
        dependenciesConfig.put(821, new DependenciesDescription(asList(824), NONE_SELECTED));
        dependenciesConfig.put(822, new DependenciesDescription(asList(824), NONE_SELECTED));
        dependenciesConfig.put(823, new DependenciesDescription(asList(824), NONE_SELECTED));

        // TC_GRID_ACTIVE -> TC_GRID_DIR
        dependenciesConfig.put(603, new DependenciesDescription(asList(600), ANY_SELECTED));

        // TC_CAST_DESIRED_THICKNESS -> TC_CAST_DESIRED_THICKNESS_VAL
        dependenciesConfig.put(813, new DependenciesDescription(asList(810), ANY_SELECTED));
    }

    @Override
    protected OptistructOptimizationTechConstraintDao aliveDao() {
        return optistructOptimizationTechConstraintDao;
    }

    public List<OptistructOptimizationTechnologicalConstraint> findOptimizationTCs(int optimizationId) {
        return optistructOptimizationTechConstraintDao.findForOptimization(optimizationId);
    }

    public List<TechnologicalConstraintTreeNode> getFullTreeOfTC(Integer constraintId) {
        return setDependencies(optistructTechnologicalConstraintTreeDao.getFullTreeForTC(constraintId));
    }


    public List<TechnologicalConstraintTreeNode> getTCTree(Integer constraintId, @Nullable Integer parentId) {
        return setDependencies(optistructTechnologicalConstraintTreeDao.getTCTree(constraintId, parentId));
    }

    @Nonnull
    @Override
    public OptistructOptimizationTechnologicalConstraint updateAndRead(OptistructOptimizationTechnologicalConstraint entity, @Nullable UserEntity currentUser) {
        OptistructOptimizationTechnologicalConstraint current = findOne(entity.getId(), currentUser);
        OptistructOptimizationTechnologicalConstraint updated = super.updateAndRead(entity, currentUser);
        if (!Objects.equals(current.getTechnologicalConstraintId(), updated.getTechnologicalConstraintId())) {
            optistructTechnologicalConstraintTreeValueDao.clearSavedValues(updated.getId());
        }
        return updated;
    }

    public TechnologicalConstraintTreeNode getField(Integer fieldId) {
        TechnologicalConstraintTreeNode field = optistructTechnologicalConstraintTreeDao.findOne(fieldId);
        if (field != null) {
            setDependencies(singletonList(field));
        }
        return field;
    }

    public <T extends TechnologicalConstraintTreeNode> List<T> setDependencies(List<T> treeNodes) {
        treeNodes.forEach(node -> node.setDependencies(dependenciesConfig.getOrDefault(node.getId(), null)));
        return treeNodes;
    }
}
