package ru.crp.lvms.optimizations.optistruct.tcl.tc;

import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNodeValue;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructOptimizationTechnologicalConstraint;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructSet;

import java.util.List;
import java.util.stream.Stream;

import static java.util.stream.Collectors.joining;

abstract public class OptistructTCConverter {

    abstract public Stream<String> convert(OptistructOptimizationTechnologicalConstraint currentConstraint,
                                           List<TechnologicalConstraintTreeNodeValue> currentConstraintValues,
                                           List<OptistructOptimizationTechnologicalConstraint> allConstraints,
                                           String setName,
                                           List<String> alreadyConvertedConstraints);

    abstract public boolean canConvert(OptistructOptimizationTechnologicalConstraint constraint);

    protected String extractValue(String commandName, String defaultIfNull, String defaultIfNotFound, List<TechnologicalConstraintTreeNodeValue> currentConstraintValues) {
        return currentConstraintValues.stream()
                .filter(c -> c.getName().equals(commandName))
                .map(TechnologicalConstraintTreeNodeValue::getValue)
                .map(v -> v == null ? defaultIfNull : v)
                .findAny().orElse(defaultIfNotFound);
    }

    protected String generateCreateCoordsArrayCommand(List<String> prefixes, List<TechnologicalConstraintTreeNodeValue> currentConstraintValues) {
        int arraySize = prefixes.size() * 3;
        return prefixes.stream()
                .flatMap(prefix -> Stream.of(
                        extractValue(prefix + "_X", "0", "0", currentConstraintValues),
                        extractValue(prefix + "_Y", "0", "0", currentConstraintValues),
                        extractValue(prefix + "_Z", "0", "0", currentConstraintValues)))
                .collect(joining(" ", "*createdoublearray " + arraySize + " ", ""));
    }

    protected String extractValueForCheckbox(String firstCommand, String firstValue, String secondCommand, String secondValue,
                                             String defaultValue, List<TechnologicalConstraintTreeNodeValue> constraintValues) {
        if (constraintValues.stream()
                .filter(c -> c.getName().equals(firstCommand))
                .findAny().isPresent()) {
            return firstValue;
        }
        if (constraintValues.stream()
                .filter(c -> c.getName().equals(secondCommand))
                .findAny().isPresent()) {
            return secondValue;
        }
        return defaultValue;
    }

    protected String extractValueForCheckbox(String commandName, String valueIfPresent, String valueIfAbsent,
                                           List<TechnologicalConstraintTreeNodeValue> allUsedValues) {
        return allUsedValues.stream()
                .filter(v -> v.getName().equals(commandName))
                .findAny().map(v -> valueIfPresent).orElse(valueIfAbsent);
    }

}
