package ru.crp.lvms.optimizations.optistruct.tcl.tc;

import ru.crp.lvms.exceptions.InternalException;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNode;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNodeValue;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructOptimizationTechnologicalConstraint;
import ru.crp.lvms.optimizations.optistruct.entities.OptistructSet;

import java.util.List;
import java.util.stream.Stream;

import static java.util.Arrays.asList;
import static org.apache.batik.svggen.SVGStylingAttributes.set;

public class SymmetryConditionTCConverter extends OptistructTCConverter {
    public static final String SYM_TYPE_PREFIX = "TC_SYM_TYPE_";

    public static final String FIRST_NODE_ID_COMMAND = "TC_SYMMETRY_CONDITION_ID_1";
    public static final String SECOND_NODE_ID_COMMAND = "TC_SYMMETRY_CONDITION_ID_2";
    public static final String THIRD_NODE_ID_COMMAND = "TC_SYMMETRY_CONDITION_ID_3";

    public static final String NUM_OF_LOOPS_COMMAND = "TC_SYMMETRY_CONDITION_NUM_OF_LOOPS";

    public static final String FIRST_NODE_COORDS_PREFIX = "TC_SYMMETRY_CONDITION_COORDS_1";
    public static final String SECOND_NODE_COORDS_PREFIX = "TC_SYMMETRY_CONDITION_COORDS_2";
    public static final String THIRD_NODE_COORDS_PREFIX = "TC_SYMMETRY_CONDITION_COORDS_3";

    public static final int CONDITION_SYMMETRY_TC_ID = 7;

    @Override
    public Stream<String> convert(OptistructOptimizationTechnologicalConstraint currentConstraint,
                                  List<TechnologicalConstraintTreeNodeValue> currentConstraintValues,
                                  List<OptistructOptimizationTechnologicalConstraint> allConstraints,
                                  String setName, List<String> alreadyConvertedConstraints) {
        String type = extractType(currentConstraintValues);

        String firstNodeId = extractValue(FIRST_NODE_ID_COMMAND, "-1", "0", currentConstraintValues);
        String secondNodeId = extractValue(SECOND_NODE_ID_COMMAND, "-1", "0", currentConstraintValues);
        String thirdNodeId = extractValue(THIRD_NODE_ID_COMMAND, "-1", "0", currentConstraintValues);

        String numOfLoops = extractValue(NUM_OF_LOOPS_COMMAND, "-1", "0", currentConstraintValues);

        String createCoordsArrayCommand = generateCreateCoordsArrayCommand(asList(FIRST_NODE_COORDS_PREFIX, SECOND_NODE_COORDS_PREFIX, THIRD_NODE_COORDS_PREFIX), currentConstraintValues);

        String tcCommand = String.format("*topologypatternupdate \"%s\" %s %s %s %s %s 1 9",
                setName, type, firstNodeId, secondNodeId, thirdNodeId, numOfLoops);
        return Stream.of(createCoordsArrayCommand, tcCommand);
    }


    private String extractType(List<TechnologicalConstraintTreeNodeValue> currentConstraintValues) {
        String type = currentConstraintValues.stream().filter(v -> v.getName().startsWith(SYM_TYPE_PREFIX))
                .map(TechnologicalConstraintTreeNode::getName)
                .findAny().orElseThrow(() -> new InternalException("Not enough arguments"));
        switch (type) {
            case "TC_SYM_TYPE_ONE_PLANE":
                return "1";
            case "TC_SYM_TYPE_TWO_PLANES":
                return "2";
            case "TC_SYM_TYPE_THREE_PLANES":
                return "3";
            case "TC_SYM_TYPE_CYCLIC":
                return "10";
            case "TC_SYM_TYPE_CYCLIC_AND_ONE_PLANE":
                return "11";
            default:
                throw new InternalException("Unexpected argument");
        }
    }

    @Override
    public boolean canConvert(OptistructOptimizationTechnologicalConstraint constraint) {
        return constraint.getTechnologicalConstraintId().equals(CONDITION_SYMMETRY_TC_ID);
    }
}
