package ru.crp.lvms.optimizations.tosca.config;

import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import ru.crp.lvms.optimizations.base.entities.OptimizationEntity;
import ru.crp.lvms.optimizations.misc.AbstractOptimizationSettings;
import ru.crp.lvms.optimizations.tosca.config.commandConverters.*;
import ru.crp.lvms.optimizations.tosca.entities.ToscaOptimizationBaseSetting;
import ru.crp.lvms.optimizations.tosca.entities.ToscaOptimizationTechnologicalConstraint;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNodeValue;
import ru.crp.lvms.optimizations.tosca.enums.ToscaBaseSettingType;
import ru.crp.lvms.optimizations.tosca.enums.ToscaSettingsType;
import ru.crp.lvms.optimizations.tosca.managers.ToscaOptimizationSettingsManager;
import ru.crp.lvms.optimizations.tosca.managers.ToscaOptimizationTechnologicalConstraintManager;
import ru.crp.lvms.optimizations.tosca.managers.ToscaTechnologicalConstraintTreeValueManager;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;

import static java.util.stream.Collectors.toList;
import static ru.crp.lvms.optimizations.tosca.config.commandConverters.CommandConverter.DVCON_ID_PREFIX;

@Component
public class ToscaTechnologicalConstraintsGenerator {
    private ToscaOptimizationSettingsManager toscaOptimizationSettingsManager;

    private ToscaOptimizationTechnologicalConstraintManager toscaOptimizationTechnologicalConstraintManager;

    private ToscaTechnologicalConstraintTreeValueManager toscaTechnologicalConstraintTreeValueManager;

    private List<CommandConverter> commandConverters;

    @Autowired
    public ToscaTechnologicalConstraintsGenerator(ToscaOptimizationSettingsManager toscaOptimizationSettingsManager,
                                                  ToscaOptimizationTechnologicalConstraintManager toscaOptimizationTechnologicalConstraintManager,
                                                  ToscaTechnologicalConstraintTreeValueManager toscaTechnologicalConstraintTreeValueManager) {
        this.toscaOptimizationSettingsManager = toscaOptimizationSettingsManager;
        this.toscaOptimizationTechnologicalConstraintManager = toscaOptimizationTechnologicalConstraintManager;
        this.toscaTechnologicalConstraintTreeValueManager = toscaTechnologicalConstraintTreeValueManager;
    }

    @PostConstruct
    public void init() {
        commandConverters = new ArrayList<>();
        commandConverters.add(new SymmetryConditionCommandConverter(toscaOptimizationTechnologicalConstraintManager));
        commandConverters.add(new TopographicCommandConverter(toscaOptimizationTechnologicalConstraintManager));
        commandConverters.add(new ShapeCommandConverter(toscaOptimizationTechnologicalConstraintManager));
        // should always be last
        commandConverters.add(new DefaultCommandConverter(toscaOptimizationTechnologicalConstraintManager));
    }


    public Stream<Command> formCommandsForTC(OptimizationTypeParameters typeParameters, ToscaOptimizationTechnologicalConstraint tc, Integer index, String space) {
        List<TechnologicalConstraintTreeNodeValue> valuesOfTC = toscaTechnologicalConstraintTreeValueManager.findValuesOfTC(tc.getId());
        return commandConverters.stream()
                .filter(ac -> ac.canConvert(tc))
                .limit(1)
                .flatMap(converter -> converter.convert(typeParameters, tc, index, space, valuesOfTC));

    }


    public List<Pair<String, String>> formOptimize(OptimizationEntity optimization, Map<ToscaBaseSettingType, ToscaOptimizationBaseSetting> baseSettings) {
        List<? extends AbstractOptimizationSettings> tcSettings = toscaOptimizationSettingsManager
                .findOptimizationSettings(optimization.getId(), ToscaSettingsType.TechnologicalConstraint, null)
                .stream().limit(baseSettings.get(ToscaBaseSettingType.NUM_TECH_CONSTRAINTS).getCurrentValue())
                .collect(Collectors.toList());
        return IntStream.range(0, tcSettings.size())
                .mapToObj(i -> Pair.of("DVCON", DVCON_ID_PREFIX + i)).collect(toList());
    }
}
