package ru.crp.lvms.optimizations.tosca.config.argumentConverters;

import org.apache.commons.lang3.tuple.Pair;
import ru.crp.lvms.optimizations.tosca.entities.ToscaOptimizationTechnologicalConstraint;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNodeValue;
import ru.crp.lvms.optimizations.tosca.managers.ToscaOptimizationTechnologicalConstraintManager;

import java.util.Collections;
import java.util.List;
import java.util.stream.Stream;

import static java.util.Arrays.asList;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class CheckDofArgumentConverter extends CommandArgumentConverter {
    public static final List<Integer> CHECK_DOF_ARGUMENT_IDS = asList(77, 231);
    public static final String CS_NUM_ARGUMENT_NAME = "CS_num";
    public static final String CHECK_DOF_ARGUMENT_NAME = "CHECK_DOF";

    public CheckDofArgumentConverter(ToscaOptimizationTechnologicalConstraintManager toscaOptimizationTechnologicalConstraintManager) {
        super(toscaOptimizationTechnologicalConstraintManager);
    }

    @Override
    public List<Pair<String, String>> convertValue(ToscaOptimizationTechnologicalConstraint constraint, TechnologicalConstraintTreeNodeValue value,
                                                   List<TechnologicalConstraintTreeNodeValue> allValues,
                                                   List<Pair<String, String>> alreadyConvertedValues) {
        // if already handled
        if (alreadyConvertedValues.stream().anyMatch(v -> v.getKey().equals(CHECK_DOF_ARGUMENT_NAME))) {
            return Collections.emptyList();
        }
        List<TechnologicalConstraintTreeNodeValue> checkDOFParams = allValues.stream()
                .filter(v -> CHECK_DOF_ARGUMENT_IDS.contains(v.getParentId()))
                .collect(toList());

        String csNum = checkDOFParams.stream()
                .filter(v -> v.getName().equals(CS_NUM_ARGUMENT_NAME))
                .map(TechnologicalConstraintTreeNodeValue::getValue)
                .findFirst().orElse("CS_0");

        String commandValue = Stream.of(csNum,
                checkDOFParams.stream().filter(v -> v.getName().equals("X")).findFirst().map(v -> "FIX").orElse("FREE"),
                checkDOFParams.stream().filter(v -> v.getName().equals("Y")).findFirst().map(v -> "FIX").orElse("FREE"),
                checkDOFParams.stream().filter(v -> v.getName().equals("Z")).findFirst().map(v -> "FIX").orElse("FREE")
        )
                .collect(joining(","));
        return Collections.singletonList(Pair.of(CHECK_DOF_ARGUMENT_NAME, commandValue));
    }


    @Override
    public boolean canConvert(ToscaOptimizationTechnologicalConstraint constraint, TechnologicalConstraintTreeNodeValue value) {
        return value.getParentId() != null && CHECK_DOF_ARGUMENT_IDS.contains(value.getParentId());
    }
}
