package ru.crp.lvms.optimizations.tosca.config.argumentConverters;

import org.apache.commons.lang3.tuple.Pair;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNodeValue;
import ru.crp.lvms.optimizations.tosca.entities.ToscaOptimizationTechnologicalConstraint;
import ru.crp.lvms.optimizations.tosca.managers.ToscaOptimizationTechnologicalConstraintManager;

import java.util.Collections;
import java.util.List;
import java.util.stream.Stream;

import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class PullDirArgumentConverter extends CommandArgumentConverter {
    public static final int PULL_DIR_ARGUMENT_ID = 6;
    private static final String PULL_DIR_COMMAND_NAME = "PULL_DIR";


    public PullDirArgumentConverter(ToscaOptimizationTechnologicalConstraintManager toscaOptimizationTechnologicalConstraintManager) {
        super(toscaOptimizationTechnologicalConstraintManager);
    }

    @Override
    public List<Pair<String, String>> convertValue(ToscaOptimizationTechnologicalConstraint constraint, TechnologicalConstraintTreeNodeValue value,
                                                   List<TechnologicalConstraintTreeNodeValue> allValues,
                                                   List<Pair<String, String>> alreadyConvertedValues) {
        // if already handled
        if (alreadyConvertedValues.stream().anyMatch(v -> v.getKey().equals(PULL_DIR_COMMAND_NAME))) {
            return Collections.emptyList();
        }
        List<TechnologicalConstraintTreeNodeValue> children = allValues.stream()
                .filter(v -> v.getParentId() != null && v.getParentId().equals(PULL_DIR_ARGUMENT_ID))
                .collect(toList());
        String commandValue = Stream.of(
                children.stream().filter(v -> v.getName().equals("Xdir")).map(TechnologicalConstraintTreeNodeValue::getValue).findFirst().orElse("0"),
                children.stream().filter(v -> v.getName().equals("Ydir")).map(TechnologicalConstraintTreeNodeValue::getValue).findFirst().orElse("0"),
                children.stream().filter(v -> v.getName().equals("Zdir")).map(TechnologicalConstraintTreeNodeValue::getValue).findFirst().orElse("0"))
                .collect(joining(","));
        return Collections.singletonList(Pair.of(PULL_DIR_COMMAND_NAME, commandValue));
    }

    @Override
    public boolean canConvert(ToscaOptimizationTechnologicalConstraint constraint, TechnologicalConstraintTreeNodeValue value) {
        return value.getParentId() != null && value.getParentId().equals(PULL_DIR_ARGUMENT_ID);
    }
}
