package ru.crp.lvms.optimizations.tosca.config.argumentConverters;

import org.apache.commons.lang3.tuple.Pair;
import ru.crp.lvms.exceptions.InternalException;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNodeValue;
import ru.crp.lvms.optimizations.tosca.entities.ToscaOptimizationTechnologicalConstraint;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNode;
import ru.crp.lvms.optimizations.tosca.managers.ToscaOptimizationTechnologicalConstraintManager;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.stream.Stream;

public class SymmetryConditionTypeArgumentConverter extends CommandArgumentConverter {
    public static final int TYPE_TC_PARAM_ID = 30;
    public static final List<Integer> ANCESTOR_IDS = Collections.singletonList(TYPE_TC_PARAM_ID);
    public static final int CS_CHILD_ARGUMENT_ID = 49;

    public SymmetryConditionTypeArgumentConverter(ToscaOptimizationTechnologicalConstraintManager toscaOptimizationTechnologicalConstraintManager) {
        super(toscaOptimizationTechnologicalConstraintManager);
    }

    @Override
    public List<Pair<String, String>> convertValue(ToscaOptimizationTechnologicalConstraint constraint,
                                                   TechnologicalConstraintTreeNodeValue value,
                                                   List<TechnologicalConstraintTreeNodeValue> allValues,
                                                   List<Pair<String, String>> alreadyConvertedValues) {
        List<TechnologicalConstraintTreeNode> ascendingBranch = getAscendingBranch(value);
        Pair<String, String> typeCommand = Pair.of(
                ascendingBranch.get(ascendingBranch.size() - 1).getName(),
                ascendingBranch.get(ascendingBranch.size() - 2).getName()
        );
        List<Pair<String, String>> commands = new ArrayList<>();
        if (alreadyConvertedValues.stream().noneMatch(c -> c.getKey().equals(typeCommand.getKey()))) {
            commands.add(typeCommand);
        }
        // TRANSLATION, ANGLE, CS_CHILD
        if (ascendingBranch.size() == 3) {
            if (value.getTechnologicalConstraintTreeId().equals(CS_CHILD_ARGUMENT_ID) && value.getValue() != null) {
                Stream.of(value.getValue().split(","))
                        .map(String::trim)
                        .map(cs -> Pair.of(value.getName(), cs))
                        .forEach(commands::add);
            } else {
                commands.add(Pair.of(value.getName(), value.getValue()));
            }
        }
        // AUTO,AVG,GROUP,MAX (for MASTER)
        if (ascendingBranch.size() == 4) {
            commands.add(Pair.of(ascendingBranch.get(1).getName(), value.getName()));
        }
        if (ascendingBranch.size() > 4) {
            throw new InternalException("Unexpected branch size");
        }
        return commands;
    }

    @Override
    public boolean canConvert(ToscaOptimizationTechnologicalConstraint constraint, TechnologicalConstraintTreeNodeValue value) {
        return toscaOptimizationTechnologicalConstraintManager.hasAncestors(value.getTechnologicalConstraintTreeId(), ANCESTOR_IDS);
    }
}
