package ru.crp.lvms.optimizations.tosca.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNodeValue;
import ru.crp.lvms.optimizations.tosca.entities.ToscaOptimizationTechnologicalConstraint;
import ru.crp.lvms.optimizations.tosca.enums.ToscaBaseSettingType;
import ru.crp.lvms.optimizations.tosca.enums.ToscaSettingsType;
import ru.crp.lvms.optimizations.tosca.managers.ToscaTechnologicalConstraintTreeValueManager;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.utils.URLs;

import java.util.List;
import java.util.Map;
import java.util.function.Function;

import static java.util.stream.Collectors.toMap;

@RestController
@RequestMapping(value = URLs.TOSCA_TECH_CONSTRAINT_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class ToscaTechnologicalConstraintController extends ToscaAbstractOptimizationSettingsController<ToscaOptimizationTechnologicalConstraint> {

    @Autowired
    private ToscaTechnologicalConstraintTreeValueManager toscaTechnologicalConstraintTreeValueManager;

    @Override
    protected ToscaSettingsType getSettingsType() {
        return ToscaSettingsType.TechnologicalConstraint;
    }

    @Override
    protected ToscaBaseSettingType getToskaOptimizationBaseSettingType() {
        return ToscaBaseSettingType.NUM_TECH_CONSTRAINTS;
    }

    @PreAuthorize("hasRole('ROLE_GUEST')")
    @RequestMapping(value = "/{optimizationTCId}/fields", method = RequestMethod.GET)
    public Map<Integer, TechnologicalConstraintTreeNodeValue> getFieldsOfOptimizationTC(@PathVariable Integer optimizationTCId) {
        return toscaTechnologicalConstraintTreeValueManager.findValuesOfTC(optimizationTCId).stream()
                .collect(toMap(TechnologicalConstraintTreeNodeValue::getTechnologicalConstraintTreeId, Function.identity()));
    }

//    @PreAuthorize("hasRole('ROLE_GUEST')")
//    @RequestMapping(value = "/{optimizationTCId}/fields", method = RequestMethod.POST)
//    public ToscaTechnologicalConstraintTreeValue createFieldOfOptimizationTC(@PathVariable Integer constraintId,
//                                                                             @RequestBody ToscaTechnologicalConstraintTreeValue created,
//                                                                             @CurrentUser PortalUser user) {
//        created.setTechnologicalConstraintId(constraintId);
//        return toscaTechnologicalConstraintTreeValueManager.createAndRead(created, user.getUser());
//    }

    @PreAuthorize("hasRole('ROLE_USER')")
    @RequestMapping(value = "/{optimizationTCId}/fields/{id}", method = RequestMethod.PUT)
    public TechnologicalConstraintTreeNodeValue updateFieldOfOptimizationTC(@PathVariable Integer optimizationTCId,
                                                                            @PathVariable(value = "id") Integer fieldTreeId,
                                                                            @RequestBody TechnologicalConstraintTreeNodeValue updated,
                                                                            @CurrentUser PortalUser user) {
        updated.setTechnologicalConstraintId(optimizationTCId);
        updated.setTechnologicalConstraintTreeId(fieldTreeId);
        TechnologicalConstraintTreeNodeValue existed = toscaTechnologicalConstraintTreeValueManager.finByTreeId(updated.getTechnologicalConstraintId(), updated.getTechnologicalConstraintTreeId());
        if (existed != null) {
            return toscaTechnologicalConstraintTreeValueManager.updateAndRead(updated, user.getUser());
        } else {
            return toscaTechnologicalConstraintTreeValueManager.createAndRead(updated, user.getUser());
        }

    }

    @PreAuthorize("hasRole('ROLE_USER')")
    @RequestMapping(value = "/{optimizationTCId}/fields/{id}", method = RequestMethod.DELETE)
    public void deleteFieldOfOptimizationTC(@PathVariable Integer optimizationTCId, @PathVariable Integer id, @CurrentUser PortalUser user) {
        toscaTechnologicalConstraintTreeValueManager.deleteByTreeId(id, optimizationTCId);
    }

    @PreAuthorize("hasRole('ROLE_USER')")
    @RequestMapping(value = "/{optimizationTCId}/fields", method = RequestMethod.DELETE)
    public void deleteSeveralFieldsByFieldIds(@PathVariable Integer optimizationTCId, @RequestParam("ids") List<Integer> ids, @CurrentUser PortalUser currentUser) {
        toscaTechnologicalConstraintTreeValueManager.deleteSeveralFieldsByFieldIds(ids, optimizationTCId);
    }
}
