package ru.crp.lvms.optimizations.tosca.dao;

import org.springframework.stereotype.Repository;
import ru.crp.cmlb.model.utils.EntityUtils;
import ru.crp.lvms.model.dao.BaseSpaceDao;
import ru.crp.lvms.optimizations.base.entities.tree.TechnologicalConstraintTreeNodeValue;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;

import static java.util.Arrays.asList;

@Repository
public class ToscaTechnologicalConstraintTreeValueDao extends BaseSpaceDao<TechnologicalConstraintTreeNodeValue> {

    public ToscaTechnologicalConstraintTreeValueDao() {
        setTableName("toska_technological_constraints_tree_values_join");
    }

    public void clearSavedValues(int optimizationTcId) {
        jdbcTemplate().update("DELETE FROM toska_technological_constraints_tree_values WHERE tc_id = ?", optimizationTcId);
    }

    public List<TechnologicalConstraintTreeNodeValue> findForConstraint(Integer optimizationTcId) {
        return find("WHERE tc_id = ?", optimizationTcId);
    }


    public void deleteByGroupAndParent(int optimizationTcId, String groupName, Integer parentId, int skippedId) {
        List<Object> params = new ArrayList<>(asList(optimizationTcId, parentId, groupName, optimizationTcId, skippedId));
        List<Integer> nodesToDelete = getJdbcTemplate().queryForList("WITH RECURSIVE recs(tc_tree_id) AS (SELECT ttct.tc_tree_id " +
                "                            FROM toska_technological_constraints_tree_values_join ttct " +
                "                            WHERE ttct.tc_id = ? AND ttct.parent_id = ? AND ttct.type = 'select' AND ttct.group_name = ? " +
                "                            UNION ALL SELECT ttct2.tc_tree_id " +
                "                                      FROM toska_technological_constraints_tree_values_join ttct2, recs AS r " +
                "                                      WHERE ttct2.parent_id = r.tc_tree_id AND ttct2.tc_id = ?) " +
                "SELECT * FROM recs WHERE tc_tree_id <> ?", params.toArray(), Integer.class);

        if (nodesToDelete.isEmpty()) {
            return;
        }
        String inSQL = EntityUtils.sqlForListParams(nodesToDelete);
        getJdbcTemplate().update("DELETE FROM toska_technological_constraints_tree_values WHERE tc_tree_id IN " + inSQL, nodesToDelete.toArray());
    }


    public void deleteByTreeId(Integer optimizationTcId, Integer treeId) {
        getJdbcTemplate().update("DELETE FROM toska_technological_constraints_tree_values WHERE tc_id = ? AND tc_tree_id = ?", optimizationTcId, treeId);
    }


    @Nullable
    public TechnologicalConstraintTreeNodeValue findByTreeId(Integer optimizationTCId, Integer technologicalConstraintTreeId) {
        List<TechnologicalConstraintTreeNodeValue> toscaTechnologicalConstraintTreeValues = find("WHERE tc_id = ? AND tc_tree_id = ?", optimizationTCId, technologicalConstraintTreeId);
        if (toscaTechnologicalConstraintTreeValues.isEmpty()) {
            return null;
        }
        return toscaTechnologicalConstraintTreeValues.get(0);
    }


    public void updateValue(TechnologicalConstraintTreeNodeValue entity) {
        jdbcTemplate().update("UPDATE toska_technological_constraints_tree_values_join SET value = ? WHERE tc_tree_id = ? AND tc_id = ?",
                entity.getValue(), entity.getTechnologicalConstraintTreeId(), entity.getTechnologicalConstraintId());
    }


    public void deleteSeveralFieldsByFieldIds(int optimizationTCId, List<Integer> treeIds) {
        if (treeIds.isEmpty()) {
            return;
        }
        List<Object> params = new ArrayList<>(treeIds);
        params.add(0, optimizationTCId);
        String inSql = EntityUtils.sqlForListParams(treeIds);
        getJdbcTemplate().update("DELETE FROM toska_technological_constraints_tree_values WHERE tc_id = ? AND tc_tree_id IN " + inSql,
                params.toArray());
    }


    public void deleteWithChildrenByTreeId(int optimizationTCId, List<Integer> fieldsToDelete) {
        if (fieldsToDelete.isEmpty()) {
            return;
        }
        List<Object> params = new ArrayList<>(fieldsToDelete);
        params.add(0, optimizationTCId);
        params.add(optimizationTCId);
        String inSQL = EntityUtils.sqlForListParams(fieldsToDelete);
        List<Integer> nodesToDelete = getJdbcTemplate().queryForList("WITH RECURSIVE recs(tc_tree_id) AS (SELECT ttct.tc_tree_id " +
                "                            FROM toska_technological_constraints_tree_values_join ttct " +
                "                            WHERE ttct.tc_id = ? AND ttct.tc_tree_id IN " + inSQL +
                "                            UNION ALL SELECT ttct2.tc_tree_id " +
                "                                      FROM toska_technological_constraints_tree_values_join ttct2, recs AS r " +
                "                                      WHERE ttct2.parent_id = r.tc_tree_id AND ttct2.tc_id = ?) " +
                "SELECT * FROM recs", params.toArray(), Integer.class);

        if (nodesToDelete.isEmpty()) {
            return;
        }
        params = new ArrayList<>(nodesToDelete);
        params.add(0, optimizationTCId);
        inSQL = EntityUtils.sqlForListParams(nodesToDelete);
        getJdbcTemplate().update("DELETE FROM toska_technological_constraints_tree_values WHERE tc_id = ? AND tc_tree_id IN " + inSQL, params.toArray());
    }
}
