package ru.crp.lvms.optimizations.tosca.enums;

import com.fasterxml.jackson.annotation.JsonFormat;
import ru.crp.lvms.optimizations.base.enums.OptimizationTypes;

import javax.annotation.Nonnull;
import java.util.Objects;

@JsonFormat(shape = JsonFormat.Shape.OBJECT)
public enum ToscaJsonParametersParents {
    TopoController(13, "TOPO_CONTROLLER", "Topo Controller"),
    TopoSensitivity(14, "TOPO_SENSITIVITY", "Topo Sensitivity"),

    BeadController(15, "BEAD_CONTROLLER", "Bead Controller"),
    BeadSensitivity(16, "BEAD_SENSITIVITY", "Bead Sensitivity"),

    ShapeController(17, "SHAPE_CONTROLLER", "Shape Controller"),

    PostprocessingParameters(18, "Параметры постпроцессинга", "Postprocessing params"),
    AnimationParameters(19, "Параметры анимации", "Animation params");

    private Integer id;
    private String name;
    private String dbName;

    ToscaJsonParametersParents(Integer id, String name, String dbName) {
        this.id = id;
        this.name = name;
        this.dbName = dbName;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public String getDbName() {
        return dbName;
    }

    public void setDbName(String dbName) {
        this.dbName = dbName;
    }

    public boolean isMethodAvailableForOptimizationType(OptimizationTypes optimizationType) {
        switch (optimizationType) {
            case TopologicalOptimization:
                return this == TopoController || this == TopoSensitivity;
            case TopographicOptimization:
                return this == BeadController || this == BeadSensitivity;
            case ShapeOptimization:
                return this == ShapeController;
            default:
                return false;
        }
    }

    @Nonnull
    public static ToscaJsonParametersParents valueOf(int id) {
        for (ToscaJsonParametersParents cur : values()) {
            if (cur.getId() == id) {
                return cur;
            }
        }
        throw new IllegalArgumentException("Enum value for id " + id + " is not found");
    }

    @Nonnull
    public static ToscaJsonParametersParents fromUniqueName(String uniqueName) {
        for (ToscaJsonParametersParents optimizationMethod : values()) {
            if (Objects.equals(optimizationMethod.getDbName(), uniqueName)) {
                return optimizationMethod;
            }
        }
        throw new IllegalArgumentException("There is no optimization method with such unique name: " + uniqueName);
    }
}
