package ru.crp.lvms.pdf.reports.vo;

import org.apache.commons.lang3.tuple.Pair;
import ru.crp.lvms.model.entities.effective.characteristics.ECParamEntity;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlRootElement;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@XmlRootElement(name = "parameter")
@XmlAccessorType(XmlAccessType.FIELD)
public abstract class Parameter {
    Parameter() {
        setComposite();
    }

    boolean isComposite;
    abstract Object getName();
    abstract Object getValue();
    abstract void setComposite();

    public static List<SimpleParameter> fromPairsList(List<Pair<String, String>> pairs) {
        return pairs
                .stream()
                .map(p -> new SimpleParameter(p.getKey(), p.getValue()))
                .collect(Collectors.toList());
    }

    private static List<SimpleParameter> getAllChildrenParameters(ECParamEntity parent, List<ECParamEntity> possibleChildren) {
        return possibleChildren.stream()
                .filter(child -> Objects.equals(child.getParentParamId(), parent.getParameterId()))
                .map(child -> new SimpleParameter(child.getParameterName(), child.getParameterValue()))
                .collect(Collectors.toList());
    }

    public static List<Parameter> handleCompositeParameters(List<ECParamEntity> params) {
        return params.stream()
                .filter(p -> p.getParentParamId() == null)
                .map(p -> {
                    List<SimpleParameter> children = Parameter.getAllChildrenParameters(p, params);
                    if (children.isEmpty()) {
                        return new SimpleParameter(p.getParameterName(), p.getParameterValue());
                    } else {
                        return new CompositeParameter(p.getParameterName(), children);
                    }
                })
                .collect(Collectors.toList());
    }
}

