package ru.crp.lvms.tasks.fileGenerators;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.PathVariable;
import ru.crp.cmlb.model.entities.FileData;
import ru.crp.cmlb.model.entities.ResourcePreparationException;
import ru.crp.cmls.api.TaskParentType;
import ru.crp.cmls.calculations.solvers.xml.config.DataType;
import ru.crp.lvms.exceptions.BadRequestException;
import ru.crp.lvms.exceptions.solving.TaskStartingException;
import ru.crp.lvms.managers.CompositeMaterialManager;
import ru.crp.lvms.managers.TechnologySimulationsManager;
import ru.crp.lvms.model.entities.UserEntity;
import ru.crp.lvms.model.entities.effective.characteristics.ECDefectEntity;
import ru.crp.lvms.model.entities.effective.characteristics.ECEntity;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.model.entities.technology.TechnologySimulationsEntity;
import ru.crp.lvms.model.entities.technology.TechnologySimulationsRibsEntity;
import ru.crp.lvms.model.entities.technology.TechnologyType;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Component
public class TSInputFileGenerator extends BaseInputFileGenerator {


    @Autowired
    private TechnologySimulationsManager technologySimulationsManager;


    public Path getInputFilesForTechnologySimulations(TechnologySimulationsEntity technologySimulationsEntity, List<TechnologySimulationsRibsEntity> technologySimulationsRibsEntity) throws IOException, ResourcePreparationException {

        ArrayList<FileData> fileData = new ArrayList<>();

        TechnologySimulationsEntity tsentity = technologySimulationsManager.extend(technologySimulationsEntity, null);


        return generateTSInputFile(fileData, tsentity,technologySimulationsRibsEntity);
    }

    protected Path generateTSInputFile(List<FileData> fileData, TechnologySimulationsEntity tsentity, List<TechnologySimulationsRibsEntity> technologySimulationsRibsEntity) throws IOException {
        TSParameters tsParameters = technologySimulationsManager.getTechnologySimulationsParameters(tsentity.getId());

        StringBuffer jsonfile = new StringBuffer (tsParameters.tsParamToJson());

        String ribs =ribsToJson(tsentity, technologySimulationsRibsEntity, null);
        jsonfile.append(ribs);

        Path tempConfigFile = Files.createTempFile(Paths.get("/home/space/Gcodes_work"),"generated",".json");

        Files.write(tempConfigFile,jsonfile.toString().getBytes());

        return tempConfigFile;

    }

    public String ribsToJson(TechnologySimulationsEntity technologySimulationsEntity, List<TechnologySimulationsRibsEntity> technologySimulationsRibsEntity, UserEntity currentUser) {

        StringBuffer lines = new StringBuffer("\"lines\": [ \n");

        for(TechnologySimulationsRibsEntity rib:technologySimulationsRibsEntity){
            StringBuffer stepData = new StringBuffer ();
            if(! rib.getSteps().contains(",")) {
                    stepData.append(rib.getSteps());
            } else
            {
                stepData.append("["+rib.getSteps()+"]");
            }
            lines.append("{ \n" +
                    "\"normal\": \n"+
                    "{ \n" +
                    "\"origin\": [" + rib.getOrigin_x() + ", " + rib.getOrigin_y() + "] , \n"+
                    "\"angle\":" + rib.getAngle() + "\n"+
                    "}, \n"+
                    "\"steps\":" + stepData.toString() +", \n"+
                    "\"multiplicity\":" + rib.getMultiplicity() + "\n"+
                    "}"+ ",\n" );
            stepData.delete(0, stepData.length());
        }
        lines.setLength(lines.length() - 2);
        lines.append("\n]\n"+"}");

        return lines.toString();
    }

}
