package ru.crp.lvms.tasks.slurm.callbacks;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import ru.crp.cmlb.model.entities.CalculationStatus;
import ru.crp.cmlb.model.entities.ResourcePreparationException;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.tasks.misc.BaseTaskCommandCallback;
import ru.crp.lvms.tasks.slurm.exceptions.SlurmConnectionException;

@Service
public class SlurmCancelTaskCommandCallback extends BaseTaskCommandCallback<Void> {
    private static final Logger LOGGER = LoggerFactory.getLogger(SlurmCancelTaskCommandCallback.class);

    @Override
    public void onSuccess(TaskEntity task, Object unused) {
        LOGGER.info("Successfully canceled task {}", task.getId());
        changeStatusToCanceled(task);
    }

    private void changeStatusToCanceled(TaskEntity task) {
        if (task.getStatusId() == CalculationStatus.Waiting.getId()) {
            taskStatusUpdater.changeTaskStatus(task, CalculationStatus.Canceled);
        } else {
            taskStatusUpdater.changeTaskStatus(task, CalculationStatus.Terminated);
        }
    }

    @Override
    public void onFailure(TaskEntity task, Throwable error) {
        if (error instanceof SlurmConnectionException) {
            LOGGER.error("Can't cancel task {} cause there is no connection to cluster", task.getId());
            return;
        } else if (error instanceof ResourcePreparationException) {
            LOGGER.error("Task {} canceled with errors", task.getId(), error);
        } else {
            LOGGER.error("Unexpected exception while cancelling task {}", task.getId(), error);
        }
        changeStatusToCanceled(task);
    }
}
