package ru.crp.lvms.tasks.slurm.requests;

import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ObjectDetailsRequest extends AbstractRequest<Map<String, String>> {
    private static final Pattern PROPERTY_PATTERN = Pattern.compile("([^\\s=]+)=([^\\s]*)\\s+");
    private String nodeName;
    private String commandLine;

    public ObjectDetailsRequest(String nodeName, String commandLine) {
        this.nodeName = nodeName;
        this.commandLine = commandLine;
    }

    @Override
    public String getCommandLine() {
        return String.format(commandLine, nodeName);
    }

    @Override
    public Map<String, String> parseResponse(String response) {
        Matcher matcher = PROPERTY_PATTERN.matcher(response);
        Map<String, String> props = new HashMap<>();
        while (matcher.find()) {
            props.put(matcher.group(1), matcher.group(2));
        }
        return props;
    }

    public static ObjectDetailsRequest createNodeDetailsCommand(String nodeName) {
        return new ObjectDetailsRequest(nodeName, "scontrol show node %s");
    }

    public static ObjectDetailsRequest createTaskDetailsCommand(String taskId) {
        return new ObjectDetailsRequest(taskId, "scontrol show job %s");
    }
}
