package ru.crp.lvms.utils;

import ru.crp.cmlb.model.entities.Suggest;
import ru.crp.lvms.web.dto.columns.FieldMapping;
import ru.crp.lvms.web.dto.columns.GridColumn;
import ru.crp.lvms.web.dto.columns.GridColumnType;

import javax.annotation.Nonnull;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public final class GridUtils {
    private GridUtils() {
    }

    @Nonnull
    public static List<FieldMapping> getColumnsByClass(Class<?> clazz) {
        List<FieldMapping> result = recursiveSearch(clazz, new ArrayList<>());
        Collections.sort(result);
        return result;
    }

    private static List<FieldMapping> recursiveSearch(Class<?> clazz, List<FieldMapping> columns) {
        if (!clazz.equals(Object.class)) {
            recursiveSearch(clazz.getSuperclass(), columns);
        }

        for (Field field : clazz.getDeclaredFields()) {
            GridColumn column = field.getAnnotation(GridColumn.class);
            if (column != null) {
                FieldMapping map = new FieldMapping();
                map.setRenderable(!column.isExcluded());
                map.setName(column.name());
                map.setLabel(column.label());
                map.setCell(GridColumnType.valueOf(column.cell().name()));
                map.setEditable(column.editable());
                map.setOrder(column.order());
                if (field.getAnnotation(Suggest.class) != null) {
                    map.setSuggest(field.getAnnotation(Suggest.class).value());
                }
                columns.add(map);
            }
        }
        return columns;
    }
}
