package ru.crp.lvms.validators.task;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;
import ru.crp.cmls.api.TaskParentType;
import ru.crp.lvms.managers.CompositeMaterialManager;
import ru.crp.lvms.managers.effective.characteristics.ECManager;
import ru.crp.lvms.model.entities.compositeMaterials.CompositeMaterialEntity;
import ru.crp.lvms.model.entities.effective.characteristics.ECEntity;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.model.entities.technology.TechnologyType;
import ru.crp.lvms.tasks.fileGenerators.ECParameters;

@Component
public class ECTaskParamsValidator extends BaseTaskParamsValidator<ECEntity> {
    private static final Logger LOGGER = LoggerFactory.getLogger(ECTaskParamsValidator.class);
    @Autowired
    private ECManager ecManager;

    @Autowired
    private CompositeMaterialManager compositeMaterialManager;

    @Override
    protected void doValidate(TaskEntity task, ECEntity taskParent, Errors errors) {
        if (task.getParentType() != TaskParentType.effectiveCharacteristic) {
            throw new IllegalArgumentException("Wrong validator is used for task with parent type " + task.getParentType());
        }

        ECParameters ecParameters = ecManager.getEffectiveCharacteristicsParameters(taskParent.getId());
        validateParentCompositeMaterial(taskParent.getCompositeMaterialId(), errors);
        CompositeMaterialEntity parentCompositeMaterial = compositeMaterialManager.findOne(taskParent.getCompositeMaterialId(), null);
        if (errors.hasErrors()) {
            return;
        }
        if (parentCompositeMaterial.getTechnologyType() == TechnologyType.ADDITIVE_3CM &&
                !ecParameters.isAllEpoxideParametersPresent()) {
            errors.reject("notEnoughEpoxideParams");
        }
        if (!ecParameters.isAllMatrixParametersPresent()) {
            errors.reject("notEnoughMatrixParams");
        }
        if (!ecParameters.isAllFiberParametersPresent()) {
            errors.reject("notEnoughFiberParams");
        }
        if (!ecParameters.isAllECParametersPresent()) {
            errors.reject("notEnoughECParams");
        }
    }

    @Override
    protected ECEntity getTaskParent(TaskEntity task) {
        return ecManager.findOne(task.getParentId(), null);
    }
}
