package ru.crp.lvms.web.controllers;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import ru.crp.lvms.security.PortalUser;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Map;

import static java.util.Collections.singletonMap;
import static ru.crp.lvms.utils.MessageUtils.WRONG_LOGIN;

@RestController
@RequestMapping(value = "/login", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
public class LoginController {
    private static final Logger LOGGER = LoggerFactory.getLogger(LoginController.class);

    @Autowired
    private MessageSource msgSrc;


    @Nullable
    @RequestMapping(value = "/ok")
    public PortalUser getCurrentUser() {
        return PortalUser.getCurrentUser();
    }


    @Nonnull
    @RequestMapping("/fail")
    @ResponseStatus(value = HttpStatus.UNAUTHORIZED)
    public Map<String, String> fail() {
        return singletonMap("message", msgSrc.getMessage(WRONG_LOGIN, null, null));
    }


    @ExceptionHandler(value = ClassCastException.class)
    @ResponseStatus(HttpStatus.UNAUTHORIZED)
    public void handleUnexpectedUserClass(ClassCastException ex) {
        LOGGER.error("Unexpected principal class: " + ex.getMessage());
    }
}
