package ru.crp.lvms.web.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import ru.crp.lvms.managers.ParametersManager;
import ru.crp.lvms.managers.misc.BaseManagerSuggest;
import ru.crp.lvms.managers.misc.EventManager;
import ru.crp.lvms.model.entities.ParametersEntityTable;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.web.controllers.misc.CrudControllerSuggest;

import javax.annotation.Nonnull;
import javax.validation.Valid;

import static ru.crp.lvms.utils.URLs.PARAMETERS_URL;

@RestController
@RequestMapping(value = PARAMETERS_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class ParametersController
        extends CrudControllerSuggest<ParametersEntityTable, ParametersEntityTable, ParametersEntityTable> {

    @Autowired
    private ParametersManager parametersManager;

    @Autowired
    private EventManager eventManager;

    @Override
    protected BaseManagerSuggest<ParametersEntityTable> getManager() {
        return parametersManager;
    }

    @Override
    protected EventManager getEventsManager() {
        return eventManager;
    }

    @Nonnull
    @PreAuthorize("hasRole('ROLE_ADMIN')")
    @RequestMapping(method = RequestMethod.POST)
    public ParametersEntityTable createParameter(@Valid @RequestBody ParametersEntityTable parameterIn,
                                                 @CurrentUser PortalUser currentUser) {
        return parametersManager.createAndRead(parameterIn, currentUser.getUser());
    }
}
