package ru.crp.lvms.web.controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import ru.crp.lvms.managers.SupportLetterManager;
import ru.crp.lvms.managers.misc.BaseManager;
import ru.crp.lvms.managers.misc.EventManager;
import ru.crp.lvms.managers.misc.MailSender;
import ru.crp.lvms.model.entities.SupportLetterEntity;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.web.controllers.misc.CrudControllerColumns;

import javax.validation.Valid;

import static ru.crp.lvms.utils.URLs.SUPPORT_URL;

@RestController
@RequestMapping(value = SUPPORT_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class SupportLetterController
        extends CrudControllerColumns<SupportLetterEntity, SupportLetterEntity, SupportLetterEntity> {

    @Autowired
    private SupportLetterManager supportLetterManager;

    @Autowired
    private EventManager eventManager;

    @Autowired
    private MailSender mailSender;

    @Value("${support.email}")
    private String supportEmail;

    @RequestMapping(method = RequestMethod.POST)
    public SupportLetterEntity sendLetter(@Valid @RequestBody SupportLetterEntity letter, @CurrentUser PortalUser user) {
        letter.setUserId(user.getUser().getId());
        letter.setIsProcessed(false);
        SupportLetterEntity supportLetter = supportLetterManager.createAndRead(letter, user.getUser());
        String msg = String.format("Пользователь %s столкнулся с проблемой на странице %s:\n%s.",
                supportLetter.getUserName(), supportLetter.getPageUrl(), supportLetter.getContent());
        mailSender.sendNotificationMailAsync(msg, supportEmail, "Support");
        return supportLetter;
    }

    @Override
    protected BaseManager<SupportLetterEntity> getManager() {
        return supportLetterManager;
    }

    @Override
    protected EventManager getEventsManager() {
        return eventManager;
    }
}
