package ru.crp.lvms.web.controllers.materials;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import ru.crp.lvms.managers.ParametersManager;
import ru.crp.lvms.model.entities.ParameterSample;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.web.dto.DefaultParameterValue;

import javax.annotation.Nonnull;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.List;

import static ru.crp.lvms.utils.URLs.*;

@RestController
@RequestMapping(value = {SINGLE_MATERIAL_URL, SINGLE_COMPOSITE_MATERIAL_URL},
        produces = MediaType.APPLICATION_JSON_VALUE)
public class MaterialParametersController {
    @Autowired
    private ParametersManager parametersManager;


    @Nonnull
    @RequestMapping(value = PARAMETERS_URL, method = RequestMethod.GET)
    public List<DefaultParameterValue> getDefaultMaterialParameters(HttpServletRequest request,
                                                                    @PathVariable("materialId") int materialId,
                                                                    @CurrentUser PortalUser currentUser) {
        boolean isCompositeMaterial = checkIsCompositeMaterial(request);
        return parametersManager.getDefaultMaterialParametersValues(isCompositeMaterial, materialId,
                currentUser.getUser());
    }


    @Nonnull
    @RequestMapping(value = PARAMETERS_URL + "/{parameterId}", method = RequestMethod.GET)
    public List<ParameterSample> getMaterialParameterValues(HttpServletRequest request,
                                                            @PathVariable("materialId") int materialId,
                                                            @PathVariable("parameterId") int parameterId,
                                                            @CurrentUser PortalUser currentUser) {
        boolean isCompositeMaterial = checkIsCompositeMaterial(request);
        return parametersManager.getMaterialParameterValues(isCompositeMaterial, materialId, parameterId,
                currentUser.getUser());
    }


    @Nonnull
    @RequestMapping(value = PARAMETERS_URL + "/{parameterId}", method = RequestMethod.PUT)
    public List<ParameterSample> setMaterialParameterValues(HttpServletRequest request,
                                                            @PathVariable("materialId") int materialId,
                                                            @PathVariable("parameterId") int parameterId,
                                                            @Valid @RequestBody List<ParameterSample> newParameterValues,
                                                            @CurrentUser PortalUser currentUser) {
        boolean isCompositeMaterial = checkIsCompositeMaterial(request);
        return parametersManager.setMaterialParameterValues(isCompositeMaterial, materialId, parameterId,
                newParameterValues, currentUser.getUser());
    }


    @Nonnull
    @RequestMapping(value = PARAMETERS_URL + "/{parameterId}", method = RequestMethod.DELETE)
    public List<ParameterSample> deleteAllMaterialParameterValues(HttpServletRequest request,
                                                                  @PathVariable("materialId") int materialId,
                                                                  @PathVariable("parameterId") int parameterId,
                                                                  @CurrentUser PortalUser currentUser) {
        boolean isCompositeMaterial = checkIsCompositeMaterial(request);
        return parametersManager.deleteMaterialParameterValues(isCompositeMaterial, materialId, parameterId,
                currentUser.getUser());
    }


    @Nonnull
    @RequestMapping(value = PARAMETERS_URL + "/{parameterId}/point", method = RequestMethod.POST)
    public List<ParameterSample> upsertMaterialParameterValue(HttpServletRequest request,
                                                              @PathVariable("materialId") int materialId,
                                                              @PathVariable("parameterId") int parameterId,
                                                              @Valid @RequestBody ParameterSample newParameterValue,
                                                              @CurrentUser PortalUser currentUser) {
        boolean isCompositeMaterial = checkIsCompositeMaterial(request);
        return parametersManager.upsertMaterialParameterValue(isCompositeMaterial, materialId, parameterId,
                newParameterValue, currentUser.getUser());
    }


    @RequestMapping(value = PARAMETERS_URL + "/{parameterId}/point/{pointId}", method = RequestMethod.PUT)
    public List<ParameterSample> updateParameterValue(HttpServletRequest request,
                                                      @PathVariable("materialId") int materialId,
                                                      @PathVariable("parameterId") int parameterId,
                                                      @PathVariable("pointId") int pointId,
                                                      @Valid @RequestBody ParameterSample newParameterValue,
                                                      @CurrentUser PortalUser currentUser) {
        newParameterValue.setId(pointId);
        boolean isCompositeMaterial = checkIsCompositeMaterial(request);
        return parametersManager.updateMaterialParameterValue(isCompositeMaterial, materialId, parameterId,
                newParameterValue, currentUser.getUser());
    }


    @ResponseStatus(HttpStatus.OK)
    @RequestMapping(value = PARAMETERS_URL + "/{parameterId}/point/{pointId}", method = RequestMethod.DELETE)
    public void deleteParameterValue(HttpServletRequest request,
                                     @PathVariable("materialId") int materialId,
                                     @PathVariable("parameterId") int parameterId,
                                     @PathVariable("pointId") int pointId,
                                     @CurrentUser PortalUser currentUser) {
        boolean isCompositeMaterial = checkIsCompositeMaterial(request);
        parametersManager.deleteParameterValue(isCompositeMaterial, materialId, parameterId, pointId,
                currentUser.getUser());
    }
}
