package ru.crp.lvms.web.controllers.misc;

import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import ru.crp.cmlb.model.SuggestRequest;
import ru.crp.cmlb.model.entities.base.Entity;
import ru.crp.lvms.managers.misc.BaseManagerSuggest;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.utils.Mapper;
import ru.crp.lvms.web.dto.pagination.DataRequest;
import ru.crp.lvms.web.dto.pagination.SortingOrder;

import javax.validation.Valid;
import java.util.List;

public abstract class CrudControllerSuggest<E extends Entity, InVO, OutVO> extends
        CrudControllerColumns<E, InVO, OutVO> {
    @Override
    protected abstract BaseManagerSuggest<E> getManager();

    @RequestMapping(value = "/suggest", method = RequestMethod.POST)
    public List<OutVO> suggest(@RequestBody @Valid SuggestRequest suggestRequest, @CurrentUser PortalUser currentUser) {
        return Mapper.map(getManager().suggest(suggestRequest, currentUser.getUser()), getListOutVOs());
    }

    @Transactional(readOnly = true)
    @RequestMapping(value = "/recent", method = RequestMethod.POST)
    public PageImpl<OutVO> getRecent(@CurrentUser PortalUser user, @RequestBody DataRequest dataRequest) throws Exception {
        SortingOrder sortByDate = new SortingOrder(Sort.Direction.DESC, "modificationDate");
        if (!dataRequest.getSort().contains(sortByDate)) {
            dataRequest.getSort().add(sortByDate);
        }
        Pageable pageable = dataRequest.getPageable().toSpringPageable(
                new Sort(SortingOrder.toSpringSortabble(dataRequest.getSort())));
        PageImpl<E> data = getManager().findByOwnerId(user.getUser().getId(), dataRequest.getFilters(), pageable, user.getUser());
        return new PageImpl<>(Mapper.map(data.getContent(), getListOutVOs()), pageable, data.getTotalElements());
    }
}
