package ru.crp.lvms.web.controllers.tasks;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import ru.crp.lvms.managers.tasks.TaskManager;
import ru.crp.lvms.model.entities.UserEntity;
import ru.crp.lvms.model.entities.tasks.TaskEntity;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.web.dto.pagination.DataRequest;
import ru.crp.lvms.web.dto.pagination.PageingResponse;
import ru.crp.lvms.web.dto.pagination.SortingOrder;

import java.util.Map;
import java.util.function.BiFunction;

@RestController
@RequestMapping(value = "/tasks", produces = MediaType.APPLICATION_JSON_VALUE)
public class TaskListsController {
    public static final String OWNED_ONLY_FILTER_KEY = "ownedOnly";

    @Autowired
    private TaskManager taskManager;

    @RequestMapping(value = "/queue", method = RequestMethod.POST)
    public PageingResponse<TaskEntity> getQueue(@RequestBody DataRequest dataRequest, @CurrentUser PortalUser currentUser) {
        return getTasksList(dataRequest, currentUser.getUser(), taskManager::getTaskQueue);
    }

    @RequestMapping(value = "/running", method = RequestMethod.POST)
    public PageingResponse<TaskEntity> getRunningTasksPage(@RequestBody DataRequest dataRequest, @CurrentUser PortalUser currentUser) {
        return getTasksList(dataRequest, currentUser.getUser(), taskManager::findRunning);
    }

    @RequestMapping(value = "/history", method = RequestMethod.POST)
    public PageingResponse<TaskEntity> getFinishedTasks(@RequestBody DataRequest dataRequest, @CurrentUser PortalUser currentUser) {
        return getTasksList(dataRequest, currentUser.getUser(), taskManager::getFinishedTasksPage);

    }

    private PageingResponse<TaskEntity> getTasksList(DataRequest dataRequest, UserEntity currentUser,
                                                     BiFunction<Map<String, Object>, Pageable, PageImpl<TaskEntity>> tasksListSupplier) {
        Map<String, Object> filters = dataRequest.getFilters();
        Object ownedOnly = filters.get(OWNED_ONLY_FILTER_KEY);
        if (ownedOnly != null && Boolean.valueOf(ownedOnly.toString())) {
            filters.put("ownerId", currentUser.getId());
        }

        Pageable pageable;
        Sort sort = null;
        if (!dataRequest.getSort().isEmpty()) {
            sort = new Sort(SortingOrder.toSpringSortabble(dataRequest.getSort()));
        }
        pageable = dataRequest.getPageable().toSpringPageable(sort);
        PageImpl<TaskEntity> result = tasksListSupplier.apply(filters, pageable);

        if (filters.containsKey(OWNED_ONLY_FILTER_KEY)) {
            filters.remove("ownerId");
        }

        return new PageingResponse<>(result.getContent(), filters, pageable, result.getTotalElements());
    }
}
