package ru.crp.lvms.web.controllers.technology;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.ObjectWriter;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import ru.crp.cmlb.model.entities.ResourcePreparationException;
import ru.crp.lvms.exceptions.BadRequestException;
import ru.crp.lvms.exceptions.NotFoundException;
import ru.crp.lvms.exceptions.ValidationException;
import ru.crp.lvms.managers.FileManager;
import ru.crp.lvms.managers.TechnologyManager;
import ru.crp.lvms.managers.TechnologySimulationsManager;
import ru.crp.lvms.managers.effective.characteristics.ECManager;
import ru.crp.lvms.managers.image.ImageHandler;
import ru.crp.lvms.managers.misc.BaseManagerSuggest;
import ru.crp.lvms.managers.misc.EventManager;
import ru.crp.lvms.model.SuggestDTO;
import ru.crp.lvms.model.entities.FileEntity;
//import ru.crp.lvms.model.entities.effective.characteristics.ECParamEntity;
import ru.crp.lvms.model.entities.effective.characteristics.ECEntity;
import ru.crp.lvms.model.entities.effective.characteristics.ECParamEntity;
import ru.crp.lvms.model.entities.technology.TechnologySimulationsParamEntity;
import ru.crp.lvms.model.entities.simulations.SimulationEntity;
import ru.crp.lvms.model.entities.technology.*;
import ru.crp.lvms.security.CurrentUser;
import ru.crp.lvms.security.PortalUser;
import ru.crp.lvms.tasks.fileGenerators.TSParameters;
import ru.crp.lvms.utils.Mapper;
import ru.crp.lvms.web.controllers.misc.ControllerImageNew;
import ru.crp.lvms.web.controllers.misc.CrudController;
import ru.crp.lvms.web.controllers.misc.CrudControllerSuggest;
import ru.crp.lvms.web.controllers.misc.FileHelper;
import ru.crp.lvms.web.dto.pagination.DataRequest;
import ru.crp.lvms.web.dto.pagination.PageingResponse;

import javax.annotation.Nonnull;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolation;
import javax.validation.Valid;
import javax.validation.Validator;
import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Comparator;
import java.util.List;
import java.util.Set;
import java.util.stream.Stream;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import static java.util.stream.Collectors.toList;
import static ru.crp.lvms.utils.URLs.TECHNOLOGY_SIMULATIONS_URL;
import static ru.crp.lvms.utils.URLs.TECHNOLOGY_URL;

@RestController
@RequestMapping(value = TECHNOLOGY_SIMULATIONS_URL, produces = MediaType.APPLICATION_JSON_VALUE)
public class TechnologySimulationsController
        extends CrudController<TechnologySimulationsEntity, TechnologySimulationsEntity, TechnologySimulationsEntity>
        implements FileHelper, ControllerImageNew {
    @Autowired
    private TechnologyManager technologyManager;

    @Autowired
    private TechnologySimulationsManager technologySimulationsManager;

    @Autowired
    private EventManager eventManager;

    @Autowired
    private FileManager fileManager;

    @Autowired
    private Validator validator;

    @Override
    public Class getOutVoClass() {
        return TechnologySimulationsRibsEntity.class;
    }


    @RequestMapping(method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_VALUE)
    public TechnologySimulationsEntity createTechnologySimulations(@RequestBody @Valid TechnologySimulationsEntity newTechnologySimulations,
                                                               @CurrentUser PortalUser currentUser) {
        return technologySimulationsManager.createTechnologySimulations(newTechnologySimulations, currentUser.getUser());
    }



    @NotNull
    @Override
    public TechnologySimulationsEntity get(HttpServletRequest request, @PathVariable("id") int technologySimulationsId, @CurrentUser PortalUser currentUser) {
        return getManager().extend(super.get(request, technologySimulationsId, currentUser), currentUser.getUser());
    }

    @Nonnull
    @Override
    public TechnologySimulationsEntity update(HttpServletRequest request, @PathVariable("id") int technologySimulationsId, @Valid @RequestBody TechnologySimulationsEntity updateVO,
                           @CurrentUser PortalUser user) {
        TechnologySimulationsEntity existed = getManager().findOne(technologySimulationsId, user.getUser());
        checkEntityExists(technologySimulationsId, existed);

        updateVO.setTechnologyId(existed.getTechnologyId());
        updateVO.setOwnerId(existed.getOwnerId());
        updateVO.setCreationDate(existed.getCreationDate());
        updateVO.setId(existed.getId());

        Mapper.map(updateVO, existed);
        return getManager().updateAndRead(existed, user.getUser());
    }

    @Nonnull
    @RequestMapping(value = "/{id}/copy", method = RequestMethod.POST)
    public TechnologySimulationsEntity copyTechnologySimulation(@PathVariable("id") int technologySimulationsId,
                                                                @RequestParam("name") String name,
                                              @CurrentUser PortalUser user) {
        return getManager().copyTechnologySimulation(technologySimulationsId,name, user.getUser());
    }



    @RequestMapping(value = "/{id}/simulations/list", method = RequestMethod.POST)
    public PageingResponse<SimulationEntity> getTechnolotySimulationsPage(@PathVariable("id") Integer technologySimulationsId,
                                                                  @RequestBody DataRequest dataRequest,
                                                                  @CurrentUser PortalUser currentUser) throws IOException {
        Pageable pageable = dataRequest.getSpringPageable();
        PageImpl<SimulationEntity> page = technologyManager.getSimulationsPageForTechnology(technologySimulationsId, pageable,
                dataRequest.getFilters(), currentUser.getUser());
        return new PageingResponse<>(page.getContent(), dataRequest.getFilters(), pageable, page.getTotalElements());
    }

    @RequestMapping(value = "/{id}/start", method = RequestMethod.GET)
    public void startTechnologySimulations(HttpServletResponse response,
                                           @PathVariable("id") Integer technologySimulationsId,
                                                                   @CurrentUser PortalUser user) throws IOException, ResourcePreparationException {

        technologySimulationsManager.startTechnologySimulations(response,technologySimulationsId, user.getUser());

    }



    //параметры
    @Nonnull
    @RequestMapping(value = "/{id}/parameters", method = RequestMethod.GET)
    public List<TechnologySimulationsParamEntity> getAllParameters(@PathVariable("id") Integer technologySimulationsId, @CurrentUser PortalUser currentUser) {
        checkEntityExists(technologySimulationsId, currentUser);
        return getManager()
                .getTSParameters(technologySimulationsId);
    }


    @Nonnull
    @RequestMapping(value = "/{id}/parameters", method = RequestMethod.POST)
    public TechnologySimulationsParamEntity createParameter(@PathVariable("id") Integer technologySimulationsId, @Valid @RequestBody TechnologySimulationsParamEntity newParamDTO,
                                                            @CurrentUser PortalUser currentUser) {
     //   throw new BadRequestException("Менять список параметров запрещено");
        checkEntityExists(technologySimulationsId, currentUser);
        newParamDTO.setTechnologySimulationId(technologySimulationsId);
        return getManager().createTSParam(newParamDTO);
    }

    @Nonnull
    @RequestMapping(value = "/{id}/parameters/{paramId}", method = RequestMethod.PUT)
    public TechnologySimulationsParamEntity updateParameter(@PathVariable("id") Integer technologySimulationsId,
                                         @PathVariable("paramId") Integer paramId,
                                         @Valid @RequestBody TechnologySimulationsParamEntity updateDTO,
                                         @CurrentUser PortalUser currentUser) {
        checkEntityExists(technologySimulationsId, currentUser);
        TechnologySimulationsParamEntity existedParam = checkParamExistsAndBelongsToTS(technologySimulationsId, paramId, currentUser);
        return getManager().updateTSParam(existedParam, updateDTO);
    }

    @Nonnull
    @RequestMapping(value = "/{id}/parameters/{paramId}", method = RequestMethod.DELETE)
    public TechnologySimulationsParamEntity deleteParameter(@PathVariable("id") Integer technologySimulationsId, @PathVariable("paramId") Integer paramId,
                                         @CurrentUser PortalUser currentUser) {
        throw new BadRequestException("Менять список параметров запрещено");
//        checkEntityExists(ecId, currentUser);
//        ECParamEntity existedParam = checkParamExistsAndBelongsToEC(ecId, paramId, currentUser);
//        getManager().deleteECParam(existedParam);
//        return existedParam;
    }

    @RequestMapping(value = "/{id}/parameters", method = RequestMethod.DELETE)
    public void deleteSeveralParameters(@PathVariable("id") Integer technologySimulationsId,
                                        @RequestParam("ids") List<Integer> paramIds,
                                        @CurrentUser PortalUser currentUser) throws BadRequestException {
        throw new BadRequestException("Менять список параметров запрещено");
//        deleteSeveralEntities(ECParamEntity.class, paramIds, (id) -> {
//            deleteParameter(ecId, id, currentUser);
//        });
    }


    private TechnologySimulationsParamEntity checkParamExistsAndBelongsToTS(Integer technologySimulationsId, Integer paramId, PortalUser currentUser) {
        TechnologySimulationsParamEntity defect = getManager().getTSParameter(technologySimulationsId, paramId, currentUser);
        if (defect == null) {
            throw new NotFoundException(paramId, TechnologySimulationsParamEntity.class);
        }
        return defect;
    }



    //ребра

    @Nonnull
    @RequestMapping(value = "/{id}/ribs/order", method = RequestMethod.POST)
    public List<TechnologySimulationsRibsEntity> setRibs(@PathVariable("id") int technologySimulationsId,
                                                         @Valid @RequestBody List<Integer> newRibs,
                                                         @CurrentUser PortalUser currentUser) {
        technologySimulationsManager.updateRibsOrder(technologySimulationsId, newRibs, currentUser.getUser());
        return getRibs(technologySimulationsId, currentUser);
    }

    @Nonnull
    @RequestMapping(value = "/{id}/ribs", method = RequestMethod.GET)
    public List<TechnologySimulationsRibsEntity> getRibs(@PathVariable("id") int technologySimulationsId,
                                                         @CurrentUser PortalUser currentUser) {
        return technologySimulationsManager.getRibs(technologySimulationsId, true, currentUser.getUser());
    }


    @Nonnull
    @RequestMapping(value = "/{id}/ribs", method = RequestMethod.POST)
    @ResponseStatus(HttpStatus.OK)
    public void addRibs(@PathVariable("id") int technologySimulationsId,
                        @Valid @RequestBody TechnologySimulationsRibsEntity newRibs,
                        @CurrentUser PortalUser currentUser) {
        technologySimulationsManager.addRibs(technologySimulationsId, newRibs, currentUser.getUser());
    }

    @Nonnull
    @RequestMapping(value = "/{id}/ribs/{ribsId}", method = RequestMethod.DELETE)
    @ResponseStatus(HttpStatus.OK)
    public void deleteRibs(@PathVariable("id") int technologySimulationsId,
                           @PathVariable("ribsId") int ribsId,
                           @CurrentUser PortalUser currentUser) {
        technologySimulationsManager.deleteRibs(technologySimulationsId, ribsId, currentUser.getUser());
    }

    @RequestMapping(value = "/{id}/ribs" +
            "", method = RequestMethod.DELETE)
    public void deleteSeveralRibs(@PathVariable("id") Integer technologySimulationsId,
                                  @RequestParam("ids") List<Integer> ribsIds,
                                  @CurrentUser PortalUser currentUser) throws BadRequestException {
        deleteSeveralEntities(TechnologySimulationsRibsEntity.class, ribsIds, (id) -> {
            deleteRibs(technologySimulationsId, id, currentUser);
        });
    }



//    @Override
//    protected BaseManagerSuggest<TechnologySimulationsEntity> getManager() {
//        return technologySimulationsManager;
//    }


    @Override
    protected TechnologySimulationsManager getManager() {
        return technologySimulationsManager;
    }


    @Override
    protected EventManager getEventsManager() {
        return eventManager;
    }


    @Override
    public FileManager getFileManager() {
        return fileManager;
    }


    @Override
    public ImageHandler imageHandler() {
        return technologyManager;
    }


    @Override
    public Object getEntity(HttpServletRequest request, Integer entityId, PortalUser currentUser) {
        return get(request, entityId, currentUser);
    }
}
