package ru.crp.lvms.web.dto.user;

import org.hibernate.validator.constraints.Email;
import org.hibernate.validator.constraints.NotBlank;
import ru.crp.lvms.model.entities.UserEntity;
import ru.crp.lvms.validators.Unique;
import ru.crp.lvms.web.dto.misc.ValueObject;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.util.Date;

@Unique(entityClass = UserEntity.class, fieldName = "login", columnNane = "login",
        message = "Логин должен быть уникален")
public class UserCreate extends ValueObject {

    @NotBlank(message = "Логин не может быть пустым")
    private String login;
    @NotBlank(message = "Пароль не может быть пустым")
    private String password;
    @NotBlank(message = "Почта неможет быть пуста")
    @Email(message = "Неверный формат почты")
    private String email;
    @NotBlank(message = "Имя пользователя не может быть пусто")
    private String name;
    @NotNull(message = "Телефон не может быть пустым")
    private String phone = "";
    @Min(message = "Роль не может быть пустой", value = 1)
    private int roleId;
    private Date birthDate;

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getPhone() {
        return phone;
    }

    public void setPhone(String phone) {
        this.phone = phone;
    }

    public int getRoleId() {
        return roleId;
    }

    public void setRoleId(int roleId) {
        this.roleId = roleId;
    }

    public Date getBirthDate() {
        return birthDate != null ? new Date(birthDate.getTime()) :  null;
    }

    public void setBirthDate(Date birthDate) {
        this.birthDate = birthDate != null ? new Date(birthDate.getTime()) : null;
    }

    public String getLogin() {
        return login;
    }

    public void setLogin(String login) {
        this.login = login;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }
}
