CREATE TABLE support_letters (
  id SERIAL,
  content VARCHAR(5000) NOT NULL,
  page_url VARCHAR(255) NOT NULL,
  date_created TIMESTAMP WITHOUT TIME ZONE DEFAULT now() NOT NULL,
  user_id INTEGER NOT NULL,
  is_processed BOOLEAN NOT NULL,

  FOREIGN KEY(user_id) REFERENCES users(id) ON DELETE CASCADE
);

CREATE VIEW support_letters_join AS
  SELECT
    sl.*,
    u.name AS user_name
  FROM support_letters sl
    JOIN users u ON sl.user_id = u.id;

CREATE RULE support_letters_join_CREATE AS ON INSERT TO support_letters_join DO INSTEAD (
  INSERT INTO support_letters (content, user_id, page_url, is_processed)
  VALUES (NEW.content, NEW.user_id, NEW.page_url, NEW.is_processed)
  RETURNING
    support_letters.*,
    NULL :: VARCHAR(255) AS user_name
);

CREATE RULE support_letters_join_UPDATE AS ON UPDATE TO support_letters_join DO INSTEAD (
  UPDATE support_letters
  SET content = NEW.content, page_url = NEW.page_url, is_processed = NEW.is_processed
  WHERE id = NEW.id);

CREATE RULE support_letters_join_DELETE AS ON DELETE TO support_letters_join DO INSTEAD (
  DELETE FROM support_letters WHERE id = OLD.id
);
