ALTER TABLE optimization_folders ADD COLUMN base_optimization_id INTEGER REFERENCES optimizations (id);

ALTER TABLE optimization_folders ADD CONSTRAINT base_optimization_present CHECK
( (parent_folder_id IS NULL AND base_optimization_id IS NULL) OR
  (parent_folder_id IS NOT NULL AND base_optimization_id IS NOT NULL));

CREATE OR REPLACE VIEW optimizations_tree AS
  SELECT
    id,
    pid,
    name,
    path,
    path_names,
    'Folder' :: tree_node_type AS item_type,
    NULL :: INTEGER            AS optimization_type_id,
    NULL :: INTEGER            AS construction_element_id,
    NULL :: INTEGER            AS base_optimization_id
  FROM folders
  WHERE tree_type = 'constructionElements'
  UNION
  SELECT
    id,
    pid,
    name,
    path,
    path_names,
    'ConstructionElement' :: tree_node_type AS item_type,
    NULL :: INTEGER                         AS optimization_type_id,
    NULL :: INTEGER                         AS construction_element_id,
    NULL :: INTEGER                         AS base_optimization_id
  FROM construction_elements
  UNION
  SELECT
    id,
    CASE WHEN parent_folder_id IS NULL
      THEN construction_element_id
    ELSE parent_folder_id END              AS pid,
    name,
    path,
    path_names,
    'OptimizationFolder' :: tree_node_type AS item_type,
    optimization_type_id,
    construction_element_id,
    base_optimization_id
  FROM optimization_folders;

DROP VIEW optimizations_tree_join CASCADE;
CREATE OR REPLACE VIEW optimizations_tree_join AS
  SELECT
    ot.*,
    (SELECT string_agg(DISTINCT item_type :: TEXT, ',')
     FROM optimizations_tree ot2
     WHERE ot2.pid = ot.id) AS children_types
  FROM optimizations_tree ot;


CREATE OR REPLACE RULE optimizations_tree_update_elements AS ON UPDATE TO optimizations_tree_join DO INSTEAD (
  UPDATE construction_elements
  SET NAME = NEW.name
  WHERE id = NEW.id);

CREATE OR REPLACE RULE optimizations_tree_update_folders AS ON UPDATE TO optimizations_tree_join DO INSTEAD (
  UPDATE folders
  SET NAME = NEW.name
  WHERE id = NEW.id);

CREATE OR REPLACE RULE optimizations_tree_update_opt_folders AS ON UPDATE TO optimizations_tree_join DO INSTEAD (
  UPDATE optimization_folders
  SET NAME = NEW.name
  WHERE id = NEW.id);

CREATE OR REPLACE VIEW tree AS
  SELECT *
  FROM materials_tree_join
  UNION
  SELECT *
  FROM composite_materials_tree_join
  UNION
  SELECT *
  FROM technologies_tree
  UNION
  SELECT *
  FROM construction_elements_tree_join
  UNION
  SELECT
    id,
    pid,
    name,
    path,
    path_names,
    item_type,
    children_types
  FROM optimizations_tree_join;


CREATE OR REPLACE RULE tree_path_update_folders AS ON UPDATE TO tree DO INSTEAD (
  UPDATE folders
  SET NAME = NEW.name
  WHERE id = NEW.id);

CREATE OR REPLACE RULE tree_path_update_const_elem AS ON UPDATE TO tree DO INSTEAD (
  UPDATE construction_elements
  SET path = NEW.path, path_names = NEW.path_names
  WHERE id = NEW.id);

CREATE OR REPLACE RULE tree_path_update_file_types AS ON UPDATE TO tree DO INSTEAD (
  UPDATE construction_element_file_types
  SET path = NEW.path, path_names = NEW.path_names
  WHERE id = NEW.id);

CREATE OR REPLACE RULE tree_path_update_files AS ON UPDATE TO tree DO INSTEAD (
  UPDATE files
  SET path = NEW.path, path_names = NEW.path_names
  WHERE id = NEW.id);

CREATE OR REPLACE RULE tree_path_update_materials AS ON UPDATE TO tree DO INSTEAD (
  UPDATE materials
  SET path = NEW.path, path_names = NEW.path_names
  WHERE id = NEW.id);

CREATE OR REPLACE RULE tree_path_update_comp_materials AS ON UPDATE TO tree DO INSTEAD (
  UPDATE composite_materials
  SET path = NEW.path, path_names = NEW.path_names
  WHERE id = NEW.id);

CREATE OR REPLACE RULE tree_path_update_technologies AS ON UPDATE TO tree DO INSTEAD (
  UPDATE technologies
  SET path = NEW.path, path_names = NEW.path_names
  WHERE id = NEW.id);

CREATE OR REPLACE RULE tree_path_update_opt_folders AS ON UPDATE TO tree DO INSTEAD (
  UPDATE optimization_folders
  SET path = NEW.path, path_names = NEW.path_names
  WHERE id = NEW.id);