CREATE TABLE possible_defects_names (
  id   SERIAL,
  name VARCHAR,

  PRIMARY KEY (id),
  UNIQUE (name)
);
INSERT INTO possible_defects_names (name) VALUES ('Расслоение');
INSERT INTO possible_defects_names (name) VALUES ('Непроклей');
INSERT INTO possible_defects_names (name) VALUES ('Трещина');
INSERT INTO possible_defects_names (name) VALUES ('Поверхностный дефект');
INSERT INTO possible_defects_names (name) VALUES ('Канал');
INSERT INTO possible_defects_names (name) VALUES ('Пора');

ALTER TABLE possible_defects ADD COLUMN defect_name_id INTEGER NOT NULL DEFAULT 1 REFERENCES possible_defects_names (id);

-- todo migration script
ALTER TABLE possible_defects DROP COLUMN defect_name;

CREATE VIEW possible_defects_join AS
  SELECT
    pd.*,
    pdn.name AS defect_name
  FROM possible_defects pd
    JOIN possible_defects_names pdn ON pdn.id = pd.defect_name_id;

CREATE OR REPLACE RULE possible_defects_create AS ON INSERT TO possible_defects_join DO INSTEAD (
  INSERT INTO possible_defects (technology_id, defect_name_id, defect_value, act_name) VALUES
    (NEW.technology_id, NEW.defect_name_id, NEW.defect_value, NEW.act_name)
  RETURNING
    possible_defects.*, '' :: VARCHAR AS defect_name);


CREATE OR REPLACE RULE possible_defects_update AS ON UPDATE TO possible_defects_join DO INSTEAD (
  UPDATE possible_defects
  SET technology_id = NEW.technology_id, defect_name_id = NEW.defect_name_id, defect_value = NEW.defect_value,
    act_name        = NEW.act_name
  WHERE id = NEW.id);

CREATE OR REPLACE RULE possible_defects_delete AS ON DELETE TO possible_defects_join DO INSTEAD (
  DELETE FROM possible_defects
  WHERE id = OLD.id
);
