-- adding results count to views

CREATE INDEX ON results(ec_id);
CREATE INDEX ON results(sim_id);
CREATE INDEX ON results(optimization_id);

-- updating simulations
DROP VIEW virtual_simulations_join;
CREATE VIEW virtual_simulations_join AS (
  SELECT
    vs.*,
    cm.name AS composite_material_name,
    f.name  AS sample_file_name,
    s.name  AS standard_name,
    u.name AS owner,
    f2.name AS experiment_results_file_name,
    ec.name AS ec_name,
    (SELECT COUNT(*) FROM results WHERE sim_id = vs.id) AS results_count
  FROM virtual_simulations vs
    INNER JOIN composite_materials cm ON vs.composite_material_id = cm.id
    INNER JOIN users u ON u.id = vs.owner_id
    INNER JOIN effective_characteristics ec ON ec.id = vs.ec_id

    LEFT JOIN files f ON vs.sample_file_id = f.id
    LEFT JOIN standards s ON vs.standard_id = s.id
    LEFT JOIN files f2 ON vs.experiment_results_file_id = f2.id
);

-- updating effective characteristics
DROP VIEW effective_characteristics_join;
CREATE VIEW effective_characteristics_join AS (
  SELECT ec.*,
  u.name AS owner,
  cm.name AS composite_material_name,
  mf.name AS microstructure_file_name,
  (SELECT COUNT(*) FROM results WHERE ec_id = ec.id) AS results_count
  FROM effective_characteristics ec
  JOIN users u ON u.id = ec.owner_id
  JOIN composite_materials cm ON cm.id = ec.composite_material_id
  LEFT JOIN files mf ON mf.id = ec.microstructure_file_id
);
