CREATE TABLE toska_technological_constraints (
  id SERIAL,
  name VARCHAR(255) UNIQUE,
  type VARCHAR(255),
  group_name VARCHAR(255),
  comand_name VARCHAR(255),

  PRIMARY KEY(id)
);

CREATE TABLE toska_optimization_technological_constraints (
  id SERIAL,
  constraint_id INTEGER NOT NULL DEFAULT 1,
  optimization_id INTEGER NOT NULL,
  name VARCHAR(255),
  set_id INTEGER,

  FOREIGN KEY(optimization_id) REFERENCES optimizations(id) ON DELETE CASCADE,
  FOREIGN KEY(constraint_id) REFERENCES toska_technological_constraints(id) ON DELETE CASCADE,
  FOREIGN KEY(set_id) REFERENCES toska_sets(id) ON DELETE SET NULL,
  PRIMARY KEY(id)
);

CREATE VIEW toska_technological_constraints_join AS
  SELECT totc.*, ttc.name as constraint_name, ttc.comand_name
  FROM toska_optimization_technological_constraints totc
    JOIN toska_technological_constraints ttc
      ON ttc.id = totc.constraint_id;

CREATE RULE toska_technological_constraints_join_CREATE AS ON INSERT TO toska_technological_constraints_join DO INSTEAD (
  INSERT INTO toska_optimization_technological_constraints (name, constraint_id, optimization_id, set_id)
  VALUES (NEW.name, NEW.constraint_id, NEW.optimization_id, NEW.set_id)
  RETURNING
    toska_optimization_technological_constraints.*,
    NULL :: VARCHAR(255) AS constraint_name,
    NULL :: VARCHAR(255) AS comand_name
);

CREATE RULE toska_technological_constraints_join_UPDATE AS ON UPDATE TO toska_technological_constraints_join DO INSTEAD (
  UPDATE toska_optimization_technological_constraints
  SET name = NEW.name, constraint_id = NEW.constraint_id, set_id = NEW.set_id
  WHERE id = NEW.id);

CREATE RULE toska_technological_constraints_join_DELETE AS ON DELETE TO toska_technological_constraints_join DO INSTEAD (
  DELETE FROM toska_optimization_technological_constraints WHERE id = OLD.id
);

INSERT INTO toska_technological_constraints(id, name, type, group_name, comand_name) VALUES
  (1, 'Фиксированная область', 'java.lang.String', 'NONE', 'FROZEN'),
  (2, 'Максимальный диаметр', 'java.lang.String', 'NONE', 'MAX_MEMBER'),
  (3, 'Минимальный диаметр', 'java.lang.String', 'NONE', 'MIN_MEMBER'),
  (4, 'Расстояние между структурными элементами', 'java.lang.String', 'NONE', 'MAX_MEMBER');
--   ('Направление литья', 'ru.crp.lvms.optimizations.toska.enums.FoundryDirection', 'Изготовление литьем'),
--   ('Уклон', 'java.lang.String', 'Изготовление литьем'),
--   ('Элементы для проверки', 'java.lang.String', 'Изготовление литьем'),
--   ('Cрединная плоскость', 'ru.crp.lvms.optimizations.toska.enums.MidPlain', 'Изготовление литьем'),
--   ('Cистема координат литья', 'java.lang.String', 'Изготовление литьем'),
--   ('Радиус', 'java.lang.String', 'Изготовление литьем'),
--   ('Автоматически создавать группы элементов', 'java.lang.Boolean', 'Изготовление литьем'),
--   ('Фильтрация материалов', 'java.lang.Boolean', 'Изготовление литьем'),
--   ('Ось симметрии', 'ru.crp.lvms.optimizations.toska.enums.Symmetries', 'Взаимосвязи и условия симметрии'),
--   ('Угол', 'java.lang.String', 'Взаимосвязи и условия симметрии'),
--   ('Сдвиг', 'java.lang.String', 'Взаимосвязи и условия симметрии'),
--   ('Основная группа', 'java.lang.String', 'Взаимосвязи и условия симметрии'),
--   ('Система координат', 'java.lang.String', 'Взаимосвязи и условия симметрии'),
--   ('Мастер', 'ru.crp.lvms.optimizations.toska.enums.Master', 'Взаимосвязи и условия симметрии');

-------------------------------------------------------------------------------------------------------

CREATE TABLE toska_technological_constraints_fields (
  id SERIAL,
  name VARCHAR(255),
  tech_constr_id INTEGER NOT NULL,

  FOREIGN KEY(tech_constr_id) REFERENCES toska_technological_constraints(id) ON DELETE CASCADE,
  PRIMARY KEY(id)
);

CREATE TABLE toska_technological_constraints_fields_values (
  id SERIAL,
  field_id INTEGER NOT NULL,
  tech_constr_id INTEGER NOT NULL,
  value VARCHAR(255),

  FOREIGN KEY(field_id) REFERENCES toska_technological_constraints_fields(id) ON DELETE CASCADE,
  FOREIGN KEY(tech_constr_id) REFERENCES toska_optimization_technological_constraints(id) ON DELETE CASCADE,
  PRIMARY KEY(id)
);

CREATE VIEW toska_technological_constraints_fields_join AS
  SELECT ttcfv.*, ttcf.name as field_name
  FROM toska_technological_constraints_fields ttcf
    JOIN toska_technological_constraints_fields_values ttcfv
      ON ttcfv.field_id = ttcf.id;

CREATE RULE toska_technological_constraints_fields_join_CREATE AS ON INSERT TO toska_technological_constraints_fields_join DO INSTEAD (
  INSERT INTO toska_technological_constraints_fields_values (field_id, tech_constr_id, value)
  VALUES (NEW.field_id, NEW.tech_constr_id, NEW.value)
  RETURNING
    toska_technological_constraints_fields_values.*,
    NULL :: VARCHAR(255) AS field_name
);

CREATE RULE toska_technological_constraints_fields_join_UPDATE AS ON UPDATE TO toska_technological_constraints_fields_join DO INSTEAD (
  UPDATE toska_technological_constraints_fields_values
  SET field_id = NEW.field_id, tech_constr_id = NEW.tech_constr_id, value = NEW.value
  WHERE id = NEW.id);

CREATE RULE toska_technological_constraints_fields_join_DELETE AS ON DELETE TO toska_technological_constraints_fields_join DO INSTEAD (
  DELETE FROM toska_technological_constraints_fields_values WHERE id = OLD.id
);

INSERT INTO toska_technological_constraints_fields(id, name, tech_constr_id) VALUES
  (1, 'THICKNESS', 3), (2, 'THICKNESS', 2), (3, 'MIN_THICKNESS', 2), (4, 'DISTANCE', 2),
  (5, 'THICKNESS', 4), (6, 'MIN_THICKNESS', 4), (7, 'DISTANCE', 4);
