CREATE TABLE toska_optimization_base_setting_types (
  id SERIAL,
  max_value INTEGER NOT NULL,
  display_name VARCHAR(255),

  PRIMARY KEY (id)
);

INSERT INTO toska_optimization_base_setting_types (max_value, display_name) VALUES
 (2, 'Количество целевых функций'),
 (8, 'Количество пространств оптимизации'),
 (4, 'Количество технологических ограничений'),
 (8, 'Количество ограничений');

CREATE TABLE toska_optimization_base_settings (
  id SERIAL,
  type_id INTEGER NOT NULL,
  optimization_id INTEGER NOT NULL,
  current_value INTEGER,

  PRIMARY KEY(id),
  FOREIGN KEY(optimization_id) REFERENCES optimizations(id) ON DELETE CASCADE,
  FOREIGN KEY(type_id) REFERENCES toska_optimization_base_setting_types(id),
  UNIQUE(optimization_id, type_id)
);
CREATE INDEX ON toska_optimization_base_settings(optimization_id);

CREATE VIEW toska_optimization_base_settings_join AS (
  SELECT obs.*,
  obst.max_value AS max_value,
  obst.display_name AS name

  FROM toska_optimization_base_settings obs
  JOIN toska_optimization_base_setting_types obst ON obs.type_id = obst.id
);

CREATE RULE toska_optimization_base_settings_join_CREATE AS ON INSERT TO toska_optimization_base_settings_join DO INSTEAD (
    INSERT INTO toska_optimization_base_settings (type_id, optimization_id, current_value)
     VALUES (NEW.type_id, NEW.optimization_id, NEW.current_value)
    RETURNING
      toska_optimization_base_settings.*,
      NULL :: INTEGER AS max_value,
      NULL :: VARCHAR(255) AS name
);

CREATE RULE toska_optimization_base_settings_join_UPDATE AS ON UPDATE TO toska_optimization_base_settings_join DO INSTEAD (
  UPDATE toska_optimization_base_settings
  SET current_value = NEW.current_value
  WHERE id = NEW.id);

CREATE RULE toska_optimization_base_settings_join_DELETE AS ON DELETE TO toska_optimization_base_settings_join DO INSTEAD (
  DELETE FROM toska_optimization_base_settings WHERE id = OLD.id
);
