CREATE TABLE technologies (
  id               INTEGER DEFAULT nextval('tree_seq' :: REGCLASS),
  pid              INTEGER DEFAULT NULL,
  name             VARCHAR UNIQUE                             NOT NULL,
  owner_id         INTEGER                                    NOT NULL,
  path             VARCHAR [], --NOT NULL,--
  path_names       VARCHAR UNIQUE, --NOT NULL,--
  date_created     TIMESTAMP WITHOUT TIME ZONE DEFAULT now()  NOT NULL,
  date_modified    TIMESTAMP WITHOUT TIME ZONE DEFAULT now()  NOT NULL,
  lock             BOOLEAN DEFAULT FALSE,
  video_url        VARCHAR,
  description_file_id INTEGER,
  image_file_id       INTEGER,

  PRIMARY KEY (id),
  FOREIGN KEY (owner_id) REFERENCES users (id),
  FOREIGN KEY (description_file_id) REFERENCES files (id),
  FOREIGN KEY (image_file_id) REFERENCES files (id)
);


CREATE TRIGGER technologies_path_create
BEFORE INSERT ON technologies
FOR EACH ROW
EXECUTE PROCEDURE create_path('Technology');

CREATE TRIGGER technologies_path_update
BEFORE UPDATE ON technologies
FOR EACH ROW
WHEN (OLD.name IS DISTINCT FROM NEW.name)
EXECUTE PROCEDURE update_path('Technology');



CREATE VIEW technologies_join AS
  SELECT
    tech.*,
    descr.name         AS descr_name,
    descr.path_to_file AS descr_source,
    image.name         AS image_name,
    image.path_to_file AS image_source
  FROM technologies tech
    LEFT OUTER JOIN files descr
      ON tech.description_file_id = descr.id
    LEFT OUTER JOIN files image
      ON tech.image_file_id = image.id;


CREATE TABLE technology_parameters (
  id              SERIAL,
  technology_id   INTEGER NOT NULL,
  parameter_name  VARCHAR NOT NULL,
  parameter_value VARCHAR NOT NULL,

  PRIMARY KEY (id),
  FOREIGN KEY (technology_id) REFERENCES technologies (id),
  UNIQUE (technology_id, parameter_name)
);


CREATE TABLE possible_defects (
  id              SERIAL,
  technology_id   INTEGER NOT NULL,
  defect_name     VARCHAR NOT NULL,
  defect_value    VARCHAR NOT NULL,
  act_name        VARCHAR NOT NULL,

  PRIMARY KEY (id),
  FOREIGN KEY (technology_id) REFERENCES technologies (id),
  UNIQUE (technology_id, defect_name)
);
