CREATE TABLE toska_optimization_type_tf (
  id SERIAL,
  opt_type_id INTEGER,
  parameter_id INTEGER,
  is_tf BOOLEAN,
  is_costraint BOOLEAN,

  FOREIGN KEY(parameter_id) REFERENCES toska_target_function_parameters(id) ON DELETE CASCADE,
  FOREIGN KEY(opt_type_id) REFERENCES optimization_types(id) ON DELETE CASCADE
);

-- TOPOLOGICAL

INSERT INTO toska_optimization_type_tf (opt_type_id, parameter_id, is_tf, is_costraint)
  SELECT 2, id, TRUE, TRUE FROM toska_target_function_parameters;

UPDATE toska_optimization_type_tf SET is_costraint = FALSE
FROM toska_target_function_parameters ttfp
WHERE ttfp.id = toska_optimization_type_tf.parameter_id AND ttfp.dresp_type = 'DYN_FREQ_KREISSEL';

-- TOPOGRAPHIC

INSERT INTO toska_optimization_type_tf (opt_type_id, parameter_id, is_tf, is_costraint)
  SELECT 3, parameter_id, is_tf, is_costraint from toska_optimization_type_tf;

INSERT INTO toska_target_function_parameters (id, name, dresp_type, node_or_element_id, group_name) VALUES
  (77,'Высота рельефа','BEAD_HEIGHT',2,'Спектральный и акустический анализ');

INSERT INTO toska_optimization_type_tf (opt_type_id, parameter_id, is_tf, is_costraint) VALUES
  (3, 77, FALSE, TRUE);

-- SHAPE

INSERT INTO toska_target_function_parameters (id, name, dresp_type, node_or_element_id, group_name) VALUES
  (78,'Эффективная пластическая деформация (ABAQUS)','ABQ_ND_PEEQ',2,'Базовые'),
  (79,'Поврежденность','DAMAGE',2,'Базовые'),
  (80,'Поврежденность (LC)','DAMAGE_LC',2,'Базовые'),
  (81,'Плотность энергии деформации','STRAIN_ENERGY_DENS',4,'Базовые'),
  (82,'1-ое главное напряжение','SIG_1',4,'Напряжения'),
  (83,'2-ое главное напряжение','SIG_2',4,'Напряжения'),
  (84,'3-ее главное напряжение','SIG_3',4,'Напряжения'),
  (85,'Нормальное напряжение (11)','SIG_11',4,'Напряжения'),
  (86,'Нормальное напряжение (22)','SIG_22',4,'Напряжения'),
  (87,'Нормальное напряжение (33)','SIG_33',4,'Напряжения'),
  (88,'Касательное напряжение (12)','SIG_12',4,'Напряжения'),
  (89,'Касательное напряжение (12)','SIG_23',4,'Напряжения'),
  (90,'Касательное напряжение (12)','SIG_13',4,'Напряжения'),
  (91,'Максимальное по модулю главное напряжение','SIG_ABS_123',4,'Напряжения'),
  (92,'Модуль 3-го главного напряжения','SIG_ABS_3',4,'Напряжения'),
  (93,'Норма Бельтрами','SIG_BELTRAMI',4,'Напряжения'),
  (94,'Норма Друкера-Прагера','SIG_DRUCKER_PRAGER',4,'Напряжения'),
  (95,'Норма Галилея','SIG_ GALILEI',4,'Напряжения'),
  (96,'Норма Куна','SIG_KUHN',4,'Напряжения'),
  (97,'Норма Мариотта','SIG_MARIOTTE',4,'Напряжения'),
  (98,'Норма Мизеса','SIG_MISES',4,'Напряжения'),
  (99,'Норма Сандлера','SIG_SANDLER',4,'Напряжения'),
  (100,'Норма Сотера','SIG_SAUTER',4,'Напряжения'),
  (101,'Норма Треска','SIG_TRESCA',4,'Напряжения'),
  (102,'Нормальные напряжения в области контакта','SIG_CONTACT_PRESSURE',2,'Напряжения'),
  (103,'Сдвиговые напряжения в области контакта','SIG_CONTACT_SHEAR',2,'Напряжения'),
  (104,'Сдвиговые напряжения в области контакта (X)','SIG_CONTACT_SHEAR_X',2,'Напряжения'),
  (105,'Сдвиговые напряжения в области контакта (Y)','SIG_CONTACT_SHEAR_Y',2,'Напряжения'),
  (106,'Суммарные напряжения в области контака','SIG_CONTACT_TOTAL',2,'Напряжения'),
  (107,'Упругие деформации','STRAIN_ELASTIC',2,'Деформации'),
  (108,'Пластические деформации','STRAIN_PLASTIC',2,'Деформации'),
  (109,'Суммарные деформации','STRAIN_TOTAL',2,'Деформации')
;

INSERT INTO toska_optimization_type_tf (opt_type_id, parameter_id, is_tf, is_costraint)
  WITH RECURSIVE t(opt_type_id, parameter_id, is_tf, is_costraint) AS (
    VALUES (7, 78, TRUE, FALSE)
    UNION ALL
    SELECT
      opt_type_id,
      (parameter_id + 1),
      is_tf,
      is_costraint
    FROM t
    WHERE parameter_id < 109
  ) SELECT *
    FROM t;

-- VIEWS UPDATE

DROP RULE toska_constraints_join_CREATE ON toska_constraints_join;
DROP RULE toska_constraints_join_UPDATE ON toska_constraints_join;
DROP RULE toska_constraints_join_DELETE ON toska_constraints_join;
DROP VIEW toska_constraints_join;
DROP VIEW toska_constraints_parameters;

CREATE VIEW toska_constraints_join AS
  SELECT tc.*, ttfp.name as param_name, ttfp.dresp_type, ttfp.node_or_element_id
  FROM toska_optimization_constraints tc
    JOIN toska_target_function_parameters ttfp
      ON ttfp.id = tc.parameter_id;

CREATE RULE toska_constraints_join_CREATE AS ON INSERT TO toska_constraints_join DO INSTEAD (
  INSERT INTO toska_optimization_constraints (name, parameter_id, optimization_id, type_id, el_group, nd_group, scalar, value, isAbsolute)
  VALUES (NEW.name, NEW.parameter_id, NEW.optimization_id, NEW.type_id, NEW.el_group, NEW.nd_group, NEW.scalar, NEW.value, NEW.isAbsolute)
  RETURNING
    toska_optimization_constraints.*,
    NULL :: VARCHAR(255) AS param_name,
    NULL :: VARCHAR(255) AS dresp_type,
    NULL :: INTEGER AS node_or_element_id
);

CREATE RULE toska_constraints_join_UPDATE AS ON UPDATE TO toska_constraints_join DO INSTEAD (
  UPDATE toska_optimization_constraints
  SET name = NEW.name, parameter_id = NEW.parameter_id, optimization_id = NEW.optimization_id,
    type_id = NEW.type_id, el_group = NEW.el_group, nd_group = NEW.nd_group, scalar = NEW.scalar,
    value = NEW.value, isAbsolute = NEW.isAbsolute
  WHERE id = NEW.id);

CREATE RULE toska_constraints_join_DELETE AS ON DELETE TO toska_constraints_join DO INSTEAD (
  DELETE FROM toska_constraints_join WHERE id = OLD.id
);
