/*global Backbone */

(function () {
  "use strict";
  $.ajaxSetup({cache: false});
  EJS.config({cache: true});

  if (alertify.defaults && alertify.defaults.glossary) {
    alertify.defaults.glossary.ok = "Сохранить";
    alertify.defaults.glossary.cancel = "Отменить";
  }

  window.socketURL = "ws://" + window.location.host + "/space/rest/ws";

  String.prototype.capitalize = function() {
    return this.charAt(0).toUpperCase() + this.slice(1);
  };

  String.prototype.replaceBetween = function(start, end, what) {
    return this.substring(0, start) + what + this.substring(end);
  };

  Backbone.Marionette.Region.prototype.attachHtml = function (view) {
    this.$el.html(view.el);
    view.showEffect ? view.showEffect() : "";
  };

  Backbone.Marionette.Renderer.render = function (template, data) {
    if (!template) throw "Template '" + template + "' not found";

    if (typeof template === "function") {
      return template(data);
    }
    return new EJS({
      url: template
    }).render(data);
  };

  var SPACE = new Backbone.Marionette.Application();

  Marionette.Behaviors.behaviorsLookup = function () {
    return window.App.Behaviors;
  };


  SPACE.routeParams = {};
  SPACE.param = function (fragment, params) {

    var matchedRoute;
    _.any(Backbone.history.handlers, function (handler) {
      if (handler.route.test(fragment)) {
        matchedRoute = handler.route;
      }
    });
    if (params !== undefined) {
      SPACE.routeParams[fragment] = params;
    }
    return SPACE.routeParams[fragment];

  };

  // pass params through the App.navigate
  SPACE.navigate = function (route, options) {
    var routeOption = {
      trigger: true
    }, params = (options && options.params) ? options.params : null;

    routeOption = _.extend(routeOption, options);

    delete routeOption.params;

    this.param(route, params);
    Backbone.history.navigate(route, routeOption);
  };

  SPACE.getCurrentRoute = function () {
    return Backbone.history.fragment;
  };

  SPACE.addRegions({
    mainAppHeaderRegion: '#header',
    mainAppContentRegion: '#content-wrapper',
    modalAppWindowRegion: "#modal-window-region"
  });

  SPACE.addInitializer(function (options) {

    SPACE.appController = new window.App.Controllers.AppController(options);
    SPACE.router = new window.App.Routers.AppRouter({
      controller: SPACE.appController
    });

    SPACE.projectNavigationController = new window.App.Controllers.ProjectNavigationController(options);
    SPACE.projectRouter = new window.App.Routers.ProjectRouter({
      controller: SPACE.projectNavigationController
    });

    SPACE.adminNavigationController = new window.App.Controllers.AdminNavigationController(options);
    SPACE.adminRouter = new window.App.Routers.AdminRouter({
      controller: SPACE.adminNavigationController
    });

    Backbone.history.start();
  });

  window.Methods = {};

  window.WebSocketBus = function () {
    this.socket = new WebSocket(window.socketURL);
    this.stompClient = Stomp.over(this.socket);
    this.stompClient.debug = null;
    this.wsClientId = this.stompClient.connect();
  };

  // Initialize WebSocketBus
  SPACE.webSocketBus = new window.WebSocketBus();

  // Creates modules
  SPACE.module("Behaviors");
  SPACE.module("Views");
  SPACE.module("BackgridFormatters");
  SPACE.module("Models");
  SPACE.module("Collections");
  SPACE.module("Controllers");
  SPACE.module("Routers");
  SPACE.module("Utils");


  SPACE.Data = {};
  SPACE.Data.UserProfile = {};
  SPACE.Data.Language = 'ru';

  SPACE.Data.Trees = {
    0: {type: 'materials', name: 'Materials'},
    1: {type: 'technologies', name: 'Technologies'},
    2: {type: 'constructionElements', name: 'constructionElements'},
    3: {type: 'optimizations', name: 'Optimizations'},
    4: {type: 'compositeMaterials', name: 'Composite Materials'}
  };

  SPACE.Data.EntityTypes = {
    0: {type: 'Folder', name: "Folder"},
    1: {type: "MaterialFile", name: "Material File"},
    2: {type: "Material", name: "Material"},
    3: {type: "CompositeMaterial", name: "Composite Material"},
    4: {type: "File", name: "File"},
    5: {type: "Technology", name: "Technology"},
    6: {type: "ConstructionElement", name: "ConstructionElement"},
    7: {type: "ConstructionElementFolder", name: "Construction Element Folder"},
    8: {type: "OptimizationFolder", name: "Optimization Folder"},
    9: {type: "optimization", name: "Optimization"}
  };

  SPACE.Data.EntityTypesRU = {
    0: {type: 'Folder', name: "Директория"},
    1: {type: "MaterialFile", name: "Файл материала"},
    2: {type: "Material", name: "Материал"},
    3: {type: "CompositeMaterial", name: "Композиционный материал"},
    4: {type: "File", name: "Файл"},
    5: {type: "Technology", name: "Технология"},
    6: {type: "ConstructionElement", name: "Элемент конструкции"},
    7: {type: "ConstructionElementFolder", name: "Директория элементов конструкций"},
    8: {type: "OptimizationFolder", name: "Директория расчетов"},
    9: {type: "optimization", name: "Оптимизация"}
  };


  var appDictionary = {
    ru: {
      actions: 'Действия',
      edit: 'Редактирование',
      remove:'Удалить',
      add: 'Добавить',
      addThisLevel: 'Создать',
      addChildLevel: 'Создать в ...',
      addNewItem: "Создать новый элемент ",
      enterName: "Введите имя"
    },
    en: {
      actions: 'Actions',
      edit: 'Edit',
      remove: 'Remove',
      add: "New",
      addThisLevel: 'Add This Level',
      addChildLevel: 'Add Child Level',
      addNewItem: "ADD NEW ITEM",
      enterName: "Enter name"
    }

  };

  SPACE.Data.Dictionary = appDictionary[SPACE.Data.Language];


  SPACE.Data.TreeContextMenuItems = {
    actions: SPACE.Data.Dictionary.actions,
    edit: SPACE.Data.Dictionary.edit,
    remove: SPACE.Data.Dictionary.remove,
    add: SPACE.Data.Dictionary.add,
    addThisLevel: SPACE.Data.Dictionary.addThisLevel,
    addChildLevel: SPACE.Data.Dictionary.addChildLevel
  };

  SPACE.Data.EntityTypes = SPACE.Data.EntityTypesRU;

  // Set global object
  _.extend(window.App || {}, SPACE);

})();