(function () {

  window.App = {};
  window.App.Helpers = {};

  window.alertify.defaults.transition = "zoom";
  window.alertify.defaults.theme.ok = "btn btn-primary";
  window.alertify.defaults.theme.cancel = "btn btn-danger";
  window.alertify.defaults.theme.input = "form-control";

  window.App.Helpers.getParamFromHash = function getParmFromHash(param, encoded, url) {
    url = url || window.location.hash;
    var re = new RegExp("#.*[?&]" + param + "=([^&]+)(&|$)");
    var match = url.match(re);
    return (match ? encoded?decodeURIComponent(match[1]):match[1] : "");
  };

  window.App.Helpers.getParmsFromURLHash = function getParmsFromURLHash(url) {
    if (_.isEmpty(url)) {
      return
    }
    var parms = {}, pieces, parts, i;
    var hash = url.lastIndexOf("#");
    if (hash !== -1) {
      // isolate just the hash value
      url = url.slice(hash + 1);
    }
    var question = url.indexOf("?");
    if (question !== -1) {
      url = url.slice(question + 1);
      pieces = url.split("&");
      for (i = 0; i < pieces.length; i++) {
        parts = pieces[i].split("=");
        if (parts.length < 2) {
          parts.push("");
        }
        parms[decodeURIComponent(parts[0])] = decodeURIComponent(parts[1]);
      }
    }
    return parms;
  };

  window.App.Helpers.ValidateUrl = function learnRegExp(s) {
    var regexp = /(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/;

    var re_weburl = new RegExp(
      "^" +
      // protocol identifier
      "(?:(?:https?|ftp)://)" +
      // user:pass authentication
      "(?:\\S+(?::\\S*)?@)?" +
      "(?:" +
      // IP address exclusion
      // private & local networks
      "(?!(?:10|127)(?:\\.\\d{1,3}){3})" +
      "(?!(?:169\\.254|192\\.168)(?:\\.\\d{1,3}){2})" +
      "(?!172\\.(?:1[6-9]|2\\d|3[0-1])(?:\\.\\d{1,3}){2})" +
      // IP address dotted notation octets
      // excludes loopback network 0.0.0.0
      // excludes reserved space >= 224.0.0.0
      // excludes network & broacast addresses
      // (first & last IP address of each class)
      "(?:[1-9]\\d?|1\\d\\d|2[01]\\d|22[0-3])" +
      "(?:\\.(?:1?\\d{1,2}|2[0-4]\\d|25[0-5])){2}" +
      "(?:\\.(?:[1-9]\\d?|1\\d\\d|2[0-4]\\d|25[0-4]))" +
      "|" +
      // host name
      "(?:(?:[a-z\\u00a1-\\uffff0-9]-*)*[a-z\\u00a1-\\uffff0-9]+)" +
      // domain name
      "(?:\\.(?:[a-z\\u00a1-\\uffff0-9]-*)*[a-z\\u00a1-\\uffff0-9]+)*" +
      // TLD identifier
      "(?:\\.(?:[a-z\\u00a1-\\uffff]{2,}))" +
      ")" +
      // port number
      "(?::\\d{2,5})?" +
      // resource path
      "(?:/\\S*)?" +
      "$", "i"
    );

    return regexp.test(s);
  };

  window.App.Helpers.HandleErrorCodes = function (callback) {

    function showErrorMessage(xhr, status, error) {
      var errorMessage = xhr.responseJSON && xhr.responseJSON.message ? xhr.responseJSON.message : error;
      console.error("Error ", errorMessage);
      window.alertify.error(errorMessage);
    }

    jQuery.ajaxSetup({
      statusCode: {
        401: function (xhr, status, error) {
          // Redirect the to the login page.
          showErrorMessage(xhr, status, error);
          window.location.replace('login.jsp');
        },
        400: function (xhr, status, error) {
          console.error("app_helpers.js xhr, exception, status ", xhr, status, error)
          showErrorMessage(xhr, status, error);
        },
        403: function (xhr, status, error) {
          showErrorMessage(xhr, status, error);
        },
        409: function (xhr, status, error) {
          showErrorMessage(xhr, status, error);
        },
        500: function (xhr, status, error) {
          showErrorMessage(xhr, status, error);
        }
      }
    });
    typeof callback === "function" ? callback() : null;

  };

  window.App.Helpers.ParseNodeStatus = function (status) {
    var parsedStatus;
    switch (status) {
      case "Free":
        parsedStatus = "Свободен";
        break;
      case "Sending":
        parsedStatus = "Копирование данных на расчетную машину";
        break;
      case "Copying data":
        parsedStatus = "Копирование данных на расчетную машину";
        break;
      case "Starting":
        parsedStatus = "Копирование данных на расчетную машину";
        break;
      case "Solving":
        parsedStatus = "Расчет";
        break;
      case "CalculationFinished":
        parsedStatus = "Расчет завершен";
        break;
      case "ImportingResults":
        parsedStatus = "Импорт результатов";
        break;
      case "Terminating":
        parsedStatus = "Завершение расчета";
        break;
      default:
        parsedStatus = status;
    }
    return parsedStatus;
  };

  window.App.Helpers.ParseObjectStatus = function (status) {
    var parsedStatus;
    switch (status) {
      case "Created":
        parsedStatus = "Создан";
        break;
      case "Processing":
        parsedStatus = "Идет расчет";
        break;
      case "Failed":
        parsedStatus = "Завершен с ошибкой";
        break;
      case "Completed":
        parsedStatus = "Завершен успешно";
        break;
      case "Queued":
        parsedStatus = "В очереди";
        break;
      default:
        parsedStatus = status;
    }
    return parsedStatus;
  };

  window.App.Helpers.ShowPath = function (pathArr, type) {
    pathArr = pathArr || [];

    if (!pathArr.length) return;

    switch (type) {
      case 'dir_id':
        var id = pathArr.length > 1 ? +pathArr[pathArr.length - 2].id : pathArr[0].id;
        switch (_.last(pathArr).type) {
          case window.App.Data.EntityTypes[2].type:
            return id;
          case window.App.Data.EntityTypes[4].type:
            return id;
          case window.App.Data.EntityTypes[5].type:
            return +_.last(pathArr).id;
          default :
            return +_.last(pathArr).id;
        }
      case 'dir':
        switch (_.last(pathArr).type) {
          case window.App.Data.EntityTypes[2].type:
            return '/' + _.pluck(pathArr.slice(0, pathArr.length - 1), 'name').join('/') + '/'
          case window.App.Data.EntityTypes[4].type:
            return '/' + _.pluck(pathArr.slice(0, pathArr.length - 1), 'name').join('/') + '/'
          case window.App.Data.EntityTypes[5].type:
            return '/' + _.pluck(pathArr.slice(0, pathArr.length - 1), 'name').join('/')
          default:
            console.warn('Nothing matched, default path');
            return '/' + _.pluck(pathArr.slice(0, pathArr.length), 'name').join('/') + '/'
        }
        break;
      case 'dir_root':
        var pathCopy = _.clone(pathArr); // Prevent calling model "change" listeners
        if (_.last(pathCopy).type !== "Folder") {
          pathCopy.splice(pathCopy.length - 1, 1);
        }
        return '/' + _.pluck(pathCopy, "name").join("/");
      case 'filename':
        return _.last(pathArr).name;
      case 'filetype':
        return _.last(pathArr).type;
      case 'from_root':
        return _.pluck(pathArr, 'name').join('/');
      default  : //full
        return "/" + _.pluck(pathArr, 'name').join('/');
    }
  };

  window.App.Helpers.HandleErrorCodes();

})(jQuery);