/**
 * Created by lexeek on 12.01.15.
 */
(function () {
  "use strict";

  window.App.Behaviors.CommonEntityPageActions = Marionette.Behavior.extend({
    events: {
      "click @ui.flagBtn": "onClickFlag",
      "click @ui.editBtn": "onClickEdit",
      "click @ui.saveBtn": "onClickSave",
      "click @ui.deleteBtn": "onClickDelete",
      "click @ui.copyToClipboardBtn": "onClickCopyToClipboard",
      "keyup input, textarea": "onChangeField"
    },

    onClickFlag: function (e) {
      e.preventDefault();
      var flagData = $(e.target).attr('data-flag');

      if (this.view.model.get('flag') !== flagData) {
        this.toggleSaveButton(true);
      }

      this.view.model.set('flag', flagData);
    },

    setFlagActive: function () {
      this.$('.flag-' + this.view.model.get('flag')).addClass('active');
    },

    onRender: function () {
      var that = this;
      this.setFlagActive();
      this.showEditMode();
      this.view.on("reference:update", function() {
        that.toggleSaveButton(true);
      });
    },

    onDestroy: function() { this.view.off("reference:update"); },

    onClickSave: function () {
      var that = this;
      this.view.model.save().success(function (data) {
        window.alertify.success("Saved: " + data.name);
        that.view.editMode = !that.view.editMode;
        window.App.vent.trigger('updateTreeNodes', {id: that.view.model.get('pid')}, 'submodels', null, "add");
        window.App.vent.trigger('updateTreeNodes', {id: that.view.oldPid}, 'submodels', null, "add");
        that.view.render();
      });
    },

    onClickCopyToClipboard: function () {
      window.clipboard.push(this.view.model.name, [this.view.model.id]);
    },

    onClickDelete: function (e) {
      e.preventDefault();
      var that = this;

      function onDeleteOk() {
        that.view.model.destroy({
          success: function () {
            window.App.navigate("project/" + that.view.model.name + "s", {trigger: true});
          }
        });
      }

      var entityType = window.App.Helpers.ShowPath(this.view.model.get("path"), 'filetype');
      var message = entityType ? "Are you sure you want to delete " + entityType + "<b><br>" : "Are you sure you want to delete?;";
      window.alertify.confirm("Warning!", "<h4>" + message + "</h4>" + this.view.model.get('name') + '</b>', onDeleteOk, null)
        .set('movable', false);
    },

    toggleSaveButton: function (enabled) {
      if (enabled) {
        this.view.ui.saveBtn.removeClass("hidden", 300).addClass('edited');
      } else {
        this.view.ui.saveBtn.addClass("hidden").removeClass('edited');
      }
    },

    onChangeField: function (e) {
      var $target = $(e.target);
      if (!$target.attr("name")) { return; }

      this.view.model.set($target.attr("name"), $target.val());
      this.toggleSaveButton(true);
    },

    showEditMode: function () {
      var $editBtn = this.view.ui.editBtn;

      var entityType = window.App.Helpers.ShowPath(this.view.model.get("path"), 'filetype');
      var stillDisabled = "";

      if (entityType === window.App.Data.EntityTypes[5].type) {
        stillDisabled = this.view.model.get('hasRelations') ? ",#js-name,#js-reference,#js-path,.js-path,#js-description" : "";
      }

      if (entityType === window.App.Data.EntityTypes[4].type) {
        stillDisabled = this.view.model.get('hasRelations') ? ",#js-name,#js-reference,#js-path,.js-path,#js-description" : "";
      }

      if (this.view.model.keys().length > 1 && this.view.editMode) {
        $editBtn.addClass('editing').text("Отменить");
        this.$('[disabled]:not(#js-image-upload.sessionfile,#js-date-modified,#js-date-created,#js-owner)')
          .attr("disabled", !this.view.editMode)
          .addClass('edit')
          .removeClass('disabled')
          .removeClass('hidden')
          .removeClass('disabled-lnk');

        var newName = this.view.options.name || null;
        if (newName) {
          this.view.ui.nameInput.parents('.form-group').addClass('has-success')
          this.view.ui.saveBtn.addClass('edited');
          this.view.ui.nameInput.val(newName)
          this.view.model.set('name', newName);
        }

        if (this.view.ui.saveBtn.hasClass("edited")) {
          this.toggleSaveButton(true);
        }

        this.view.ui.referenceLnk.removeClass("disabled");
      }
    },

    onClickEdit: function (e) {
      var $editBtn = this.view.ui.editBtn;

      if ($editBtn.hasClass('editing')) {
        $editBtn.removeClass('editing');
        this.view.editMode = false;
        this.view.model.fetch().done(function () {
          window.location.href = window.location.href.split('?')[0];
        });
      } else {
        this.view.editMode = true;
        this.showEditMode();
      }
    }
  })
})();