(function () {
  "use strict";

  window.App.Helpers.CurveView = {
    mapDescription: function (descriptionArray) {
      return _.map(descriptionArray, function (string) {
        return '<div>' + string + '</div>';
      })
    },

    mapCharts: function (curve) {
      let that = this;
      let charts = {
        data: {
          columns: [],
          xs: {},
          types: {}
        },
        regions: [],
        grid: {
          x: {
            lines: []
          },
          y: {
            lines: []
          }
        }
      };
      let iterator = 0;
      _.each(curve.charts, function (chart, i) {
        let xArray = [], yArray = [];
        switch (chart.type) {
          case "CURVE":
            xArray = xArray.concat(_.pluck(chart.points, 'x'));
            yArray = yArray.concat(_.pluck(chart.points, 'y'));
            xArray.unshift('x' + iterator);
            yArray.unshift('data' + iterator);
            charts.data.xs['data' + iterator] = 'x' + iterator;
            charts.data.types['data' + iterator] = that.mapCurveType(chart.type);
            charts.data.columns[iterator] = xArray;
            charts.data.columns[iterator + 1] = yArray;
            charts['data' + iterator] = {};

            if (!_.isEmpty(chart.borders.x.lines)) {
              charts.grid['x'].lines = charts.grid['x'].lines.concat(chart.borders.x.lines);
            }
            if (!_.isEmpty(chart.borders.y.lines)) {
              charts.grid['y'].lines = charts.grid['y'].lines.concat(chart.borders.y.lines);
            }
            iterator = iterator + 2;
            break;
          case "DOTS":
            xArray = xArray.concat(_.pluck(chart.points, 'x'));
            yArray = yArray.concat(_.pluck(chart.points, 'y'));
            xArray.unshift('x' + iterator);
            yArray.unshift('data' + iterator);
            charts.data.xs['data' + iterator] = 'x' + iterator;
            charts.data.types['data' + iterator] = that.mapCurveType(chart.type);
            charts.data.columns[iterator] = xArray;
            charts.data.columns[iterator + 1] = yArray;
            charts['data' + iterator] = {};


            if (!_.isEmpty(chart.borders.x.lines)) {
              charts.grid['x'].lines = charts.grid['x'].lines.concat(chart.borders.x.lines);
            }
            if (!_.isEmpty(chart.borders.y.lines)) {
              charts.grid['y'].lines = charts.grid['y'].lines.concat(chart.borders.y.lines);
            }
            iterator = iterator + 2;
            break;
          case "CONST_Y":
            charts.grid['y'].lines.push({
              value: chart.value,
              text: 'Y: ' + chart.value
            });
            break;
          case "CONST_X":
            charts.grid['x'].lines.push({
              value: chart.value,
              text: 'X: ' + chart.value
            });
            break;
        }

      });
      return charts;
    },

    mapCurveType: function (type) {
      let curveType;
      switch (type) {
        case "CURVE":
          curveType = "spline";
          break;
        case "CONST_Y":
          curveType = "step";
          break;
        case "CONST_X":
          curveType = "scatter";
          break;
        case "DOTS":
          curveType = "scatter";
      }
      return curveType;
    },

    showCurve: function (curve) {
      let that = this;
      let charts = this.mapCharts(curve);

      charts.data['color'] = function (color, d) {
        let updatedColor = color;
        if (d.id) {
          let chart = curve.charts[d.id.replace("data", "")];
          if (!_.isUndefined(d.index)) {
            let type = chart.points[d.index].type;
            switch (type) {
              case "COMMON":
                updatedColor = color;
                break;
              case "IMPROPER":
                updatedColor = "#BD4E46";
                break;
              case "SUPERIOR":
                updatedColor = "#408F62";
                break;
              default:
                updatedColor = color;
            }
          }
        }

        return updatedColor;
      };

      let description = _.isNull(curve.description) ? "Отсутствует" : this.mapDescription(curve.description);
      $(".js-description").html(description);

      let chart = c3.generate({
        bindto: '.js-curves',
        data: charts.data,
        zoom: {
          enabled: true
        },
        grid: charts.grid,
        point: {
          r: function (d) {
            let r = 2;
            if (!_.isUndefined(d.index)) {
              let chart = curve.charts[d.id.replace("data", "")];
              if (chart.points[d.index].type === "SUPERIOR") {
                r = 3;
              }
            }
            return r;
          }
        },
        line: {
          show: false
        },
        axis: {
          x: {
            label: curve ? (curve.xAxisLabel + ', ' + curve.xDimension) : "",
            tick: {culling: true, format: d3.format(that.getFormat(curve.domainX)), fit: false}
          },
          y: {
            label: curve ? (curve.yAxisLabel + ', ' + curve.yDimension) : "",
            tick: {culling: true, format: d3.format(that.getFormat(curve.domainY))}
          }
        },
        tooltip: {
          format: {
            title: function (d) {
              return 'X: ' + d;
            },
            name: function (name) {
              return name.capitalize();
            },
            value: function (value, ratio, id) {
              return d3.format('.4r')(value);
            }
          }
        },
        legend: {
          show: false
        }
      });

    },

    getFormat: function (domain) {
      let format = '.0e';
      switch (domain) {
        case "Integer":
          format = ',.0f';
          break;
        case "Real":
          format = '.0e';
          break;
        default:
          format = null;
      }
      return format;
    }
  }


})();