/**
 * Created by lexeek on 19.09.14.
 */
(function () {
  "use strict";

  var MyDatePickerCellEditor = Backgrid.InputCellEditor.extend({
    events: {},
    initialize: function () {
      Backgrid.InputCellEditor.prototype.initialize.apply(this, arguments);
      var input = this;

      this.$el.datepicker({
        autoclose: true,
        format: "yyyy-mm-dd",
        language: 'ru'
      }).on("changeDate", function (e, newdate) {
        moment().localeData("ru");
        var command = new Backgrid.Command({});
        input.model.set(input.column.get("name"), moment(moment(e.date).format()).valueOf());
        input.model.trigger("backgrid:edited", input.model, input.column, command);
        command = input = null;
      }).on("hide", function (e) {
        var command = new Backgrid.Command({});
        input.model.trigger("backgrid:edited", input.model, input.column, command);
      });

    },

    onShow: function () {

    },
    onRender: function (e, data) {},

    saveOrCancel: function (e) {
      console.log("backgrid_core.js saveOrCancel ", e)
    }
  });
  var MyDatePickerCell = Backgrid.Extension.MomentCell.extend({
    className: function () {
      return Backgrid.Extension.MomentCell.prototype.className + " " + "datepicker"
    },
    modelInUnixOffset: true,
    displayLang: "ru",
    modelLang: "ru",
    modelFormat: "HH:mm:ss.SS",
    displayFormat: "DD.MM.YYYY HH:mm",
    editor: MyDatePickerCellEditor
  });
  var ImageCell = Backgrid.ImageCell = Backgrid.Cell.extend({
    className: "image"
  });

  var ClickableRow = Backgrid.Row.extend({
    attributes: {
      "data-toggle": "context"
    },
    events: {
      "contextmenu": "onClick"
    },

    initialize: function (options) {
      this.listenTo(this.model, "backgrid:selected", this.triggerSelect);
      Backgrid.Row.prototype.initialize.call(this, options);
    },

    triggerSelect: function (model) {
      this.model.collection.trigger("backgrid:row:selected", model);
    },

    className: function () {
      switch (this.model.get("type")) {
        case "Solving only":
          return "row-green";
          break;
        case "Solving and postprocessing":
          return "row-brown";
          break;
        case "Postprocessing only":
          return "row-yellow";
          break;
        case "Data Check":
          return "row-blue";
          break;
        default:
          return "";
          break;
      }
    },
    onClick: function (e) {
      window.App.trigger("contextmenu:rowclicked", this, e);
    }
  });

  window.App.on("contextmenu:rowclicked", function (row, event) {
    var modelName;

    $(event.currentTarget).contextmenu({
      before: function (e, context) {
        var dropdownEl = '.dropdown-menu';
        try {
          modelName = row.model.getName ? row.model.getName() : "noname";
        } catch (e) {
          console.error(e.message);
        } finally {
          if ($('#' + row.$el.parents("table").data("menu-id")).has(dropdownEl).length === 0) {
            throw new Error("No context menu template:  " + modelName.toUpperCase());
          } else {
            var $contextMenu = $('#' + row.$el.parents("table").data("menu-id")).find(dropdownEl);
            row.model.collection.trigger("contextmenu:show", {name: modelName, model: row.model, $target: $contextMenu});
          }
          return !!modelName;
        }
      },

      onItem: function (context, e) {
        var shoudNotBeClosed = false;
        e.preventDefault();
        e.stopPropagation();

        function notImplemented() {
          alert("Not implemented yet");
        }

        switch ($(e.target).attr('id')) {
          case "item-download-tcl":
            var name = row.model.get("name");
            name = name.indexOf(".tcl") !== -1 ? name.replace(".tcl", "") : name;
            window.open("rest/tcls/" + name, '_self');
            break;
          case "item-delete-tcl":
            var c = confirm("Do you really want to remove this TCL Script?");
            if (c) {
              $.ajax({
                type: "DELETE",
                url: "rest/tcls/" + row.model.id,
                success: function () {
                  window.alertify.success("TCL Script was successfully removed.");
                  row.model.collection.fetch();
                }
              });
            }
            break;
          case "menu-item-select":
            row.model.trigger("backgrid:select", row.model, true);
            break;
          case "menu-item-open":
            switch (modelName) {
              case "material":
                window.open("#project/materials/" + (model.get("id") ? model.get("id") : "") + "/parameters/view");
                break;
              default:
                window.open("#project" + "/" + modelName + "/" + row.model.id + "/show/", '_self');
                break;
            }
            break;
          case "menu-item-compare":
            shoudNotBeClosed = true;
            if (!$(e.target).hasClass("disabled-lnk")) {
              if (modelName !== "simulation" && modelName !== "submodel" && modelName !== "sessionfile") {
                notImplemented();
                break;
              }
              row.model.collection.trigger("contextmenu:selected:compare", modelName)
            }
            break;
          case "menu-item-taskstop":
            window.App.commands.execute("contextmenu:selected:taskstop", row.model.id);
            break;
          case "menu-item-taskstart":
            window.App.commands.execute("contextmenu:selected:taskstart", row.model.id);
            break;
          case "menu-item-setactive":
            window.App.commands.execute("contextmenu:selected:setactive", row.model);
            break;
          case "menu-item-userdelete":
            if (row.model.get("role") === "Administrator" && window.App.Data.UserRole !== "Administrator") {
              window.alertify.error("You don't have enough rights to delete this user.");
              return false;
            }
            window.App.commands.execute("contextmenu:selected:userdelete", row.model);
            break;
          case "menu-item-resetpassword":
            if (row.model.get("role") === "Administrator" && window.App.Data.UserRole !== "Administrator") {
              window.alertify.error("You don't have enough rights to reset password to this user.");
              return false;
            }
            window.App.commands.execute("contextmenu:selected:resetpassword", row.model);
            break;
          case "menu-item-useropen":
            window.App.commands.execute("contextmenu:selected:useropen", row.model);
            break;
          case "menu-item-create":
            // only for MV table - create new as create new from parent, add GET parameter referenceId
            if (modelName === "multivariant") {
              window.open("#project" + "/" + modelName + "s/create?referenceId=" + row.model.id, '_self');
            } else {
              window.open("#project" + "/" + modelName + "s/create", '_self');
            }

            break;
          case "item-to-clipboard":
            var type = (modelName === "noname") ? row.model.get("type") : modelName;
            row.model.collection.trigger("contextmenu:selected:toclipboard", {
              type: type,
              id: row.model.id
            });
            break;
          case "menu-item-cancel-task":
            var id = row.model.id;
            var ownerId = row.model.get("ownerId");
            if (window.App.Data.UserProfile.get("user").id == ownerId) {
              var c = confirm("Do you want to remove this task from queue?");
              if (c) {
                $.ajax({
                  type: "POST",
                  url: "rest/tasksQueue/" + id + "/removeFromQueue",
                  success: function () {
                    window.alertify.success("Задача удалена из очереди");
                    row.model.collection.fetch();
                  }
                })
              }
            } else {
              window.alertify.error("У вас нет прав на удаление этой задачи из очереди. Вы не являетесь её создателем");
            }

            break;
          case "menu-item-delete":
            if (!row.model.get("userAccess") || row.model.get("userAccess").canModify) {
              var c = confirm("Are you sure you want to remove this " + modelName + "?");
              if (c) {
                var url = modelName != "simulationFile" ?
                "rest/" + modelName + "s/" +row.model.get("id") :
                "rest/simulations/" + row.model.collection.options.simulation + "/files/" + row.model.get("id");
                $.ajax({
                  type: "DELETE",
                  url: url,
                  success: function () {
                    row.model.collection.remove(row.model);
                    if (row.model.get("path")) {
                      window.App.vent.trigger('updateTreeNodes', {id: row.model.id}, "*", window.App.Helpers.ShowPath(row.model.get("path"), 'filetype'), 'remove');
                    }
                  }
                });
              }
            } else {
              window.alertify.warning("You have no rights to delete this item.")
            }
            break;
          case "menu-item-export":
            row.model.collection.trigger("export", { model: row.model, name: modelName });
         break;
          case "menu-item-remove-from-clipboard":
            var type = (modelName === "noname") ? row.model.get("type") : modelName;
            row.model.collection.trigger("contextmenu:remove:fromclipboard", {
              type: type,
              id: row.model.id
            });
            break;
          case "menu-item-delete-completely":
            var type = (modelName === "noname") ? row.model.get("type") : modelName;
            row.model.collection.trigger("contextmenu:delete:completely", {type: type, model: row.model});
            break;
          case "item-remove-from-list":
            row.model.collection.trigger("remove:from:list", row.model);
            break;
          case "menu-item-remove-from-simulation":
            row.model.collection.trigger("remove:from:simulation:menu", row.model);
            break;
          case "menu-item-add-submodel":
            var data = $(e.target).data("add");
            row.model.collection.trigger("add:submodel", { "event": data, "submodel": row.model });
            break;
          default :
            notImplemented();
        }
        if (shoudNotBeClosed) {
          shoudNotBeClosed = false
        } else {
          this.closemenu();
        }
      }
    });
  });

  window.App.Views.BackgridCoreView = Backbone.Marionette.LayoutView.extend({

    template: false,

    behaviors: {
      PluginsInit: {},
      ProjectCommonActions: {}
    },

    setCellExtendedType: function (columns) {
      return _.map(columns, function (column, idx) {

        var classType,
          columnType;

        columnType = typeof column["cell"] === "function" ? "class" : column["cell"].toLocaleLowerCase();
        
        switch (columnType) {
          case "date":
            classType = MyDatePickerCell;
            break;
          case "integer":
            classType = Backgrid.IntegerCell.extend({
              orderSeparator: ""
            });
            break;

          case "string":
            classType = Backgrid.StringCell.extend({});

            switch (column["name"]) {
              case 'flag':
                classType = classType.extend({
                  render: function () {
                    var that = this;
                    var flagState = this.model.get("flag");
                    var commentsCount = this.model.get("commentsTotal") || 0;
                    var disableClass = commentsCount === 0 ? "disabled" : "";
                    var $cellContent;
                    switch (flagState) {
                      case "success":
                        $cellContent = $("<button></button>").addClass("js-show-comments btn btn-success").html("Success (" + commentsCount + ")").addClass(disableClass);
                        break;
                      case "error":
                        $cellContent = $("<button></button>").addClass("js-show-comments btn btn-danger").html("Failed (" + commentsCount + ")").addClass(disableClass);
                        break;
                      case "undefined":
                        $cellContent = $("<button></button>").addClass("js-show-comments btn btn-undefined").html("Undefined (" + commentsCount + ")").addClass(disableClass);
                        break;
                      default:
                        $cellContent = $("<button></button>").addClass("js-show-comments btn btn-undefined").html("Undefined (" + commentsCount + ")").addClass(disableClass);
                        break;
                    }

                    this.$el.on("click", ".js-show-comments", function () {
                      var modelName = that.model.getName();
                      that.commentsView = new window.App.Views.FlagComments({
                        "type": modelName + "s",
                        "itemId": that.model.get("id")
                      });
                      that.modalComment = new Backbone.BootstrapModal({
                        title: "Comments",
                        content: that.commentsView,
                        animate: true,
                        okText: "Закрыть"
                      }).open();
                      that.listenTo(that.commentsView, "hide", function(length) {
                        that.$el.off("click");
                        that.model.set("commentsTotal", length);
                        that.render();
                      });
                    });

                    this.$el.empty();
                    this.$el.html($cellContent);
                    this.delegateEvents();
                    return this;
                  }
                });
                break;
              case 'hasPreview':
                classType = classType.extend({
                  render: function () {
                    var hasPreview = JSON.parse(this.model.get('hasPreview')); // String to boolean
                    if (hasPreview) {
                      var previewView = new window.App.Views.PreviewCellView({ model: this.model });
                      previewView.render();
                      this.$el.html(previewView.$el);
                    } else {
                      this.$el.html('<span style="cursor: default">Недоступно</span>');
                    }
                    return this;
                  }
                });
                break;
              case 'targetValue':
                classType = classType.extend({
                  render: function () {
                    this.$el.empty();
                    var formattedValue = this.formatter.fromRaw(this.model.get(this.column.get("name")), this.model).trim();
                    if (formattedValue) {
                      this.model.tValue = parseFloat(formattedValue.slice(2,formattedValue.lastIndexOf(" ")));
                      this.model.tValueCond = formattedValue.charCodeAt(0);
                      switch (this.model.tValueCond) {
                        case 60:
                          if (this.model.cValue < this.model.tValue) {
                            this.$el.html($("<span data-toggle='tooltip'></span>").attr("title", formattedValue).text(formattedValue));
                            this.$el.append($("<span class='pull-right'><i class='fa fa-check targetSuccess' aria-hidden='true'></i></span>"));
                          } else {
                            this.$el.html($("<span data-toggle='tooltip'></span>").attr("title", formattedValue).text(formattedValue));
                            this.$el.append($("<span class='pull-right'><i class='fa fa-times targetFail' aria-hidden='true'></i></span>"));
                          }
                          break;
                        case 62:
                          if (this.model.cValue > this.model.tValue) {
                            this.$el.html($("<span data-toggle='tooltip'></span>").attr("title", formattedValue).text(formattedValue));
                            this.$el.append($("<span class='pull-right'><i class='fa fa-check targetSuccess' aria-hidden='true'></i></span>"));
                          } else {
                            this.$el.html($("<span data-toggle='tooltip'></span>").attr("title", formattedValue).text(formattedValue));
                            this.$el.append($("<span class='pull-right'><i class='fa fa-times targetFail' aria-hidden='true'></i></span>"));
                          }
                          break;
                      }
                    }
                    this.delegateEvents();
                    return this;
                  }
                });
                break;
              case 'value':
                classType = classType.extend({
                  render: function () {
                    this.$el.empty();
                    var formattedValue = this.formatter.fromRaw(this.model.get(this.column.get("name")), this.model).trim();
                    if (formattedValue) {
                      this.model.cValue = parseFloat(formattedValue.slice(0,formattedValue.indexOf(" ")));
                    }
                    this.$el.html($("<span data-toggle='tooltip'></span>").attr("title", formattedValue).text(formattedValue));
                    this.delegateEvents();
                    return this;
                  }
                });
                break;
              default:
                classType = classType.extend({
                  render: function () {
                    this.$el.empty();
                    var formattedValue = this.formatter.fromRaw(this.model.get(this.column.get("name")), this.model);
                    this.$el.html($("<span data-toggle='tooltip'></span>").attr("title", formattedValue).text(formattedValue));
                    this.delegateEvents();
                    return this;
                  }
                });
                break;
            }
            break;
          case "text":
            classType = Backgrid.Extension.TextCell.extend({});
            classType = classType.extend({
              render: function () {
                this.$el.empty();
                var formattedValue = this.formatter.fromRaw(this.model.get(this.column.get("name")), this.model);
                this.$el.html($("<span data-toggle='tooltip'></span>").attr("title", formattedValue).text(formattedValue));
                this.delegateEvents();
                return this;
              }
            });
            break;
          case "uri":
            classType = Backgrid.UriCell.extend({
              render: function () {
                this.$el.empty();
                var rawValue = "";
                var title, formattedValue, linkParams, isMedia, isResult = false;
                var tooltip;
                switch (column.name.toLocaleLowerCase()) {
                  case
                  'name' :
                    var modelName = this.model.getName();
                    switch (modelName.toLocaleLowerCase()) {
                      case 'user':
                        rawValue = "#profile/" + (this.model.get("id") ? this.model.get("id") : "") + "/view";
                        break;
                      case 'material':
                        this._blankTarget = false;
                        rawValue = "#project/materials/" + (this.model.get("id") ? this.model.get("id") : "") + "/parameters/view";
                        break;
                      case 'technologycompositematerial':
                        this._blankTarget = false;
                        rawValue = "#project/compositeMaterials/" + (this.model.get("id") ? this.model.get("id") : "") + "/parameters/view";
                        break;
                      case 'compositematerial':
                        this._blankTarget = false;
                        rawValue = "#project/compositeMaterials/" + (this.model.get("id") ? this.model.get("id") : "") + "/parameters/view";
                        break;
                      case 'constructionelement':
                        this._blankTarget = false;
                        rawValue = "#project/constructionElements/" + (this.model.get("id") ? this.model.get("id") : "") + "/parameters/view";
                        break;
                      case 'technology':
                        this._blankTarget = false;
                        rawValue = "#project/technologies/" + (this.model.get("id") ? this.model.get("id") : "") + "/parameters/view";
                        break;
                      case 'technologysimulation':
                        this._blankTarget = false;
                        rawValue = "#project/technologySimulations/" + (this.model.get("id") ? this.model.get("id") : "") + "/parameters/view";
                        break;
                      case 'optimization':
                        this._blankTarget = false;
                        rawValue = "#project/optimizations/" + (this.model.get("id") ? this.model.get("id") : "") + "/parameters/view";
                        break;
                      case 'simulation':
                        this._blankTarget = false;
                        rawValue = "#project/simulations/" + (this.model.get("id") ? this.model.get("id") : "") + "/parameters/view";
                        break;
                      case 'file':
                        this._blankTarget = false;
                        rawValue = "#project/file/" + (this.model.get("id") ? this.model.get("id") : "") + "/view";
                        break;
                      case "effectivecharacteristic":
                        this._blankTarget = false;
                        rawValue = "#project/effectiveCharacteristics/" + (this.model.get("id") ? this.model.get("id") : "") + "/parameters/view";
                        break;
                      case "result":
                        this._blankTarget = false;
                        tooltip = "Скачать файл " + this.model.get("name");
                        rawValue = "rest/results/" + (this.model.get("id") ? this.model.get("id") : "") + "/content";
                        break;
                      default:
                        rawValue = "#project/" + modelName + "/" + (this.model.get("id") ? this.model.get("id") : "") + "/view";
                        break;
                    }
                    break;
                  case 'parentname':
                    var parentType = this.model.get("parentType");
                    switch (parentType.toLocaleLowerCase()) {
                      case 'effectivecharacteristic':
                        this._blankTarget = false;
                        rawValue = "#project/effectiveCharacteristics/" + (this.model.get("parentId") ? this.model.get("parentId") : "") + "/parameters/view";
                        break;
                      case 'simulation':
                        this._blankTarget = false;
                        rawValue = "#project/simulations/" + (this.model.get("parentId") ? this.model.get("parentId") : "") + "/parameters/view";
                        break;
                      case 'optimization':
                        this._blankTarget = false;
                        rawValue = "#project/optimizations/" + (this.model.get("parentId") ? this.model.get("parentId") : "") + "/parameters/view";
                        break;
                    }
                    break;
                  case 'reference':
                    var modelName = this.model.getName();
                    switch (modelName.toLocaleLowerCase()) {
                      case 'submodel':
                        rawValue = "#project/submodel/" + (this.model.get(column.name + "Id") ? this.model.get(column.name + "Id") : "") + "/show";
                        break;
                      case 'sessionfile':
                        rawValue = "#project/sessionfile/" + (this.model.get(column.name + "Id") ? this.model.get(column.name + "Id") : "") + "/show";
                        break;
                      case 'simulation':
                        rawValue = "#project/simulation/" + (this.model.get(column.name + "Id") ? this.model.get(column.name + "Id") : "") + "/show";
                        break;
                      case 'multivariant':
                        rawValue = "#project/multivariant/" + (this.model.get(column.name + "Id") ? this.model.get(column.name + "Id") : "") + "/show";
                        break;
                      default:
                        rawValue = "#project/" + modelName + "/" + (this.model.get("id") ? this.model.get("id") : "") + "/show";
                        break;
                    }
                    break;
                  case 'owner':
                    rawValue = "#profile/" + (this.model.get(column.name + "Id") ? this.model.get(column.name + "Id") : "") + "/show";
                    break;
                  case 'simulation':
                    rawValue = "#project/simulation/" + (this.model.get(column.name + "Id") ? this.model.get(column.name + "Id") : "") + "/show";
                    break;
                  case 'submodel':
                    rawValue = "#project/submodel/" + (this.model.get(column.name + "Id") ? this.model.get(column.name + "Id") : "") + "/show";
                    break;
                  case 'sessionfile':
                    rawValue = "#project/sessionfile/" + (this.model.get(column.name + "Id") ? this.model.get(column.name + "Id") : "") + "/show";
                    break;
                  case 'fiber':
                    rawValue = "#project/materials/" + (this.model.get("fiberId") ? this.model.get("fiberId") : "") + "/parameters/view";
                    break;
                  case 'matrix':
                    rawValue = "#project/materials/" + (this.model.get("matrixId") ? this.model.get("matrixId") : "") + "/parameters/view";
                    break;
                  case 'epoxide':
                    rawValue = "#project/materials/" + (this.model.get("epoxideId") ? this.model.get("epoxideId") : "") + "/parameters/view";
                    break;
                  case "actname":
                    rawValue = _.isNull(this.model.get("defectFileId")) ? "javascript:void(0)" : "rest/files/" + this.model.get("defectFileId") + "/content";
                    this._blankTarget = true;
                    break;
                  default:
                    rawValue = this.model.get(column.name + "Url"); //what ever you want to put in the href;;
                    break;
                }
                if (_.isEmpty(linkParams)) {
                  title = this.model.get(column.name);
                  linkParams = {
                    tabIndex: -1,
                    href: rawValue,
                    title: this.title || formattedValue,
                    target: this.model.collection.inModal || this._blankTarget ? "_blank" : "_self",
                    "data-toggle": "tooltip"
                  };
                }
                formattedValue = formattedValue ? formattedValue : this.formatter.fromRaw(title, this.model);
                this.$el.append($("<a>", linkParams).text(!isMedia ? formattedValue : "").attr("title", tooltip || formattedValue));
                this.delegateEvents();
                return this;
              }

            });
            break;

          case
          'image':
            classType = Backgrid.ImageCell.extend({
              render: function () {
                this.$el.empty();
                this.$el.html(this.renderImage(this.model));
                this.delegateEvents();
                return this;
              },

              renderImage: function (model) {
                var img = (this.model.get(column.name) ? this.model.get(column.name) : "javascript:void(0)");
                return '<div align="center"><img src="' + img + '" height="30" alt="" /></div>';
              }
            });
            break;
          default :
            classType = column["cell"];
            break;

        }
        column["cell"] = classType;
        return column;
      });
    },

    getColumns: function () {
      return this.tableColumns;
    },

    initGridColumns: function (/*reset*/) {
      var that = this;
      //reset = reset || false;

      //legacy for default columns
      var Columns = Backgrid.Columns.extend({
        defaults: _.defaults({
          sortable: "toggle"
        }, Backgrid.Columns.prototype.defaults),
        initialize: function() {
          var that = this;
          this.fetched = false;

          this.listenTo(this, 'sync', function () {
            that.fetched = true;
          });
          Backgrid.Columns.prototype.initialize.apply(this, arguments);
        },
        url: (this.options.gridCollection.columnsUrl ? this.options.gridCollection.columnsUrl() : (typeof this.options.gridCollection.url === "function" ? this.options.gridCollection.url() : this.options.gridCollection.url) + "/columns")

      });

      this.tableColumns = new Columns();
      this.tableColumns.fetch().success(function (cols) {
        
        if (that.options.showableColumns) {
          cols = _.filter(cols, function (column) {
            return _.indexOf(that.options.showableColumns, column.name) !== -1;
          });
        }

        var mappedCols = that.setCellExtendedType(cols);

        if (that.options.selectable) {
          mappedCols.unshift(that.selectableColumn);
        }
        that.columns = mappedCols;
        that.BackGrid.columns.reset(mappedCols);

        that.BackGrid.columns.map(function (m, k) {
          m.set('sortable', that.options.columnSortable);

          if (k == 0) {
            m.set('minWidth', 20);
            m.set('maxWidth', 20);
            m.set('width', 20);
            m.set('resizeable', true)
          } else {
            m.set('minWidth', 20);
            m.set('maxWidth', 150);
            m.set('width', 75);
            m.set('resizeable', true)
          }

          return m;
        });

        //that.trigger('backgrid:columns', that.BackGrid.columns);

        if (that.options.fetchGridData) {
          that.BackGrid.collection.fetch({reset: true});
        } else {
          that.BackGrid.collection.trigger('reset');
        }
      });


    },

    initialize: function (options) {
      var that = this;
      this.options = options;
      this.options.selectable = (options.selectable !== false);
      this.options.sortableColumns = (options.sortableColumns !== false);
      this.options.saveToServer = (options.saveToServer !== false);
      this.options.fetchColumns = (options.fetchColumns !== false);
      this.options.columnSortable = (options.columnSortable !== false);


      this.backgridCollection = options.gridCollection;
      this.columns = options.columns || {};

      this.selectableColumn = {
        name: "",
        cell: "select-row",
        headerCell: "select-all",
        size: 3
      };

      var tableId = Math.random().toString(36).substring(7);
      this.BackGrid = new Backgrid.Grid({
        attributes: {"data-menu-id": tableId},
        className: 'table backgrid table-bordered table-striped table-condensed table-hover',
        columns: this.options.selectable ? [this.selectableColumn

        ].concat(this.columns) : this.columns,
        //TODO use collection option instead
        //collection: this.backgridCollection,
        collection: this.options.fetchGridData ? this.backgridCollection : new window.App.Collections.GridBaseCollection(),
        emptyText: "нет данных",
        header: Backgrid.Header.extend({
          render: function () {
            Backgrid.Header.prototype.render.call(this, arguments);
            //TODO need to think how to implement column width save/load
            this.$el.find('th').each(function (id, el) {});
            return this;
          }
        }),
        row: ClickableRow.extend({
          attributes: function () {
            return {
              "data-toggle": "context",
              "data-target": "#" + tableId,
              "data-id": this.model.id,
              "data-type": this.model.get("type"),
              "data-own": window.App.Data.UserProfile.get("user").id == this.model.get("ownerId")
            }
          },

          render: function () {
            if (that.options.draggableRows) this.initDND();
            if (this.model.get("userAccess") && !this.model.get("userAccess").canView) this.$el.addClass("ui-access-denied-row").empty();

            ClickableRow.prototype.render.call(this, arguments);
            return this;
          },

          initDND: function () {
            var c = {};
            var data = {};
            if (window.App.Data.UserRole !== "Administrator" && window.App.Data.UserProfile.get("user").id !== this.model.get("ownerId")) return false;
            $(this.el).draggable({
              axis: "y",
              helper: "clone",
              cursor: "move",
              cursorAt: {left: -15},
              activeClass: "ui-dragged",
              start: function (e, ui) {
                c.tr = this;
                c.helper = ui.helper;

                data.task1Id = $(e.target).data("id");

                var type = $(e.target).closest("tr").data("type");
                var own = $(e.target).closest("tr").data("own");

                $(c.tr).addClass("ui-on-drag-item");
                $(c.helper).addClass("fix-opacity");

                var $droppableTargets = $("tbody tr[data-type='" + type + "']").not(c.helper).not(this);
                if (window.App.Data.UserRole !== "Administrator") $droppableTargets = $droppableTargets.not("[data-own='false']");

                $droppableTargets.droppable({
                  hoverClass: "ui-drop-hover-item",
                  drop: function (e, ui) {
                    data.task2Id = $(e.target).data("id");
                    $.ajax({
                      type: "POST",
                      url: "rest/tasksQueue/swap",
                      data: JSON.stringify(data),
                      contentType: "application/json",
                      success: function () {
                        window.alertify.dismissAll();
                        window.alertify.success("Порядок задач изменен.");
                        that.BackGrid.collection.fetch();
                      },
                      error: function () {
                        $(c.tr).removeClass("ui-on-drag-item");
                        $(c.helper).removeClass("fix-opacity");
                        $(".ui-droppable").removeClass('ui-droppable');
                      }
                    })
                  }
                });
              },
              stop: function () {
                $(c.tr).removeClass("ui-on-drag-item");
                $(c.helper).removeClass("fix-opacity");
              }
            });
          }
        })
      });

      if (options.gridCollection.url && this.options.fetchColumns) {
        this.initGridColumns();
      } else {
        this.columns = this.setCellExtendedType(this.columns);
      }

      this.listenTo(this.backgridCollection, "stop:task", function (options) {
        if (options.model.get("stoppable")) {
          var confStatus = confirm("Do you want to stop this task?");
          if (confStatus) {
            $.ajax({
              type: "POST",
              url: "rest/tasks/" + options.model.id + "/stop",
              success: function () {
                window.alertify.success("Stop command was sent successfully.");
                that.model.fetch();
              }
            });
          }
        } else {
          window.alertify.warning("This task is in not stoppable state.")
        }
      }, this);


      this.listenTo(this.backgridCollection, "cancel:task", function (options) {
        var id = options.model.id;
        var ownerId = options.model.get("ownerId");
        if (window.App.Data.UserProfile.get("user").id == ownerId) {
          if (options.model.get("statusId") == 2) {
            var c = confirm("Do you want to remove this task from queue?");
            if (c) {
              $.ajax({
                type: "POST",
                url: "rest/tasksQueue/" + id + "/removeFromQueue",
                success: function () {
                  window.alertify.success("Task was successfully removed from queue.");
                  options.model.collection.fetch();
                }
              });
            }
          } else {
            window.alertify.error("You can cancel only waiting tasks.");
          }
        } else {
          window.alertify.error("You are not owner of this task. You can not remove it from queue.");
        }
      }, this);

      this.listenTo(this.backgridCollection, "backgrid:edited", function (m, c, o) {
        var count = _.filter(this.columns, function (c) {
          return c.editable !== false && c.cell !== 'select-row';
        }).length;

        _.each(_.keys(m.changedAttributes()), function (k) {
          if (m.changedAttributes()[k] == m.previousAttributes()[k]) {
            that.options.saveToServer = false;
          } else {
            !!c.get("editable") ? that.options.saveToServer = true : "";
          }
        });

        count <= m.keys().length && that.options.saveToServer ? m.hasChanged() ? m.save().success(function () {
          window.alertify.success("Saved");
        }).error(function () {
          window.alertify.error("Saving Error");
          setPreviousValue();
        }) : "" : setPreviousValue();

        function setPreviousValue() {
          _.each(m.changed, function (v, k) {
            if (typeof m.previousAttributes()[k] !== 'undefined') {
              m.set(k, m.previousAttributes()[k]);
            }
          });
        }

      }, this);
      //
      this.listenTo(this.BackGrid, "backgrid:rendered", function () {

        that.BackGrid.$el.colResizable({
          minWidth: 20,
          fixed: false,
          postbackSafe: true,
          partialRefresh: true
        });

      });

      this.listenTo(this.backgridCollection, "add", function (m, c, o) {

      }, this);

      this.listenTo(this.backgridCollection, "contextmenu:show", function (options) {
        var selectedModelsLength = this.BackGrid.getSelectedModels().length;
        options.$target.find("#menu-item-compare").addClass("disabled-lnk");
        if (options.name === "noname" && options.model.get("type")) options.name = options.model.get("type"); // Get result modelName
        switch (options.name) {
          case "simulation":
            if (selectedModelsLength == 2) options.$target.find("#menu-item-compare").removeClass("disabled-lnk");
            break;
          case "submodel":
            if (selectedModelsLength == 2) options.$target.find("#menu-item-compare").removeClass("disabled-lnk");
            break;
          default:
            // Is true for any result (value, curve, animation, picture, etc.)
            if (selectedModelsLength >= 2 && selectedModelsLength <= 6) options.$target.find("#menu-item-compare").removeClass("disabled-lnk");
            break;
        }
      });

      this.listenTo(this.backgridCollection, "export", function(options) {
        var selectedFilesLength = this.BackGrid.getSelectedModels().length;
        var model = selectedFilesLength == 1 ? this.BackGrid.getSelectedModels()[0] : options.model;
        if (options.name == "simulationFile" && selectedFilesLength <= 1)
          window.open("rest/simulations/" + model.collection.options.simulation + "/files/" + model.get("id") + "/export", "_self");
      });

      this.listenTo(this.backgridCollection, "contextmenu:selected:toclipboard", function (options) {
        var selectedModels = this.BackGrid.getSelectedModels();
        if (selectedModels.length === 0) {
          window.clipboard.push(options.type, options.id);
        } else {
          if (options.type !== "value" && options.type !== "animation" && options.type !== "picture" && options.type !== "curve") {
            window.clipboard.pushM(selectedModels, options.type);
          } else {
            window.clipboard.pushM(selectedModels);
          }
        }
      }, this);

      this.listenTo(this.backgridCollection, "remove:from:simulation:menu", function(model) {
        var selectedModels = this.BackGrid.getSelectedModels();
        selectedModels = !_.isEmpty(selectedModels) ? selectedModels : [model];
        model.collection.trigger("remove:from:simulation", selectedModels);
      });

      this.listenTo(this.backgridCollection, "contextmenu:delete:completely", function (options) {
        var url = options.type === "simulation" || options.type === "submodel" || options.type === "sessionfile" ? "rest/" + options.type + "s/" + options.model.get("id") : "rest/simulations/" + options.model.get("simulationId") + "/results/" + options.model.get("id");
        var c = confirm("Are you sure to delete this " + options.type + " completely?");
        if (c) {
          $.ajax({
            type: "DELETE",
            url: url,
            success: function () {
              window.clipboard.delete(options.type, options.model.get("id"));
              window.alertify.success("Успешно удалено");
            }
          });
        }
      }, this);


      this.listenTo(this.backgridCollection, "contextmenu:selected:compare", function (entity) {
        var selectedModelsIDs = _.pluck(this.BackGrid.getSelectedModels(), 'id');
        var params = {};
        params.selectedIds = selectedModelsIDs;
        var query = URI.buildQuery(params);

        Backbone.history.navigate("project" + "/compare/" + entity + "s?" + query, {trigger: true});
      }, this);



      window.App.reqres.setHandler('backgrid:columns', function() {
        return that.tableColumns;
      })
    },

    insertRow: function (gridModel) {
      var m = {};
      if (typeof gridModel === "function") {
        m = new gridModel();
      } else {
        m = gridModel;
      }
      this.BackGrid.insertRow(m);
    },

    renderPaginator: function () {
      this.backgridPaginator = new Backgrid.Extension.Paginator({
        windowSize: 20, //the number of page handles to show // Default is 10
        goBackFirstOnSort: false,
        slideScale: 0.25, // Default is 0.5
        collection: this.backgridCollection
      });

      return this.backgridPaginator.render().$el;
    }
  });

})();