(function () {
  "use strict";
  window.App.Views.TableHeaderActionsView = Marionette.LayoutView.extend({
    template: "./views/templates/project/table_header_actions.html.ejs",

    attributes: {
      "unselectable": "on"
    },

    className: "actions-box clearfix",

    ui: {
      action: ".js-action",
      oneItemAction: ".js-one-item-action",
      twoItemAction: ".js-two-item-action"
    },

    events: {
      "click @ui.action": "onActionClick"
    },

    initialize: function (options) {
      this.actions = options.actions || [];
      this.type = options.type || null;
      this.isFile = options.isFile;
      this.gridCollection = options.gridCollection || {};
      this.backgrid = options.backgrid || {};
      this.listenTo(options.gridCollection, "backgrid:row:selected", this.onRowSelect);
      this.listenTo(options.gridCollection, "reset:selected", this.onResetSelected);
      this.disabledStrict = false;
      this.selected = [];
      this.listenTo(options.gridCollection, "toggle:icons:disable:state", this.toggleIconsDisableState);
    },

    toggleIconsDisableState: function (isDisabled) {
      this.disabledStrict = isDisabled;
      this.disabledStrict ? this.ui.action.addClass("disabled") : this.render();
    },

    onRowSelect: function (model) {
      if (this.disabledStrict) return false;
      var index = _.indexOf(this.selected, model);
      index == -1 ? this.selected.push(model) : this.selected.splice(index, 1);

      switch (true) {
        case (this.selected.length == 0):
          this.ui.action.addClass("disabled");
          this.$("#js-new").removeClass("disabled");
          this.$("#js-add-model-to-optimization").removeClass("disabled");
          this.$("#js-add-to-optimization").removeClass("disabled");
          break;
        case (this.selected.length == 1):
          this.ui.action.removeClass("disabled");
          this.ui.twoItemAction.addClass("disabled");
          break;
        case (this.selected.length == 2):
          this.ui.action.removeClass("disabled");
          this.ui.oneItemAction.addClass("disabled");
          this.ui.twoItemAction.removeClass("disabled");
          break;
        case (this.selected.length > 2):
          this.ui.action.removeClass("disabled");
          this.ui.oneItemAction.addClass("disabled");
          this.ui.twoItemAction.addClass("disabled");
          break;
      }
    },

    onResetSelected: function () {
      this.selected = [];
      this.render();
    },

    onActionClick: function (e) {
      var that = this;
      if ($(e.target).hasClass("disabled")) return false;
      var modelName = this.type || this.selected[0].get("type") || this.selected[0].getName();

      var event = $(e.target).attr("title");
      switch (event) {
        case "open":
          this._open(modelName);
          break;
        case "delete":
          this._delete(modelName);
          break;
        case "clipboard":
          var eventName = "contextmenu:selected:toclipboard";
          this._clipboard(eventName, modelName);
          break;
        case "compare":
          this._compare(modelName);
          break;
        case "new":
          this._new(modelName, this.isFile);
          break;
        case "user":
          this._createUser();
          break;
        case "remove":
          var eventName = "contextmenu:remove:fromclipboard";
          this._clipboard(eventName);
          break;
        case "export":
          this._export(modelName);
          break;
        case "before":
          this._add(event);
          break;
        case "after":
          this._add(event);
          break;
        case "instead":
          this._add(event);
          break;
        case "add-model-to-optimization":
          this._addModelToOptimization();
          break;
        case "add-to-optimization":
          this._addToOptimization();
          break;
        case "remove-from-optimization":
          this._removeFromOptimization();
          break;
        case "remove-from-sampleFiles":
          this._removeFromSampleFiles();
          break;
        case "remove-from-simulation":
          this._removeFromSimulation();
          break;
        case "remove-from":
          this._removeFromParent();
          break;
        case "block-unblock":
          this._blockUser();
          break;
        case "reset-password":
          this._resetUserPassword();
          break;
        case "edit_task":
          this._edit_task(modelName, that.selected[0]);
          break;
        case "stop_task":
          this._stop_task(modelName, that.selected[0]);
          break;
        case "cancel_task":
          this._cancel_task(modelName, that.selected[0]);
          break;
      }
    },

    _open: function (modelName) {
      if (this.selected.length !== 1) {
        window.alertify.error("You need to one select concrete row to open it's page.")
      } else {
        var URL,
          model = this.selected[0];
        switch (modelName.toLocaleLowerCase()) {
          case 'task':
            URL = "#project/simulation/" + (model.get("simulationId") ? model.get("simulationId") : "") + "/show";
            break;
          case 'running_task':
            URL = "#project/simulation/" + (model.get("simulationId") ? model.get("simulationId") : "") + "/show";
            break;
          case 'user':
            URL = "#profile/" + (model.get("id") ? model.get("id") : "") + "/view";
            break;
          case "material":
            URL = "#project/materials/" + (model.get("id") ? model.get("id") : "") + "/parameters/view";
            break;
          case "materialfiles":
            URL = "#project/file/" + (model.get("id") ? model.get("id") : "") + "/view";
            break;
          case "samplefiles":
            URL = "#project/file/" + (model.get("id") ? model.get("id") : "") + "/view";
            break;
          case "loadconditions":
            URL = "#project/file/" + (model.get("id") ? model.get("id") : "") + "/view";
            break;
          case "designsolutions":
            URL = "#project/file/" + (model.get("id") ? model.get("id") : "") + "/view";
            break;
          case "cadfiles":
            URL = "#project/file/" + (model.get("id") ? model.get("id") : "") + "/view";
            break;
          case "caefiles":
            URL = "#project/file/" + (model.get("id") ? model.get("id") : "") + "/view";
            break;
          case "caofiles":
            URL = "#project/file/" + (model.get("id") ? model.get("id") : "") + "/view";
            break;
          default:
            URL = "#project/" + modelName + "/" + (model.get("id") ? model.get("id") : "") + "/parameters/view";
        }
        window.App.navigate(URL, {trigger: true});
      }
    },

    _delete: function (modelName) {
      var that = this;
      var str = this.selected.length > 1 ? "эти объекты" : "этот объект";
      var c = confirm("Вы уверены что хотите удалить " + str + "?");
      console.log(modelName )
      if (c) {
        var url = "rest/";
        if (that.isFile) {
          url = url + that.gridCollection.options.objectType + "/" + that.gridCollection.options.objectId + "/" + that.type;
        } else {
          switch (modelName) {
            case "user":
              url = url + "/users";
              break;
            case "technologyDefects":
              url = url + "technologies/" + that.gridCollection.options.technologyId + "/defects";
              break;
            case "technologyEquipments":
              url = url + "technologies/" + that.gridCollection.options.technologyId + "/equipments";
              break;
            case "technologyParameters":
              url = url + "technologies/" + that.gridCollection.options.technologyId + "/parameters";
              break;
            case "technologySimulations":
              url = url + "/technologySimulations";
              break;
            default:
              url = url + modelName;
          }
        }
        url = url + "?ids=" +_.pluck(this.selected, "id").join(",");
        $.ajax({
          type: "DELETE",
          url: url,
          success: function () {
            window.alertify.success("Успешно удалено");
            that.gridCollection.fetch();
            that.onResetSelected();
          },
          error: function () {
            that.gridCollection.fetch();
          }
        });
      }
    },

    _export: function (modelName) {
      switch (modelName) {
        case "simulationFile":
          $.get("rest/simulations/" + this.gridCollection.options.simulation.id + "/files/" + this.selected[0].get("id") + "/exportFtp", function () {
            window.alertify.success("Copying file started on FTP-server.");
          });
          break;
        case "tcl":
          var name = this.selected[0].get("name");
          name = name.indexOf(".tcl") !== -1 ? name.replace(".tcl", "") : name;
          window.open("rest/tcls/" + name, '_self');
          break;
        case "file":
          var id = this.selected[0].id;
          window.open("rest/files/" + id + "/content", "_self");
          break;
        case "result":
          var id = this.selected[0].id;
          window.open("rest/results/" + id + "/content", "_self");
          break;
        default:
          var id = this.selected[0].id;
          window.open("rest/files/" + id + "/content", "_self");
          break;
      }
    },

    _edit_task: function (modelName, model) {
      if (model.get('status') != 'Waiting') {
        window.alertify.error("This task is already completed.");
        return false;
      }

      if (window.App.Data.UserProfile.get("user").id == model.get('ownerId')) {
        if (model.get('parentId') && (model.get("parentType") == "simulation")) {

          new window.App.Models.Simulation({id: model.get('parentId')}).fetch().done(function (data) {
            var contentView = new window.App.Views.RunSimulationModalView(_.extend({taskId: model.get('id')}, {simulation: data}));

            var modal = window.App.Modals.create({
              content: contentView,
              title: "Run Simulation: " + data.name,
              okText: "Run"
            });
          });

        }
      } else {
        window.alertify.error("You are not owner of this task. You can not modify it.");
      }
    },

    _cancel_task: function () {
      this.selected[0].collection.trigger("cancel:task", {model: this.selected[0]});
    },

    _stop_task: function () {
      this.selected[0].collection.trigger("stop:task", {model: this.selected[0]});
    },

    // THIS one currently use context menu method
    _clipboard: function (eventName, type) {
      this.selected[0].collection.trigger(eventName, {
        type: type,
        id: this.selected[0].id
      });
    },

    _compare: function (modelName) {
      if (this.selected.length !== 2) {
        window.alertify.error("You need to select at least two " + modelName + "s");
        return false;
      }
      if (modelName !== "simulation" && modelName !== "submodel" && modelName !== "sessionfile") {
        this.notImplemented();
        return false;
      }
      this.selected[0].collection.trigger("contextmenu:selected:compare", modelName)
    },

    _new: function (modelName, isFile) {
      var reference = this.selected[0] ? "?referenceId=" + this.selected[0].id : "";
      var route = "#project/";
      var target = "_self";
      var getParams = $.param(window.App.Helpers.getParmsFromURLHash(window.location.hash));
      if (!_.isEmpty(reference)) {
        if (!_.isEmpty(getParams)) {
          getParams = "&" + getParams;
        }
      } else {
        getParams = "?" + getParams;
      }
      if (isFile) {
        route = "#project/files/" + this.gridCollection.options.objectType + "/" + this.gridCollection.options.objectId + "/";
        target = "_self";
      }
      if ((modelName.indexOf("technology") != -1) && (modelName.indexOf("technologySimulations") == -1)) {
        route += "technologies/" + this.gridCollection.options.technologyId + "/";
        target = "_self";
      }
      if (modelName.indexOf("effectiveCharacteristic") != -1) {
        route += "compositeMaterials/" + this.gridCollection.options.filters.compositeMaterialId + "/";
        target = "_self";
      }
      if (modelName.indexOf("technologySimulations") != -1) {
          route += "technologies/" + this.gridCollection.options.filters.technologyId + "/";
          target = "_self";
      }
      if (modelName == "user") modelName += "s";
      if (this.gridCollection.options.dependsOnParent) {
        route += this.gridCollection.options.objectType + "/" + this.gridCollection.options.objectId + "/";
      }
      window.open(route + modelName + "/create" + reference + getParams, target);
    },

    _addModelToOptimization: function () {
      var options = {
        type: this.options.type,
        optimizationId: this.gridCollection.options.optimizationId,
        collection: this.gridCollection
      };
      this.addModelToOptimization = new window.App.Views.AddModelToOptimization(options);
      this.addModal = new Backbone.BootstrapModal({
        content: this.addModelToOptimization,
        title: "Добавить " + this.parseDependecyType(this.options.type),
        animate: true,
        cancelText: "Отменить",
        okText: "Подтвердить"
      }).open();
      this.addModelToOptimization.passModal(this.addModal);
    },

    _addToOptimization: function () {
      var options = {
        type: this.options.type,
        optimizationId: this.gridCollection.options.optimizationId,
        collection: this.gridCollection
      };
      this.addDependencyToOptimization = new window.App.Views.AddDependencyToOptimization(options);
      this.addModal = new Backbone.BootstrapModal({
        content: this.addDependencyToOptimization,
        title: "Добавить " + this.parseDependecyType(this.options.type),
        animate: true,
        cancelText: "Отменить",
        okText: "Подтвердить"
      }).open();
      this.addDependencyToOptimization.passModal(this.addModal);
    },

    parseDependecyType: function (type) {
      var parsedType;
      switch (type) {
        case "materials":
          parsedType = "Материал";
          break;
        case "technologies":
          parsedType = "Технологию";
          break;
        case "loadConditions":
          parsedType = "Условие Нагружения";
          break;
        case "CAEFiles":
          parsedType = "CAE-файл";
          break;
        case "CADFiles":
          parsedType = "CAD-файл";
          break;
      }
      return parsedType;
    },

    _removeFromSampleFiles: function () {
      var that = this;
      if (this.selected && this.selected.length) {
        this.selected[0].urlRoot = "rest/compositeMaterials/" + this.selected[0].options.pid + "/sampleFiles/";
        this.selected[0].destroy({
          success: function () {
            that.selected.splice(0, 1);
            if (that.selected.length) {
              that._removeFromSampleFiles();
            }
          }
        });
      }
    },

    _removeFromOptimization: function () {
      var that = this;
      var selectedModels = this.selected;
      $.ajax({
        type: "DELETE",
        url: "rest/optimizations/" + this.gridCollection.options.optimizationId + "/" + this.options.type,
        contentType: "application/json",
        data: JSON.stringify(_.pluck(selectedModels, "id")),
        success: function () {
          window.App.commands.execute("optimization:dependency:removed", {
            type: that.options.type,
            num: selectedModels.length
          });
          window.alertify.success("Успешно удалено");
          that.gridCollection.fetch();
        }
      })
    },

    _removeFromSimulation: function () {
      this.selected[0].collection.trigger("remove:from:simulation", this.selected);
    },

    _removeFromParent: function () {
      if (confirm("Подтвердите удаление объекта из оптимизации")) {
        this.selected[0].destroy({url: this.selected[0].collection.url().replace("list", this.selected[0].id)});
      }
    },

    _add: function (event) {
      this.selected[0].collection.trigger("add:submodel", {"event": event, "submodel": this.selected[0]});
    },

    _blockUser: function () {
      window.App.commands.execute("contextmenu:selected:setactive", this.selected[0]);
    },

    _resetUserPassword: function () {
      window.App.commands.execute("contextmenu:selected:resetpassword", this.selected[0]);
    },

    _createUser: function () {
      window.open("#project/users/create", "_self");
    },

    notImplemented: function () {
      alert("Not implemented yet");
    },

    _toPlural: function (objectName) {
      if (objectName == "technology") objectName = "technologie";
      return objectName + "s";
    },

    templateHelpers: function () {
      return {
        actions: this.actions,
        actionClass: function (action, isGlyph) {
          var oneItemClass = " js-one-item-action"; // Helper for actions available for one row only
          var twoItemClass = " js-two-item-action"; // Helper for actions available for two row only
          switch (action) {
            case "open":
              return isGlyph ? "new-window" + oneItemClass : "open";
              break;
            case "delete":
              return isGlyph ? "remove" : "delete";
              break;
            case "clipboard":
              return isGlyph ? "pushpin" : "clipboard";
              break;
            case "compare":
              return isGlyph ? "retweet" + twoItemClass : "compare";
              break;
            case "new":
              return isGlyph ? "plus" + oneItemClass : "new";
              break;
            case "user":
              return isGlyph ? "plus" : "user";
              break;
            case "remove":
              return isGlyph ? "share" : "remove";
              break;
            case "before":
              return isGlyph ? "arrow-up" + oneItemClass : "before";
              break;
            case "after":
              return isGlyph ? "arrow-down" + oneItemClass : "after";
              break;
            case "instead":
              return isGlyph ? "share-alt" + oneItemClass : "instead";
              break;
            case "remove-from-simulation":
              return isGlyph ? "remove" : "remove-from-simulation";
              break;
            case "add-model-to-optimization":
              return isGlyph ? "plus" + oneItemClass : "add-model-to-optimization";
              break;
            case "add-to-optimization":
              return isGlyph ? "plus" : "add-to-optimization";
              break;
            case "remove-from-optimization":
              return isGlyph ? "share" : "remove-from-optimization";
              break;
            case "remove-from-sampleFiles":
              return isGlyph ? "share" : "remove-from-sampleFiles";
              break;
            case "remove-from":
              return isGlyph ? "remove" : "remove-from";
              break;
            case "block":
              return isGlyph ? "ban-circle" + oneItemClass : "block-unblock";
              break;
            case "reset-password":
              return isGlyph ? "envelope" + oneItemClass : "reset-password";
              break;
            case "export":
              return isGlyph ? "download-alt" + oneItemClass : "export";
              break;
            case "edit_task":
              return isGlyph ? 'pencil' + oneItemClass : 'edit_task';
              break;
            case "cancel_task":
              return isGlyph ? 'ban-circle' + oneItemClass : 'cancel_task';
              break;
            case "stop_task":
              return isGlyph ? 'stop' + oneItemClass : 'stop_task';
              break;
            default:
              return "none";
              break;
          }
        }
      }
    }

  });

}());