(function () {
  "use strict";

  window.App.Views._BaseMain = window.App.Views.CollapsableBlock.extend({
    onNameChange: function () {
      var name = this.ui.name.val();
      if (!_.isEmpty(name)) {
        this.ui.name.closest(".form-group").removeClass("has-error");
        this.model.save({ "name": name }, {async: false});
      } else {
        window.alertify.error("Name can not be empty.");
        this.ui.name.closest(".form-group").addClass("has-error");
      }
    },

    onDescriptionChange: function () {
      var description = this.ui.description.val();
      this.model.save({ "description": description }, { async: false });
    },

    onInputImageChange: function (e) {
      e.preventDefault();
      if (e.target.files[0]) {
        this.imageWasEdited = true;
        this.ui.previewImage.attr('src', URL.createObjectURL(e.target.files[0]));
        this.postImage();
      }
    },

    onRemoveImage: function (e) {
      e.preventDefault();
      e.stopPropagation();

      var c = confirm("Are you sure to remove this image?");
      if (!c) return;

      var that = this;
      $.ajax({
        type: "DELETE",
        url: this.model.url() + "/image",
        success: function () {
          that.ui.previewImage.attr('src',that.model.url() + "/image?rand="+Math.random());
        }
      });
    },

    postImage: function () {
      var that = this;
      this.$el.find('.image-upload-form').ajaxForm({
        url: "rest/" + this.model.get("objectType") + "s/" + that.model.id + "/image",
        type: "POST",
        success: function (data) {
          window.alertify.success("New image was uploaded successfully");
        },
        error: function (xhr) {
          window.alertify.error("Error, new image was not uploaded successfully");
        }
      }).submit();
    },

    onReferenceClick: function () {
      if (this.ui.reference.hasClass("disabled")) {
        var referenceId = this.ui.reference.data("id");
        if (referenceId) window.App.navigate("#project/" + this.model.get("objectType") + "/" + referenceId + "/view", { trigger: true });
        return false;
      }
      var that = this;
      this.chooseReferenceView = new window.App.Views.ChooseReferenceView({ model: this.model });
      var modal = window.App.Modals.create({
        title: "Choose Reference",
        content: this.chooseReferenceView,
        animate: true,
        cancelText: "Cancel",
        okText: "Choose",
        className: "modal add-row-modal"
      });
      this.chooseReferenceView.on("reference:selected", function (options) {
        that.model.save({
          "reference": options.name,
          "referenceId": options.id
        });
        that.ui.reference.text(options.name);
        that.ui.removeReference.removeClass("hidden");
      });
    },

    onRemoveReferenceClick: function () {
      this.model.save({
        "referenceId": null,
        "reference": null
      });
      this.ui.reference.text("Choose Reference");
      this.ui.removeReference.addClass("hidden");
    },

    setPath: function (path) {
      this.ui.path.text(window.App.Helpers.ShowPath(path));
      this.model.set('path', path);
      this.model.set('pid', window.App.Helpers.ShowPath(path, 'dir_id'));
      window.App.Helpers.CutPathChecker(this);
      this.model.save();
    },

    onShow: function () {
      if (!_.isUndefined(this.model.get("path"))) window.App.Helpers.CutPathChecker(this);
      if (!_.isUndefined(this.model.get("description"))) window.App.Helpers.InitializeResizableDescription(this, this.options.isView);
    }

  });

}());
