(function() {
  "use strict";
  window.App.Views.BasePage = Marionette.LayoutView.extend({
    template: "./views/templates/project/base/base_page.html.ejs",
    className: function () {
      return this.options.mode;
    },

    regions: {
      controlsRegion:   ".js-controls-region",
      mainRegion:       ".js-main-region"
    },

    ui: {
      navItem: ".js-nav-item",
      run: ".js-run"
    },

    events: {
      "click @ui.navItem": "onClickNavItem",
      "click @ui.run": "onRunClick",
    },

    initialize: function (options) {
      this.options = options || {};
      this.mode = this.options.mode;
      this.options.modelName==="optimizations" ? this.controls = ["copy", "edit", "getRes", "showLog"] : this.controls = ["new", "edit"];
      this.page = "parameters"; // Default
      this.pageTitle = this.model.get("name") ? this.model.get("name") : "";
    },

    onRender: function () {
      this.initAndShowControlsView(this.getRegion("controlsRegion"));
    },

    onNewClick: function () {
      window.App.navigate("#project/" + this.options.modelName +  "/create", { trigger: true });
    },

    onCopyClick: function () {
        window.App.navigate("#project/" + this.options.modelName + "/" + this.model.id + "/copy", { trigger: true });
    },

    onEditClick: function () {
      var url = "#project/" + this.options.modelName + "/" + this.model.id + "/" + this.page + "/edit";
      var tab = parseInt(location.hash.slice(-1));
      if (!_.isNaN(tab)) url = url + "/" + tab;
      window.App.navigate(url, { trigger: true });
    },

    onRunClick: function () {
      this.runModalView = new window.App.Views.OptimizationRunModal({
        "parentType": "optimization",
        "parentName": this.model.get("name"),
        "parentId": this.model.id
      });
      this.runModal = new Backbone.BootstrapModal({
        content: this.runModalView,
        title: "Запуск расчета",
        animate: true,
        cancelText: "Отменить",
        okText: "Запустить"
      }).open();
      this.runModalView.passModal(this.runModal);
    },

    onCancelClick: function () {
      var url = "#project/" + this.options.modelName + "/" + this.model.id + "/" + this.page + "/view";
      var tab = parseInt(location.hash.slice(-1));
      if (!_.isNaN(tab)) url = url + "/" + tab;
      window.App.navigate(url, { trigger: true });
    },

    onGetResClick: function () {
      $.ajax({
        type: "GET",
        url: "rest/optimizations/" + this.model.id + "/downloadCurrentResults",
        success: function(){
          var url = location.hash;
          window.App.navigate(url, { trigger: true });
          window.alertify.success("Запрос отправлен");
        },
        statusCode: {
          404: function () {
            window.alertify.error("Ошибка");
          }
        }
      });
    },

    onRunTSClick: function () {
      $.ajax({
        type: "GET",
        url: "rest/technologySimulations/" + this.model.id + "/start",
        success: function(){
          window.alertify.success("Запрос отправлен");
          window.open(this.url, "_self");
          var url = location.hash;
          window.App.navigate(url, { trigger: true });
        },
        statusCode: {
          404: function () {
              window.alertify.error("Ошибка");
          }
        }
      });
    },

    onShowLogClick: function() {
        $.ajax({
            type: "GET",
            url: "rest/optimizations/" + this.model.id + "/extractLogs",
            success: function(){
                var url = location.hash;
                window.App.navigate(url, { trigger: true });
                window.alertify.success("Запрос отправлен");
            },
            statusCode: {
                404: function () {
                    window.alertify.error("Ошибка");
                }
            }
        });
    },

    onClickNavItem: function (e) {
      var $target = $(e.target).closest("li");
      if (!$target.hasClass("active")) {
        var route = "#project/" + this.options.modelName + "/" + this.model.id + "/" + $target.data("route") + "/view";
        if ($target.data("parameters")) route = route + "/" + $target.data("parameters");
        window.App.navigate(route, { trigger: true });
      }
    },

    onSaveClick: function () {
      var that = this;
      this.model.save(null, {
        success: function () {
          var url = "#project/" + that.options.modelName + "/" + that.model.id + "/" + that.page + "/view";
          if (that.options.tab) url = url + "/" + that.options.tab;
          window.App.navigate(url, { trigger: true });
        },
        error: function (xhr) {
          console.error("Failed to save: ", xhr);
        }
      });
    },

    initAndShowControlsView: function (region) {
      this.controlsView = new window.App.Views.BaseControlsView({
        layoutView: this,
        model: this.model,
        controls: this.controls,
        mode: this.mode
      });
      region.show(this.controlsView);
    },

    templateHelpers: function () {
      if (this.options.modelName === "technologies") {
        return {
            active: this.page,
            tType: this.model.get("type")
        }
      }
      return {
        active: this.page
      }
    }

  });

})();