(function () {
  "use strict";

  var AddLayerView = Marionette.LayoutView.extend({
    template: "./views/templates/project/composite_material/composite_material_add_layer.html.ejs",
    ui: {
      position:   ".js-position",
      thickness:  ".js-thickness",
      angle:      ".js-angle"
    },

    initialize: function () {
      this.listenTo(this, "ok", this.onSubmit);
    },

    onSubmit: function (md) {
      md.preventClose();
      var isValid = true;
      _.each(this.ui, function($item) {
        if (_.isEmpty($item.val())) {
          isValid = false;
          $item.closest(".form-group").removeClass("has-success").addClass("has-error");
        } else {
          $item.closest(".form-group").removeClass("has-error").addClass("has-success");
        }
      });
      if (isValid) {
        this.trigger("layer:added", {
          position: parseInt(this.ui.position.val()) - 1,
          thickness: parseFloat(this.ui.thickness.val()),
          angle: parseFloat(this.ui.angle.val())
        });
        md.close();
      }
    }

  });

  window.App.Views.CompositeMaterialLayersView = Marionette.LayoutView.extend({
    template: "./views/templates/project/composite_material/composite_material_layers.html.ejs",
    className: "row",
    ui: {
      addLayer:               ".js-add-layer",
      remove:                 ".js-remove",
      toggleSymmetricalLayer: ".js-simmetrycal-layers"
    },

    events: {
      "click @ui.addLayer":               "onAddLayerClick",
      "click @ui.remove":                 "onRemoveClick",
      "click @ui.toggleSymmetricalLayer": "onToggleSymmetricalLayer"
    },

    initialize: function () {
      this.listenTo(this.collection, "sync", this.render);
    },

    onShow: function () {
      this.$(".ui-sortable-layers").sortable({ axis: "x" });
    },

    onAddLayerClick: function () {
      this.addLayerView = new AddLayerView();
      this.modal = new Backbone.BootstrapModal({
        content: this.addLayerView,
        title: "Новый слой",
        animate: true,
        cancelText: "Отменить",
        okText: "Сохранить"
      }).open();
      this.listenTo(this.addLayerView, "layer:added", this.addLayer);
    },

    addLayer: function (options) {
      var that = this;
      var newLayer = new window.App.Models.Layer({ compositeMaterialId: this.model.get("id") });
      newLayer.set(options);
      newLayer.save({}, {
        success: function () {
          that.collection.fetch();
        },
        error: function (model, xhr) {
          if (xhr.status == 200) that.collection.fetch();
        }
      });
    },

    onRemoveClick: function (e) {
      var that = this;
      var id = $(e.target).data("id");
      var model = this.collection.findWhere({ id: id });
      model.destroy({
        dataType: "text",
        success: function () {
          that.collection.fetch();
        },
        error: function (xhr) {
          if (xhr.status == 200) that.collection.fetch();
        }
      });
    },

    onToggleSymmetricalLayer: function (e) {
      this.model.save({ symmetricalLayers: !this.model.get("symmetricalLayers") });
    },

    templateHelpers: function () {
      return {
        layers: this.collection.toJSON()
      }
    }
  });

}());