(function () {
  "use strict";

  window.App.Views.CreateCompositeMaterialView = window.App.Views.CreateMaterialView.extend({
    template: "./views/templates/project/composite_material/create_composite_material.html.ejs",

    behaviors: {
      PluginsInit: {
        select2Events: [],
        select2: [
          new window.App.Helpers.Select2Config({
            el: ".js-technology",
            placeholder: "Выберите технологию",
            url: "rest/technologies/suggest",
            isNumber: false,
            allowClear: true,
            minimumInputLength: 0
          })
        ]
      }
    },

    ui: _.extend({
      technology: ".js-technology",
      equipment:  ".js-equipment"
    }, window.App.Views.CreateMaterialView.prototype.ui),

    events: _.extend({
      "change @ui.technology":            "onTechnologyChange",
      "select2-removed @ui.technology":   "onTechnologyClear"
    }, window.App.Views.CreateMaterialView.prototype.events),

    initialize: function (options) {
      this.options = options || {};
      window.App.Views.CreateMaterialView.prototype.initialize.apply(this, arguments);
      this.model = new window.App.Models.CompositeMaterial();
    },

    onPathClick: function () {
      this.$(".js-main-region").addClass("hidden");
      this.md.$el.find(".btn").addClass('disabled');
      this.pathModalView = new window.App.Views.CreateModalPath({ treeType: "compositeMaterials", onlyFolders: true });
      this.getRegion("treeRegion").show(this.pathModalView);
      this.listenTo(this.pathModalView, "path:select:canceled", this.pathSelectCanceled);
      this.listenTo(this.pathModalView, "path:select:done", this.pathSelectDone);
    },

    onTechnologyChange: function () {
      var technologyId = this.ui.technology.val();
      if (technologyId) {
        this.ui.equipment.prop("disabled", false);
        this.initializeEquipmentSuggest(technologyId)
      } else {
        if (this.ui.equipment.data('select2')) {
          this.ui.equipment.select2("destroy");
        }
        this.ui.equipment.prop("disabled", true);
      }
    },

    onSubmit: function(md) {
      md.preventClose();
      var name = this.ui.name.val();

      if (_.isEmpty(name))
        this.ui.name.closest(".form-group").addClass("has-error");
      else if (_.isNull(this.pid))
        this.ui.path.closest(".form-group").addClass("has-error");
      else {
        this.model.save({
          name: name,
          pid: this.pid,
          manufacture: this.ui.manufacture.val(),
          source: this.ui.source.val(),
          technologyId: this.ui.technology.val() ? this.ui.technology.val() : null,
          equipmentId: this.ui.technology.val() ? this.ui.equipment.val() : null
        }, {
          success: function(model, resp) {
            window.App.vent.trigger('updateTreeNodes', {id: model.get("pid")}, 'compositeMaterials', window.App.Data.EntityTypes[3].type, "add");
            window.App.navigate("#project/compositeMaterials/" + model.get("id") + "/parameters/edit", { trigger: true });
            md.close();
          }
        });
      }
    },

    initializeEquipmentSuggest: function (technologyId) {
      if (this.ui.equipment.data('select2')) {
        this.ui.equipment.select2("destroy");
      }
      this.ui.equipment.prop("disabled", false);
      this.ui.equipment.select2(
        new window.App.Helpers.Select2Config({
          el: ".js-equipment",
          placeholder: "Выберите оборудование",
          url: "rest/technologies/" + technologyId + "/equipments/suggest",
          isNumber: false,
          allowClear: true,
          minimumInputLength: 0
        })
      );
    },

    templateHelpers: function () {
      return {
        selectedPid: this.options.treeNode ? this.options.treeNode.id : "",
        nodePath: this.options.treeNode ? this.options.treeNode.path : "",
      }
    }

  });

}());