(function () {
  "use strict";

  window.App.Views.CreateConstructionElementView = Marionette.LayoutView.extend({
    template: "./views/templates/project/construction_element/create_construction_element.html.ejs",

    behaviors: {
      PluginsInit: {
        fileupload: {
          el: "#js-file-upload"
        }
      }
    },

    regions: {
      treeRegion: ".js-tree-region",
      mainRegion: ".js-main-region"
    },

    ui: {
      name:         ".js-name",
      path:         ".js-path",
      pathFormInp:  ".js-path-actual",
      fileForm:     ".js-file-form",
      fileInp:      "#js-file-upload"
    },

    events: {
      "click @ui.path": "onPathClick"
    },

    initialize: function (options) {
      this.options = options || {};
      this.pid = null;
      this.model = new window.App.Models.ConstructionElement();
      this.listenTo(this, "cancel", this.onCancel);
      this.listenTo(this, "ok", this.onSubmit)
    },

    onPathClick: function () {
      this.$(".js-main-region").addClass("hidden");
      this.md.$el.find(".btn").addClass('disabled');
      this.pathModalView = new window.App.Views.CreateModalPath({ treeType: "constructionElements", onlyFolders: true });
      this.getRegion("treeRegion").show(this.pathModalView);
      this.listenTo(this.pathModalView, "path:select:canceled", this.pathSelectCanceled);
      this.listenTo(this.pathModalView, "path:select:done", this.pathSelectDone);
    },

    pathSelectCanceled: function () {
      this.$(".js-main-region").removeClass("hidden");
      this.md.$el.find(".btn").removeClass('disabled');
      this.getRegion("treeRegion").empty();
    },

    pathSelectDone: function (treeNode) {
      this.pathSelectCanceled();
      this.ui.pathFormInp.val(treeNode.id);
      this.ui.path.val(window.App.Helpers.ShowPath(treeNode.path));
    },

    passModal: function (md) {
      this.md = md;
    },

    onCancel: function() {
      window.App.navigate("#project/constructionElements", { trigger: true });
    },

    uploadCad: function (md) {
      this.ui.fileForm.ajaxForm({
        "url": "rest/constructionElements",
        type: "POST",
        success: function (model) {
          window.App.vent.trigger('updateTreeNodes', {id: model.pid}, window.App.Data.Trees[2].type, window.App.Data.EntityTypes[6].type, "add");
          window.App.navigate("#project/constructionElements/" + model.id + "/parameters/view", { trigger: true });
          md.close();
        }
      }).submit();
    },

    onSubmit: function(md) {
      md.preventClose();
      var name = this.ui.name.val();
      var pid = this.ui.pathFormInp.val();
      var fileInp = this.ui.fileInp.val();

      if (_.isEmpty(name))
        this.ui.name.closest(".form-group").addClass("has-error");
      else if (_.isEmpty(pid))
        this.ui.pathFormInp.closest(".form-group").addClass("has-error");
      else if (_.isEmpty(fileInp))
        this.ui.fileInp.closest(".form-group").addClass("has-error");
      else
        this.uploadCad(md);
    },

    templateHelpers: function () {
      return {
        nodeId: this.options.treeNode ? this.options.treeNode.id : "",
        nodePath: this.options.treeNode ? this.options.treeNode.path : ""
      }
    }

  });

}());