(function () {
  "use strict";

  window.App.Views.OptimizationRunModal = window.App.Views.BaseRunModal.extend({
    template: "./views/templates/project/modals/optimization_run_modal.html.ejs",

    initializeSolverDropdown: function () {
      var config = new window.App.Helpers.Select2Config({
        el: ".js-node",
        placeholder: "Выберите расчетный узел",
        url: "rest/nodes/suggest",
        isNumber: false,
        allowClear: true,
        minimumInputLength: 0,
        queries: [{column: 'clusterId', value: parseInt(this.ui.cluster.val())}],
      });

      this.ui.node.select2(config);
    },

    _hideFields: function (type) {
      //number of fields to hide
      _.each(['isParallel', 'numOfNodes'], function (v, i) {
        this.toggleFieldHide(v, !this.isSlurm);
      }, this);
    },

    _disableFields: function (type) {
      //number of fields to disable
      _.each(['node'], function (v, i) {
        this.toggleFieldDisable(v, this.isSlurm);
      }, this);
    },

    onChangeCluster: function () {
      this.ui.node.val(null).trigger('change');
      this.initializeSolverDropdown();
      var selectedData = this.ui.cluster.select2('data');
      this.isSlurm = (selectedData[0] && selectedData[0].type) === 'Slurm';

      this._hideFields();
      this._disableFields();
    },


    initializeClusterDropdown: function () {
      var that = this;
      var onGETSuccess = function (data) {
        that.ui.cluster.select2({
          placeholder: 'Выберите кластер',
          allowClear: true,
          data: data
        }).on('change', that.onChangeCluster.bind(that));
        that.onChangeCluster();
        that.initializeSolverDropdown();
      };
      $.get('rest/clusters', onGETSuccess);
    },

    onRender: function () {
      this.initializeClusterDropdown();
    },

    validateForm: function () {
      var requiredFields = ['clusterId', 'nodeId'];
      var clusterId = this.ui.cluster.val();
      var nodeId = this.ui.node.val();
      var cores = this.ui.cores.val();
      var memory = this.ui.memory.val();
      var numOfNodes = this.ui.numOfNodes.val();
      var isParallel = this.ui.isParallel.prop("checked");

      var data = {
        "clusterId": parseInt(clusterId),
        "nodeId": parseInt(nodeId),
        "numOfCores": parseInt(cores),
        "memory": parseInt(memory),
        "isParallel": isParallel,
        "numOfNodes": parseInt(numOfNodes),
        "parentName": this.options.parentName,
        "parentType": this.options.parentType,
        "parentId": this.options.parentId
      };

      var valid = !_.findKey(data, function (v, k) {
        if (!v && requiredFields.indexOf(k) !== -1) {
          if (k === 'nodeId' && this.isSlurm) {
            this.ui.node.closest(".form-group").removeClass("has-error");
            return false;
          }
          console.warn("Required field ", k, " had empty value");
          this.ui[k.substr(0, k.indexOf('Id'))] && this.ui[k.substr(0, k.indexOf('Id'))].closest(".form-group").addClass("has-error");
          return true;
        }
        return false
      }, this);

      return valid ? data : valid;
    },

    onSubmit: function (md) {
      md.preventClose();
      var task = new window.App.Models.Task(),
        data = this.validateForm();

      if (!data) {
        console.log("Collected data is not valid");
        return;
      }

      task.save(data, {
        success: function () {
          window.alertify.success("Задача успешно создана");
          md.close();
          window.App.navigate("project/queue", {trigger: true});
        }
      });
    }

  });

})();