(function () {
  "use strict";

  window.App.Views.CreateOptimizationFolderView = Marionette.LayoutView.extend({
    template: "./views/templates/project/optimization/create_optimization_folder.html.ejs",
    className: "create-optimization row",

    regions: {
      mainRegion: ".js-main-region",
      treeRegion: ".js-tree-region"
    },

    behaviors: {
      PluginsInit: {
        fileupload: {
          el: "#js-file-upload"
        },
        select2: [
          new window.App.Helpers.Select2Config({
            el: ".js-optimization-type",
            placeholder: "Выберите тип",
            url: "rest/optimizationTypes/suggest",
            isNumber: false,
            allowClear: true,
            minimumInputLength: 0
          }),
          new window.App.Helpers.Select2Config({
            el: ".js-optimization",
            placeholder: "Выберите оптимизационный расчет",
            url: "rest/optimizations/suggest",
            isNumber: false,
            allowClear: true,
            minimumInputLength: 0
          })
        ]
      }
    },

    ui: {
      form: 'form',
      name: ".js-name",
      optimizationType: ".js-optimization-type",
      optimization : ".js-optimization"
    },

    events: {
      "change @ui.optimizationType": "onChangeOptimizationType"
    },

    initialize: function (options) {
      this.options = options || {};
      this.initDialogModel();

      this.listenTo(this, "ok", this.onSubmit);
      this.listenTo(this, "cancel", this.onCancel);
    },

    initDialogModel: function () {

      var that = this;

      this.model = new Backbone.Model();
      this.model.validate = function (attrs, options) {
        var errors = [];

        if (typeof(attrs.name) !== "undefined") {
          if (!attrs.name) {
            errors.push('name');
          }
        }
        if (typeof(attrs.optimizationTypeId) !== "undefined") {
          if (isNaN(attrs.optimizationTypeId)) {
            errors.push('optimizationType');
          }
        }

        if(attrs.parentType !== "ConstructionElement" && typeof(attrs.baseOptimizationId) !== "undefined") {
          if (isNaN(attrs.baseOptimizationId)) {
            errors.push('baseOptimizationId');
          }
        }

        if (errors.length) {
          return errors;
        }
      };

      this.model.on('invalid', function (model, error) {
        console.log("error", error);
        that.ui.form.validator('validate')
      });

      this.model.url = function () {
        return "rest/tree/" + that.options.params.treeId;
      };
      this.model.set(this.options.params);
    },

    passModal: function (md) {
      this.md = md;
    },

    onCancel: function () {
      window.App.navigate("#project/optimizations", {trigger: true});
    },

    onSubmit: function (md) {
      md.preventClose();
      var that = this;

      this.model.set({
        baseOptimizationId: parseInt(this.ui.optimization.val()),
        optimizationTypeId: parseInt(this.ui.optimizationType.val()),
        name: this.ui.name.val()
      }).save(null,{
          success: function (model, resp) {
            window.App.vent.trigger('updateTreeNodes', {id: model.get("pid")}, 'optimizations', window.App.Data.EntityTypes[8].type, "add");
            that.onCancel();
            md.close();
          }
        }
      );


      //var name = this.ui.name.val();
      //
      //if (_.isEmpty(name))
      //  this.ui.name.closest(".form-group").addClass("has-error");
      //else if (_.isNull(this.selectedPid))
      //  this.ui.path.closest(".form-group").addClass("has-error");
      //else {
      //  this.model.save({
      //    name: name,
      //    pid: this.selectedPid
      //  }, {
      //    success: function (model, resp) {
      //      window.App.navigate("#project/optimizations/" + model.get("id") + "/parameters/edit", { trigger: true });
      //      md.close();
      //    },
      //    error: function () {
      //
      //    }
      //  });
      //}


    },

    templateHelpers: function(){
      return{
        rootDirectory : this.model.get('parentType') === "ConstructionElement" ? true : false
      }
    }

  });

}());