(function () {
  "use strict";

  window.App.Views.CreateSliceChartView = Marionette.LayoutView.extend({
    template: "./views/templates/project/optimization/create_slice_chart.html.ejs",
    className: "create-slice-chart-view row",

    ui: {
      varyingParameter: ".js-varying-parameter",
      targetFunction:   ".js-target-function",
      border:           ".js-border",
      bound:            ".js-bound",
      creationBlock:    ".js-create-chart",
      chartBlock:       ".js-chart"
    },

    events: {
      "change @ui.varyingParameter":  "onChangeVaryingParameter",
      "change @ui.targetFunction":    "onChangeTargetFunction",
      "change @ui.bound":             "onChangeBound"
    },

    initialize: function (options) {
      this.options = options || {};
      this.data = {
        varyingParamId: null,
        targetFuncId: null,
        intervals: _.map(this.options.borders, _.clone)
      };
      this.listenTo(this, "ok", this.onSubmit);
      this.listenTo(this, "cancel", this.onCancel);
    },

    onChangeVaryingParameter: function () {
      let id = this.ui.varyingParameter.val();
      this.ui.border.removeClass("hidden");

      if (id !== "none") {
        this.$(".js-border[data-id='" + id + "']").addClass("hidden");
        this.data.varyingParamId = parseInt(id);
      } else {
        this.data.varyingParamId = null;
      }
    },

    onChangeTargetFunction: function () {
      let id = this.ui.targetFunction.val();
      this.data.targetFuncId = id === "none" ? null : parseInt(id);
    },

    onChangeBound: function (e) {
      let $target = $(e.target);
      let type = $target.data("bound");
      let borderIndex = $target.closest(".js-border").data("index");
      let value = $target.val();
      value = _.isEmpty(value) ? this.options.borders[borderIndex][type + "Bound"] : parseFloat(value);

      if (!_.isNull(this.options.borders[borderIndex].lowerBound) && value < this.options.borders[borderIndex].lowerBound) {
        value = this.options.borders[borderIndex].lowerBound;
      }

      if (!_.isNull(this.options.borders[borderIndex].upperBound) && value > this.options.borders[borderIndex].upperBound) {
        value = this.options.borders[borderIndex].upperBound;
      }

      this.data.intervals[borderIndex][type + "Bound"] = value;
      $target.val(value);
    },

    onRender: function () {

    },

    passModal: function (md) {
      this.md = md;
    },

    onSubmit: function (md) {
      md.preventClose();
      let isValid = true;

      if (_.isNull(this.data.varyingParamId)) {
        this.ui.varyingParameter.closest(".form-group").addClass("has-error");
        isValid = false;
      } else {
        this.ui.varyingParameter.closest(".form-group").removeClass("has-error");
      }

      if (_.isNull(this.data.targetFuncId)) {
        this.ui.targetFunction.closest(".form-group").addClass("has-error");
        isValid = false;
      } else {
        this.ui.targetFunction.closest(".form-group").removeClass("has-error");
      }

      if (isValid) {
        $.ajax({
          type: "POST",
          url: "rest/optimizations/" + this.options.optimizationId + "/" + this.options.nameCAO + "/slice-chart",
          contentType: "application/json",
          data: JSON.stringify(this.data),
          success: (curve) => {
            new Backbone.BootstrapModal({
              content: new window.App.Views.SliceChartView({ curve: curve }),
              title: "Просмотр",
              animate: true,
              allowCancel: false,
              cancelText: "Отменить",
              okText: "Закрыть",
              modalWidth: $(window).width() * 0.8
            }).open();
            md.close();
          }
        });
      }
    },

    templateHelpers: function () {
      return {
        functions: this.options.functions,
        borders: this.options.borders
      }
    }

  });

}());