(function () {
  "use strict";

  window.App.Views.OptimizationParametersPage = window.App.Views.BasePage.extend({
    template: "./views/templates/project/optimization/optimization_parameters_page.html.ejs",
    regions: _.extend({
      "firstTableRegion": ".js-first-table-region",
      "secondTableRegion": ".js-second-table-region",
    }, window.App.Views.BasePage.prototype.regions),

    ui: _.extend({
      minimize: ".js-minimize",
      firstTableWrap: ".js-first-table-wrap",
      secondTableWrap: ".js-second-table-wrap",
      hideTable: ".js-hide-table",
      firstTableBlockTitle: ".js-first-table-block-title",
      secondTableBlockTitle: ".js-second-table-block-title"
    }, window.App.Views.BasePage.prototype.ui),

    events: _.extend({
      "click @ui.minimize": "onMinimizeClick",
      "click @ui.hideTable": "onHideClick"
    }, window.App.Views.BasePage.prototype.events),

    initialize: function () {
      window.App.Views.BasePage.prototype.initialize.apply(this, arguments);
      this.isView = this.options.mode === "view";
      if (this.model.get("optimizationTypeId") === 1) {
        this.controls.push("run")
      }
    },

    onNewClick: function () {
      window.App.navigate("#project/" + this.options.modelName +  "/create?referenceId=" + this.model.get("id"), { trigger: true });
    },

    onRender: function () {
      this.initAndShowMainView();
      this.initAndShowDescriptionView();
      this.initAndShowFlowView();
      window.App.Views.BasePage.prototype.onRender.call(this);
    },

    onMinimizeClick: function (e) {
      $(e.target).closest(".ui-collapsable-block").toggleClass("minimized");
      $(e.target).closest(".ui-minimize").find("span").toggle();
    },

    onHideClick: function () {
      this.getRegion("tableRegion").empty();
      this.ui.tableWrap.addClass("hidden");
    },

    onFlowBlockClick: function (type) {
      var config = {
        isFile: false,
        title: "Файлы",
        isTableView: true,
        view: undefined,
        actions: []
      };
      switch(type) {
        case "models":
          config.title = "CAE-Файлы";
          config.parsedType = "CAEFiles"; // Currently CAEs only, further can be CAD and CAO also
          config.isFile = true;
          config.actions = ["add-model-to-optimization", "remove-from", "export"];
          break;
        case "dependencies":
          config.title = "Материалы | Технологии | Условия нагружения";
          config.isTableView = false;
          config.view = new window.App.Views.OptimizationDependenciesView({ model: this.model });
          break;
        case "tasks":
          config.title = "Расчеты";
          config.isTableView = false;
          return false;
          break;
        case "results-all":
          config.title = "Ключевые результаты";
          config.isTableView = false;
          config.view = new window.App.Views.ResultsListView(_.extend({
            gridCollection: new window.App.Collections.Results([], {
              parentType: "optimizations",
              parentId: this.model.id,
              pageable: true,
              filters: this.options.filters || null
            }),
            actions: [],
            title: "Результаты",
            type: "result"
          }, {}));
          break;
        case "pictures":
          config.title = "Ключевые результаты";
          config.isTableView = false;
          config.view = new window.App.Views.ResultsListView(_.extend({
            gridCollection: new window.App.Collections.ResultsPictures([], {
              parentType: "optimizations",
              parentId: this.model.id,
              pageable: true,
              filters: this.options.filters || null
            }),
            actions: [],
            title: "Результаты",
            type: "result"
          }, {}));
          break;
        case "targetValues":
          config.title = "Ключевые результаты";
          config.isTableView = false;
          config.view = new window.App.Views.ResultsListView(_.extend({
            gridCollection: new window.App.Collections.ResultsTargetValues([], {
              parentType: "optimizations",
              parentId: this.model.id,
              pageable: true,
              filters: this.options.filters || null
            }),
            actions: [],
            title: "Результаты",
            type: "result"
          }, {}));
          break;
        case "logFiles":
            config.title = "Ключевые результаты";
            config.isTableView = false;
            config.view = new window.App.Views.ResultsListView(_.extend({
                gridCollection: new window.App.Collections.ResultsLogFiles([], {
                    parentType: "optimizations",
                    parentId: this.model.id,
                    pageable: true,
                    filters: this.options.filters || null
                }),
                actions: [],
                title: "Результаты",
                type: "result"
            }, {}));
            break;
        case "dbFiles":
          config.title = "Ключевые результаты";
          config.isTableView = false;
          config.view = new window.App.Views.ResultsListView(_.extend({
              gridCollection: new window.App.Collections.ResultsDbFiles([], {
                  parentType: "optimizations",
                  parentId: this.model.id,
                  pageable: true,
                  filters: this.options.filters || null
              }),
              actions: [],
              title: "Результаты",
              type: "result"
          }, {}));
          break;
        case "curves":
            config.title = "Ключевые результаты";
            config.isTableView = false;
            config.view = new window.App.Views.ResultsListView(_.extend({
                gridCollection: new window.App.Collections.ResultsCurves([], {
                    parentType: "optimizations",
                    parentId: this.model.id,
                    pageable: true,
                    filters: this.options.filters || null
                }),
                actions: [],
                title: "Результаты",
                type: "result"
            }, {}));
            break;
        case "animations":
            config.title = "Ключевые результаты";
            config.isTableView = false;
            config.view = new window.App.Views.ResultsListView(_.extend({
                gridCollection: new window.App.Collections.ResultsAnimations([], {
                    parentType: "optimizations",
                    parentId: this.model.id,
                    pageable: true,
                    filters: this.options.filters || null
                }),
                actions: [],
                title: "Результаты",
                type: "result"
            }, {}));
            break;
        case "stlVisualisations":
            config.title = "Ключевые результаты";
            config.isTableView = false;
            config.view = new window.App.Views.ResultsListView(_.extend({
                gridCollection: new window.App.Collections.ResultsStlVisualisations([], {
                    parentType: "optimizations",
                    parentId: this.model.id,
                    pageable: true,
                    filters: this.options.filters || null
                }),
                actions: [],
                title: "Результаты",
                type: "result"
            }, {}));
            break;
        case "reports":
          alert("В разработке");
          return false;
          break;
        default:
          console.error("Error: " + type + " is wrong object type");
          return false;
      }

      if (config.isTableView) {
        config.view = new window.App.Views.FilesListView(_.extend({
          gridCollection: new window.App.Collections.Files([], {
            objectId: this.model.id,
            objectType: "optimizations",
            fileType: config.parsedType,
            pageable: true,
            filters: this.options.filters || null,
            optimizationId: this.model.id
          }),
          actions: config.actions,
          isFile: config.isFile,
          title: config.title
        }, {}));
      }

      if (config.view) {
        this.ui.firstTableWrap.removeClass("hidden");
        this.ui.firstTableBlockTitle.text(config.title);
        this.getRegion("firstTableRegion").show(config.view)
      }

      if (type === "models" && !_.isNull(this.model.get("solverCadId"))) {
        this.ui.secondTableWrap.removeClass("hidden");
        this.ui.secondTableBlockTitle.text("CAD-Файлы");
        this.getRegion("secondTableRegion").show(new window.App.Views.FilesListView(_.extend({
          gridCollection: new window.App.Collections.Files([], {
            objectId: this.model.id,
            objectType: "optimizations",
            fileType: "CADFiles",
            pageable: true,
            filters: this.options.filters || null,
            optimizationId: this.model.id
          }),
          actions: config.actions,
          isFile: config.isFile,
          title: "CAD-Файлы"
        }, {})))
      }

    },

    initAndShowFlowView: function () {
      ReactDOM.render(
        React.createElement(window.OptimizationFlow, { onFlowBlockClick: _.bind(this.onFlowBlockClick, this), optimization: this.model, isView: this.isView}),
        this.$(".js-flow-region")[0]
      );
    },

    initAndShowMainView: function () {
      ReactDOM.render(
        React.createElement(window.OptimizationMain, { optimization: this.model, isView: this.isView }),
        this.$(".js-main-region")[0]
      );
    },

    initAndShowDescriptionView: function () {
      ReactDOM.render(
        React.createElement(window.OptimizationDescription, { optimization: this.model, isView: this.isView }),
        this.$(".js-description-region")[0]
      );
    }

  });

})();