(function () {
  "use strict";

  window.App.Views.ProjectQueuePage = window.App.Views._BasePage.extend({
    template: "./views/templates/project/queue/queue_page.html.ejs",
    className: "ui-queue-view",
    regions: _.extend(
      window.App.Views.BasePage.prototype.regions,
      {
        "statisticsRegion": ".js-statistics-region",
        "queueRegion":      ".js-queue-region",
        "runningRegion":    ".js-running-region",
        "historyRegion":    ".js-history-region"
      }
    ),

    tables: {
      queue: true,
      running: true,
      history: false
    },

    views: {
      "controls": window.App.Views.QueueControls,
      "statistics": window.App.Views.QueueStatisticsWrap
    },

    initialize: function () {
      window.App.Views._BasePage.prototype.initialize.apply(this, arguments);
      this.listenTo(this, "toggle:shown:table", _.bind(this.toggleShownTable, this));
    },

    onRender: function () {
      var that = this;
      _.each(this.tables, function (isShown, tableName) {
        if (isShown) that.showTable(tableName);
      });
      window.App.Views._BasePage.prototype.onRender.call(this, arguments);
    },

    toggleShownTable: function (tableName) {
      var isShown = this.tables[tableName];

      if (isShown) {
        this.tables[tableName] = false;
        this.hideTable(tableName);
      } else {
        this.tables[tableName] = true;
        this.showTable(tableName);
      }
    },

    hideTable: function (tableName) {
      this.getRegion(tableName + "Region").empty();
    },

    showTable: function (tableName) {
      this.getRegion(tableName + "Region").show(new window.App.Views.QueueTableWrap({ tableName: tableName }));
    }
  });

}());
