(function () {
  "use strict";

  window.App.Views.CreateSimulationView = Marionette.LayoutView.extend({
    template: "./views/templates/project/simulation/create_simulation.html.ejs",
    behaviors: function () {
      {
        return {
          PluginsInit: {
            select2Events: [],
            select2: [

              window.App.Helpers.Select2Config({
                el: ".js-effective-characteristic",
                placeholder: "Эффективная характеристика",
                url: "rest/compositeMaterials/" + this.options.compositeMaterialId +  "/effectiveCharacteristics/suggest",
                isNumber: false,
                allowClear: true,
                minimumInputLength: 0
              }),
              window.App.Helpers.Select2Config({
                el: ".js-type",
                placeholder:"Вид испытания",
                url: "rest/simulations/types/suggest",
                isNumber: false,
                allowClear: true,
                minimumInputLength: 0
              })
            ]
          }
        }
      }
    },

    ui: {
      "name": ".js-name",
      "effectiveCharacteristic": ".js-effective-characteristic",
      "type": ".js-type"
    },


    initialize: function (options) {
      this.options = options || {};
      this.model = new window.App.Models.Simulation();
      this.listenTo(this, "ok", this.onSubmit);
      this.listenTo(this, "cancel", this.onCancel);
    },

    passModal: function (md) {
      this.md = md;
    },

    onCancel: function () {
      window.App.navigate("#project/compositeMaterials/" + this.options.compositeMaterialId + "/simulations/view", {trigger: true});
    },

    onSubmit: function (md) {
      md.preventClose();
      var name = this.ui.name.val();
      var effectiveCharacteristic = this.ui.effectiveCharacteristic.val();
      var type = this.ui.type.val();

      if (_.isEmpty(name))
        this.ui.name.closest(".form-group").addClass("has-error");
      else if (_.isEmpty(effectiveCharacteristic))
        this.ui.effectiveCharacteristic.closest(".form-group").addClass("has-error");
      else if (_.isEmpty(type))
        this.ui.type.closest(".form-group").addClass("has-error");
      else {
        this.$(".has-error").removeClass("has-error");
        this.model.save({
          name: name,
          ecId: effectiveCharacteristic,
          typeId: type,
          compositeMaterialId: this.options.compositeMaterialId
        }, {
          success: function (model, resp) {
            window.App.vent.trigger('updateTreeNodes', {id: model.get("pid")}, 'compositeMaterials', window.App.Data.EntityTypes[3].type, "add");
            window.App.navigate("#project/simulations/" + model.get("id") + "/parameters/edit", {trigger: true});
            md.close();
          }
        });
      }
    }

  });

}());