
(function () {
  "use strict";
  window.App.Views.SimulationResultsPage = window.App.Views.BasePage.extend({
    template: "./views/templates/project/simulation/simulation_results_page.html.ejs",

    regions: _.extend({
      "resultsRegion": ".js-results-region"
    }, window.App.Views.BasePage.prototype.regions),

    ui: _.extend({
      "run": ".js-run",
      "parameters": ".js-parameters"
    }, window.App.Views.BasePage.prototype.ui),

    events: _.extend({
      "click @ui.run": "onRunClick",
      "click @ui.parameters": "onParametersClick"
    }, window.App.Views.BasePage.prototype.events),

    initialize: function (options) {
      window.App.Views.BasePage.prototype.initialize.apply(this, arguments);
      this.controls = ["run", "parameters"];
    },

    onRender: function () {
      this.initAndShowMainView(this.getRegion("mainRegion"));
      this.initAndShowResultsView(this.getRegion("resultsRegion"));
      window.App.Views.BasePage.prototype.onRender.call(this);
    },


    initAndShowMainView: function (region) {
      this.mainView = new window.App.Views.SimulationMainView({
        layoutView: this,
        mode: this.mode,
        model: this.model
      });
      region.show(this.mainView);
    },

    initAndShowControlsView: function (region) {
      this.controlsView = new window.App.Views.SimulationControlsView({
        layoutView: this,
        title: this.model.get("name"),
        controls: this.controls,
        mode: this.mode,
        model: this.model
      });
      region.show(this.controlsView);
    },

    initAndShowResultsView: function () {
      var gridCollection = new window.App.Collections.Results([], {
        parentType: "simulations",
        parentId: this.model.id,
        pageable: true,
        filters: this.options.filters || null
      });

      this.resultsView = new window.App.Views.ResultsListView(_.extend({
        gridCollection: gridCollection,
        actions: ["delete", "export"],
        title: "Результаты",
        type: "result"
      }, {}));
      this.getRegion("resultsRegion").show(this.resultsView);
    },

    onParametersClick: function () {
      window.App.navigate("project/simulations/" + this.model.id + "/parameters/view", { trigger: true });
    },

    onRunClick: function () {
      this.runModalView = new window.App.Views.BaseRunModal({
        "parentType": "simulation",
        "parentName": this.model.get("name"),
        "parentId": this.model.id
      });
      this.runModal = new Backbone.BootstrapModal({
        content: this.runModalView,
        title: "Запуск расчета",
        animate: true,
        cancelText: "Отменить",
        okText: "Запустить"
      }).open();
      this.runModalView.passModal(this.runModal);
    }

  });

})();