(function () {
  "use strict";
  window.App.Views.TechnologyParametersPage = window.App.Views.BasePage.extend({
    template: "./views/templates/project/technology/technology_parameters_page.html.ejs",

    regions: _.extend({
      defectsRegion:    ".js-defects-region .ui-block",
      equipmentsRegion: ".js-equipments-region .ui-block",
      parametersRegion: ".js-parameters-region .ui-block",
      materialsRegion:  ".js-materials-region .ui-block"
    }, window.App.Views.BasePage.prototype.regions),

    onRender: function () {
      this.initAndShowMainView(this.getRegion("mainRegion"));
      this.initChildTableView({
        region: this.getRegion("defectsRegion"),
        ListView: window.App.Views.TechnologyDefectsListView,
        Collection: App.Collections.TechnolgyDefects,
        type: "technologyDefects",
        actions: ["new", "delete"]
      });
      this.initChildTableView({
        region: this.getRegion("equipmentsRegion"),
        ListView: window.App.Views.TechnologyEquipmentsList,
        Collection: App.Collections.TechnologyEquipments,
        type: "technologyEquipments",
        actions: ["new", "delete"]
      });
      this.initChildTableView({
        region: this.getRegion("parametersRegion"),
        ListView: window.App.Views.TechnologyParametersList,
        Collection: App.Collections.TechnologyParameters,
        type: "technologyParameters",
        actions: ["new", "delete"]
      });
      this.initChildTableView({
        region: this.getRegion("materialsRegion"),
        ListView: window.App.Views.TechnologyMaterialsList,
        Collection: App.Collections.TechnologyCompositeMaterials,
        type: "technologyCompositeMaterials",
        actions: [],
        showableColumns: this.model.get("type") !== "ADDITIVE_3CM" ? ["name", "fiber", "matrix", "owner"] : undefined
      });
      window.App.Views.BasePage.prototype.onRender.call(this);
    },

    initAndShowMainView: function (region) {
      this.mainView = new window.App.Views.TechnologyMainView({ layoutView: this, mode: this.mode, model: this.model});
      region.show(this.mainView);
    },

    initChildTableView: function (options) {
      options.region.show(new options.ListView({
        gridCollection: new options.Collection([], {
          pageable: true,
          filters: {},
          technologyId: this.model.id
        }),
        actions: options.actions,
        type: options.type,
        showableColumns: options.showableColumns,
      }, {}));
    },

    initAndShowControlsView: function (region) {
      this.controlsView = new window.App.Views.TechnologyControlsView({
        layoutView: this,
        model: this.model,
        controls: this.controls
      });
      region.show(this.controlsView);
    }

  });

})();