(function () {
  "use strict";

  window.App.Views.TreeNavView = Marionette.LayoutView.extend({
    className: "sidebar-tree sidebar-tree-nav-btn",
    template: './views/templates/sidebar/sidebar_tree_nav.html.ejs',

    ui: {
      simulationsBtn: "#simulations-btn",
      submodelsBtn: "#submodels-btn",
      multivariantBtn: "#multivariant-btn",
      navItem: ".js-nav a"

    },
    events: {
      "shown.bs.tab @ui.navItem": "setActiveTab"
      //"contextmenu" : "onContextMenu"
    },
    regions: {
      materialsRegion: "#materials-tree",
      compositeMaterialsRegion: "#compositeMaterials-tree",
      technologiesRegion: "#technologies-tree",
      constructionElementsRegion: "#construction-tree",
      optimizationsRegion: "#optimizations-tree"
    },

    onShow: function () {
      var that = this;
      that.SidebarTreeController = new App.Controllers.TreeController();

      window.App.commands.setHandler("sidebar:show-tree", function (options) {
        that.showSidebarTree(_.extend(options || {}, {treeId: options.treeType}));
      });

      this.getActiveTab();
    },

    showSidebarTree: function (options) {
      this.zSidebarTree = this.SidebarTreeController.showTree(_.extend(options, {region: this.getRegion(options.treeType + "Region")}));
      this.listenTo(this.zSidebarTree, "ztree:onclick", this.showTreeNodeData);
    },

    onClickExpandAll: function () {
      this.zSidebarTree.trigger('ztree:expand:all');
    },

    showTreeNodeData: function (event, treeId, treeNode, options) {
      options = options || {};
      if (treeNode.itemType == window.App.Data.EntityTypes[8].type) { //OptimizationFolder
        options.showContent = "directory";
      }
      var showContent = options.showContent ? options.showContent : '',
        contentParam = showContent ? '&showContent=' + showContent : '',
        optimizationTypeId = treeNode && treeNode.optimizationTypeId ? '&optimizationTypeId=' + treeNode.optimizationTypeId : '';

      if (!treeNode.itemType) {
        console.warn('Missed treeNode.itemType: ', treeNode);
        return;
      }

      console.info('Tree - %1$s node selected, type: %2$s, node: %3$o', treeId, treeNode.itemType, treeNode)
      var filterData = $.fn.zTree.getTreeLevelData(treeId, treeNode),
        url = "";
      var encodedPath = '?id=' + treeNode.id + "&path=" + encodeURIComponent(_.pluck(treeNode.path, 'name').join('/').toLocaleLowerCase());

      switch (treeNode.itemType) {
        case window.App.Data.EntityTypes[0].type: //Folder
          url = "project/" + treeId + encodedPath ;
          break;
        case window.App.Data.EntityTypes[4].type: //File
          url = "project/file/" + treeNode.id + "/view";
          break;
        case window.App.Data.EntityTypes[5].type:
          url = 'project/technologies/' + treeNode.id + '/parameters/view';
          break;
        case window.App.Data.EntityTypes[6].type:
          switch (treeId) {
            case  window.App.Data.Trees[2].type:
              url = 'project/constructionElements/' + treeNode.id + '/parameters/view';
              break;
            case  window.App.Data.Trees[3].type:
              url = 'project/optimizations' + encodedPath;
              break;
          }
          break;
        case window.App.Data.EntityTypes[7].type: //ConstructionElementFolder
          url = 'project/constructionElements/' + treeNode.pid + '/' + treeNode.name.toLocaleUpperCase() + 'Files/view';
          break;
        case window.App.Data.EntityTypes[8].type: //OptimizationFolder
          url = 'project/optimizations' + encodedPath + '&id=' + treeNode.id + contentParam + optimizationTypeId;
          break;
        case window.App.Data.EntityTypes[9].type:
          break;
        case window.App.Data.EntityTypes[2].type:
          url = "#project/materials/" + treeNode.id + "/parameters/view";
          break;
        case window.App.Data.EntityTypes[3].type:
          url = "#project/compositeMaterials/" + treeNode.id + "/parameters/view";
          break;
        default:
          break;
      }

      window.App.navigate(url, {trigger: true, params: filterData});
    },

    _showTab: function (lastTab) {
      $('[href="' + lastTab + '"]').tab('show');
    },

    getActiveTab: function () {
      var defaultTab = '#materials-trees';
      var lastTab = localStorage.getItem('lastTab') || defaultTab;
      this._showTab(lastTab);
    },

    setActiveTab: function (e) {
      var lastTab = $(e.target).attr('href');
      localStorage.setItem('lastTab', lastTab);
      this._showTab(lastTab);
    }
  })

}());
