package ru.crp.lvms;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.apache.fop.apps.FOPException;
import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import ru.crp.lvms.model.dao.effective.characteristics.ECDefectDao;
import ru.crp.lvms.model.dao.effective.characteristics.ECParamDao;
import ru.crp.lvms.model.entities.effective.characteristics.ECEntity;
import ru.crp.lvms.model.entities.effective.characteristics.ECParamEntity;
import ru.crp.lvms.model.entities.simulations.SimulationEntity;
import ru.crp.lvms.pdf.reports.ECReportInput;
import ru.crp.lvms.pdf.reports.ReportGenerator;
import ru.crp.lvms.pdf.reports.SimulationReportInput;

import javax.xml.bind.JAXBException;
import javax.xml.transform.TransformerException;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Collections;
import java.util.Date;

public class ReportGeneratorTest extends BaseIT {

    @Autowired
    private ReportGenerator reportGenerator;

    @Autowired
    private ECParamDao ecParamDao;

    @Autowired
    private ECDefectDao ecDefectDao;

    private Path pic;

    @Before
    public void init() throws IOException {
        pic = Files.createTempFile(null, null);
        InputStream in = this.getClass().getClassLoader().getResourceAsStream("ok.png");
        FileUtils.copyInputStreamToFile(in, pic.toFile());
    }

    @After
    public void cleanUp() throws IOException {
        Files.deleteIfExists(pic);
    }

    private ECReportInput fillECReportInfo() {
        ECEntity ecEntity = projectCreator.createEffectiveCharacteristicEntity();
        ECReportInput reportInfo = new ECReportInput();
        reportInfo.setEcName(ecEntity.getCompositeMaterialName());
        reportInfo.setCreationDate(new Date());
        reportInfo.setUserName(ecEntity.getOwner());
        reportInfo.setMesoParams(ecParamDao.getByCharacteristicId(ecEntity.getId(), ECParamEntity.Type.Mesostructure));
        reportInfo.setMicroParams(ecParamDao.getByCharacteristicId(ecEntity.getId(), ECParamEntity.Type.Microstructure));
        reportInfo.setDefects(ecDefectDao.getByCharacteristicId(ecEntity.getId()));
        reportInfo.setEcMezostructure(Collections.nCopies(12, Pair.of("Mezostructure", "test")));
        reportInfo.setEcMicrostructure(Collections.nCopies(12, Pair.of("Microstructure", "test")));
        reportInfo.setPathToMicrostructurePicture(pic);
        reportInfo.setPathToMesostructurePicture(pic);
        return reportInfo;
    }

    @Test
    public void ecReportMustBeGenerated() throws IOException, TransformerException, FOPException, JAXBException {
        ECReportInput ecReportInput = fillECReportInfo();
        Path reportFile = reportGenerator.generateReport(ecReportInput);
        Assert.assertTrue(Files.exists(reportFile));
        Files.deleteIfExists(reportFile);
    }

    private SimulationReportInput fillSimReportInfo() {
        SimulationEntity simulationEntity = projectCreator.createSimulation();
        SimulationReportInput reportInfo = new SimulationReportInput();
        reportInfo.setTypeName(simulationEntity.getType());
        reportInfo.setStandartName(simulationEntity.getStandardName());
        reportInfo.setMaterialName(simulationEntity.getCompositeMaterialName());
        reportInfo.setEcName(simulationEntity.getEcName());
        reportInfo.setCreationDate(new Date());
        reportInfo.setUserName(simulationEntity.getOwner());
        reportInfo.setPicture(pic);
        reportInfo.setManualSizeH(simulationEntity.getManualSizeH());
        reportInfo.setManualSizeB(simulationEntity.getManualSizeB());
        reportInfo.setManualSizeL(simulationEntity.getManualSizeL());
        reportInfo.setResults(Collections.nCopies(5, Pair.of("Result", "test")));
        return reportInfo;
    }

    @Test
    public void simReportMustBeGenerated() throws IOException, TransformerException, FOPException, JAXBException {
        SimulationReportInput simulationReportInput = fillSimReportInfo();
        Path reportFile = reportGenerator.generateReport(simulationReportInput);
        Assert.assertTrue(Files.exists(reportFile));
        Files.deleteIfExists(reportFile);
    }
}
