package ru.crp.lvms;

import com.fasterxml.jackson.databind.JsonNode;
import org.junit.After;
import org.junit.Assert;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.web.servlet.MvcResult;
import ru.crp.cmlb.model.SuggestRequest;
import ru.crp.cmlb.model.entities.base.Entity;
import ru.crp.lvms.managers.FileManager;
import ru.crp.lvms.managers.StandardManager;
import ru.crp.lvms.model.entities.StandardEntity;
import ru.crp.lvms.model.entities.misc.FileType;
import ru.crp.lvms.utils.FilesTester;
import ru.crp.lvms.utils.TestUtils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import static org.junit.Assert.assertNull;
import static ru.crp.lvms.utils.TestUtils.CREATEobject;
import static ru.crp.lvms.utils.URLs.STANDARDS_URL;

public class StandardEndpointIT extends CRUDIT<StandardEntity, StandardEntity> {
    private StandardEntity testStandard;

    @Autowired
    private StandardManager standardManager;

    @Autowired
    private FileManager fileManager;

    @Autowired
    private FilesTester filesTester;

    @Override
    public void setup() throws Exception {
        super.setup();
        filesTester.setMockMvc(mockMvc);
        testStandard = projectCreator.createStandard();
    }

    @After
    public void cleanup() {
        standardManager.delete(testStandard.getId(), null);
    }

    @Override
    public String setUrl() {
        return STANDARDS_URL;
    }

    @Override
    public Supplier<StandardEntity> setEntityCreator() {
        return projectCreator::createStandard;
    }

    @Override
    public Supplier<StandardEntity> setEntityInitialisation() {
        return projectCreator::newStandard;
    }

    @Override
    public Function<JsonNode, List<? extends Entity>> setJsonFromList() {
        return TestUtils::standardsFromJson;
    }

    @Override
    public Function<JsonNode, List<? extends Entity>> setJsonFromPage() {
        return TestUtils::standardsPageFromJson;
    }

    @Override
    public void update(StandardEntity toUpdate) {
        toUpdate.setName("testName");
    }

    @Override
    public void validateAfterUpdate(StandardEntity before, StandardEntity after) {
        Assert.assertEquals(before.getName(), after.getName());
    }


    @Test
    public void testSuggestStandardAPI() throws Exception {
        List<Integer> expected = new ArrayList<>();
        for (int i = 0; i < 5; i++) {
            StandardEntity standard = projectCreator.newStandard();
            standard.setName("TestName" + i);
            standard = projectCreator.createStandard(standard);
            expected.add(standard.getId());
        }
        SuggestRequest suggestRequest = projectCreator.newSuggestRequest("name", "%TestName%");
        StandardEntity[] result = CREATEobject(mockMvc, STANDARDS_URL + "/suggest",
                suggestRequest, StandardEntity[].class);
        Assert.assertEquals(expected.size(), result.length);
        List<Integer> actual = Arrays.asList(result).stream()
                .map(StandardEntity::getId)
                .collect(Collectors.toList());
        for (Integer expectedId : expected) {
            Assert.assertTrue(actual.contains(expectedId));
        }
    }


    @Test
    public void itShouldBeAbleToChangeStandardFile() throws Exception {
        String url = getUrlForFile(FileType.StandardFile);
        filesTester.testUpdateFile(FileType.StandardFile, url);
    }


    @Test
    public void itShouldBeAbleToChangeLoadFile() throws Exception {
        String url = getUrlForFile(FileType.StandardLoadFile);
        filesTester.testUpdateFile(FileType.StandardLoadFile, url);
    }


    @Test
    public void itShouldBeAbleToDeleteStandardFile() throws Exception {
        String url = getUrlForFile(FileType.StandardFile);
        filesTester.testDeleteFile(FileType.StandardFile, false, url, this::fileRemovedChecker);
    }


    @Test
    public void itShouldBeAbleToDeleteLoadFile() throws Exception {
        String url = getUrlForFile(FileType.StandardLoadFile);
        filesTester.testDeleteFile(FileType.StandardLoadFile, false, url, this::fileRemovedChecker);
    }


    private void fileRemovedChecker(FileType fileType, MvcResult result) {
        try {
            StandardEntity standardWithoutFile = jacksonMapper
                    .readValue(result.getResponse().getContentAsString(), StandardEntity.class);
            if (fileType == FileType.StandardFile) {
                assertNull(standardWithoutFile.getStandardFileId());
            } else {
                assertNull(standardWithoutFile.getLoadFileId());
            }
        } catch (IOException e) {
            throw new RuntimeException();
        }
    }


    private String getUrlForFile(FileType fileType) {
        if (fileType == FileType.StandardFile) {
            return url = STANDARDS_URL + "/" + testStandard.getId() + "/standardFile";
        } else {
            return url = STANDARDS_URL + "/" + testStandard.getId() + "/loadFile";
        }
    }
}
