package ru.crp.lvms;


import com.fasterxml.jackson.databind.JsonNode;
import org.junit.Assert;
import ru.crp.cmlb.model.entities.base.Entity;
import ru.crp.lvms.model.entities.SupportLetterEntity;
import ru.crp.lvms.utils.TestUtils;
import ru.crp.lvms.utils.URLs;

import java.util.Date;
import java.util.List;
import java.util.function.Function;
import java.util.function.Supplier;

public class SupportLetterIT extends CRUDIT<SupportLetterEntity, SupportLetterEntity> {
    @Override
    public String setUrl() {
        return URLs.SUPPORT_URL;
    }

    @Override
    public Supplier<SupportLetterEntity> setEntityCreator() {
        return projectCreator::createSupportLetter;
    }

    @Override
    public Supplier<SupportLetterEntity> setEntityInitialisation() {
        return projectCreator::newSupportLetter;
    }

    @Override
    public Function<JsonNode, List<? extends Entity>> setJsonFromList() {
        return TestUtils::supportLetterEntitiesFromJson;
    }

    @Override
    public Function<JsonNode, List<? extends Entity>> setJsonFromPage() {
        return TestUtils::supportLetterEntitiesPageFromJson;
    }

    @Override
    public void update(SupportLetterEntity toUpdate) {
        toUpdate.setPageUrl("new" + toUpdate.getPageUrl());
        toUpdate.setContent("new" + toUpdate.getContent());
        toUpdate.setIsProcessed(true);

        toUpdate.setUserId(-1);
        toUpdate.setCreationDate(new Date());
        toUpdate.setUserName("test");
    }

    @Override
    public void validateAfterUpdate(SupportLetterEntity before, SupportLetterEntity after) {
        Assert.assertEquals(before.getPageUrl(), after.getPageUrl());
        Assert.assertEquals(before.getContent(), after.getContent());
        Assert.assertEquals(before.getIsProcessed(), after.getIsProcessed());

        Assert.assertNotEquals(before.getUserId(), after.getUserId());
        Assert.assertNotEquals(before.getCreationDate(), after.getCreationDate());
        Assert.assertNotEquals(before.getUserName(), after.getUserName());
    }
}
