package ru.crp.lvms;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import ru.crp.lvms.model.entities.misc.ParametersDictionaryType;
import ru.crp.lvms.optimizations.base.entities.OptimizationEntity;
import ru.crp.lvms.optimizations.base.entities.OptimizationJsonSetting;
import ru.crp.lvms.optimizations.base.entities.OptimizationListSetting;
import ru.crp.lvms.optimizations.base.enums.OptimizationCao;
import ru.crp.lvms.optimizations.mf.managers.MFOptimizationSettingsManager;
import ru.crp.lvms.optimizations.tosca.config.ToscaConfigGenerator;
import ru.crp.lvms.optimizations.tosca.entities.ToscaConstraint;
import ru.crp.lvms.optimizations.tosca.entities.ToscaTargetFunction;
import ru.crp.lvms.optimizations.tosca.enums.NodeOrElem;
import ru.crp.lvms.optimizations.tosca.enums.ToscaConstraintType;
import ru.crp.lvms.optimizations.tosca.managers.ToscaOptimizationSettingsManager;
import ru.crp.lvms.utils.SetterChecker;
import ru.crp.lvms.utils.TestUtils;
import ru.crp.lvms.utils.URLs;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Collections;
import java.util.List;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static ru.crp.lvms.utils.TestUtils.*;
import static ru.crp.lvms.utils.URLs.OPTIMIZATIONS_URL;

public class ToskaIT extends BaseIT {

    @Autowired
    private ToscaConfigGenerator toscaConfigGenerator;

    @Autowired
    private MFOptimizationSettingsManager MFOptimizationSettingsManager;

    @Autowired
    private ToscaOptimizationSettingsManager toscaOptimizationSettingsManager;

    private OptimizationEntity optimizationEntity;
    private String url;

    @Before
    public void init() {
        optimizationEntity = projectCreator.createOptimization(OptimizationCao.Tosca);
        url = OPTIMIZATIONS_URL + "/" + optimizationEntity.getId() + "/Tosca/methods";
    }

    @Test
    public void itMustGenerateCAOFileWithoutExceptions() throws IOException {
        toscaOptimizationSettingsManager.selectOptimizationMethod(optimizationEntity.getId(),
                MFOptimizationSettingsManager.findByTypeName(ParametersDictionaryType.ToskaOptimizationMethod).get(0).getId());
        Path cao = toscaConfigGenerator.generateConfig(optimizationEntity);
        Files.delete(cao);
    }

    @Test
    public void itMustUpdateToskaOptimizationConstraint() throws Exception {
        String url = URLs.TOSCA_CONSTRAINT_URL.replace("{optimizationId}", String.valueOf(optimizationEntity.getId()));
        List<ToscaConstraint> toscaConstraints =
                optimizationToskaConstraintFromJson(TestUtils.GETobjects(mockMvc, url));
//        Assert.assertEquals(ToscaBaseSettingType.NUM_CONSTRAINTS.getMaxValue(), toscaConstraints.size());
        ToscaConstraint toscaConstraint = SetterChecker.createProxy(ToscaConstraint.class);
        toscaConstraint.setId(toscaConstraints.get(0).getId());
        toscaConstraint.setOptimizationId(toscaConstraints.get(0).getOptimizationId());
        toscaConstraint.setIsAbsolute(false);
        toscaConstraint.setParameterId(1);
        toscaConstraint.setTypeId(ToscaConstraintType.GREATER_OR_EQUAL.getId());
        toscaConstraint.setValue("test");
        toscaConstraint.setName("test");
        toscaConstraint.setNodeOrElementId(NodeOrElem.NODE_AND_ELEMENT.getId());
        toscaConstraint.setDrespType("test");
        toscaConstraint.setParamName("test");
        toscaConstraint.setScalar("test");
        toscaConstraint.setNodesGroupId(null);
        toscaConstraint.setElementsGroupId(null);
        toscaConstraint.setOptimizationTypeId(toscaConstraints.get(0).getOptimizationTypeId());
        ToscaConstraint afterUpdate =
                TestUtils.UPDATEobject(mockMvc, url + "/" + toscaConstraints.get(0).getId(), toscaConstraint, ToscaConstraint.class);
        Assert.assertEquals(toscaConstraint.getIsAbsolute(), afterUpdate.getIsAbsolute());
        Assert.assertEquals(toscaConstraint.getParameterId(), afterUpdate.getParameterId());
        Assert.assertEquals(toscaConstraint.getTypeId(), afterUpdate.getTypeId());
        Assert.assertEquals(toscaConstraint.getValue(), afterUpdate.getValue());
        Assert.assertEquals(toscaConstraint.getName(), afterUpdate.getName());
    }

    @Test
    public void itMustUpdateToskaOptimizationTargetFunctions() throws Exception {
        String url = URLs.TOSCA_TARGET_FUNCS_URL.replace("{optimizationId}", String.valueOf(optimizationEntity.getId()));
        List<ToscaTargetFunction> entities =
                optimizationToskaTargetFunctionFromJson(TestUtils.GETobjects(mockMvc, url));
//        Assert.assertEquals(ToscaBaseSettingType.NUM_TARGET_FUNCS.getMaxValue(), entities.size());
        ToscaTargetFunction entity = SetterChecker.createProxy(ToscaTargetFunction.class);
        entity.setId(entities.get(0).getId());
        entity.setOptimizationId(entities.get(0).getOptimizationId());
        entity.setParameterId(1);
        entity.setTypeId(ToscaConstraintType.GREATER_OR_EQUAL.getId());
        entity.setName("test");
        entity.setNodeOrElementId(NodeOrElem.NODE_AND_ELEMENT.getId());
        entity.setDrespType("test");
        entity.setParamName("test");
        entity.setScalar("test");
        entity.setNodesGroupId(null);
        entity.setElementsGroupId(null);
        entity.setOptimizationTypeId(entities.get(0).getOptimizationTypeId());
        ToscaTargetFunction afterUpdate =
                TestUtils.UPDATEobject(mockMvc, url + "/" + entities.get(0).getId(), entity, ToscaTargetFunction.class);
        Assert.assertEquals(entity.getParameterId(), afterUpdate.getParameterId());
        Assert.assertEquals(entity.getTypeId(), afterUpdate.getTypeId());
        Assert.assertEquals(entity.getName(), afterUpdate.getName());
    }

    @Test
    public void itMustSelectOptimizationMethods() throws Exception {
        List<OptimizationListSetting> settings = optimizationListSettingFromJson(GETobjects(mockMvc, url));
        checkExactlyOneOptionSelected(selectOptimizationMethod(settings.get(0).getId()));
        checkExactlyOneOptionSelected(optimizationListSettingFromJson(GETobjects(mockMvc, url)));
    }

    @Test
    public void itMustUpdateAdditionalParametersForOM() throws Exception {
        List<OptimizationListSetting> settings = optimizationListSettingFromJson(GETobjects(mockMvc, url));
        List<OptimizationJsonSetting> additionalParams =
                optimizationJsonSettingFromJson(GETobjects(mockMvc, url + "/" + settings.get(0).getId() + "/params"));
        additionalParams.forEach(setting -> {
            editAdditionalParameter(setting);
            OptimizationJsonSetting updated = updateAdditionalParameter(setting);
            assertParameterUpdated(setting, updated);
        });
    }

    @Test
    public void itMustUpdateAdditionalParametersForSelectedOM() throws Exception {
        List<OptimizationListSetting> settings = optimizationListSettingFromJson(GETobjects(mockMvc, url));
        selectOptimizationMethod(settings.get(0).getId());
        List<OptimizationJsonSetting> additionalParams =
                optimizationJsonSettingFromJson(GETobjects(mockMvc, url + "/selected"));
        additionalParams.forEach(setting -> {
            editAdditionalParameter(setting);
            OptimizationJsonSetting updated = updateAdditionalParameter(setting);
            assertParameterUpdated(setting, updated);
        });
    }

    private List<OptimizationListSetting> selectOptimizationMethod(int omId) throws Exception {
        return optimizationListSettingFromJson(
                jacksonMapper.readTree(
                        mockMvc.perform(put(url + "/" + omId))
                                .andExpect(status().isOk())
                                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                                .andReturn()
                                .getResponse()
                                .getContentAsString()
                ));
    }

    private void editAdditionalParameter(OptimizationJsonSetting additionalParam) {
        switch (additionalParam.getType()) {
            case plain:
                additionalParam.setPlainValue("test");
                break;
            case bool:
                additionalParam.setBoolValue(false);
                break;
            case list:
                additionalParam.setListValue(Collections.singletonList(additionalParam.getListValue().get(0)));
                break;
            case composite:
                additionalParam.getCompositeValue().forEach(this::editAdditionalParameter);
                break;
        }
    }

    private void assertParameterUpdated(OptimizationJsonSetting original, OptimizationJsonSetting updated) {
        Assert.assertEquals(original.getType(), updated.getType());
        switch (original.getType()) {
            case plain:
                Assert.assertEquals(original.getPlainValue(), updated.getPlainValue());
                break;
            case bool:
                Assert.assertEquals(original.getBoolValue(), updated.getBoolValue());
                break;
            case list:
                Assert.assertEquals(original.getListValue().size(), updated.getListValue().size());
                break;
            case composite:
                List<OptimizationJsonSetting> originalCompositeValue = original.getCompositeValue();
                List<OptimizationJsonSetting> updatedCompositeValue = updated.getCompositeValue();
                Assert.assertEquals(originalCompositeValue.size(), updatedCompositeValue.size());
                for (int i = 0; i < originalCompositeValue.size(); i++) {
                    assertParameterUpdated(originalCompositeValue.get(i), updatedCompositeValue.get(i));
                }
                break;
            default:
                throw new IllegalArgumentException("Unexepected enum value: " + original.getType());
        }
    }

    private OptimizationJsonSetting updateAdditionalParameter(OptimizationJsonSetting parameter) {
        try {
            return jacksonMapper.readValue(mockMvc.perform(put(url + "/params/" + parameter.getId())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(toJsonNoProxy(parameter)))
                    .andExpect(status().isOk())
                    .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                    .andReturn().getResponse().getContentAsString(), OptimizationJsonSetting.class);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private void checkExactlyOneOptionSelected(List<OptimizationListSetting> options) {
        Assert.assertEquals(1, options.stream().filter(OptimizationListSetting::isSelected).count());
    }
}
