package ru.crp.lvms.construction.element;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import ru.crp.lvms.BaseIT;
import ru.crp.lvms.model.entities.construction.elements.AdjacentElementsEntityTable;
import ru.crp.lvms.model.entities.construction.elements.ConstructionElementDocsEntity;
import ru.crp.lvms.model.entities.construction.elements.ConstructionElementEntity;
import ru.crp.lvms.utils.TestUtils;

import java.util.List;

import static ru.crp.lvms.utils.URLs.CONSTRUCTION_ELEMENTS_URL;

public class AdjElementsIT extends BaseIT {

    private ConstructionElementEntity constructionElementEntity;

    private String adjUrl;

    @Before
    public void initTests() {
        constructionElementEntity = projectCreator.createConstructionElementEntity();
        adjUrl = CONSTRUCTION_ELEMENTS_URL + "/" + constructionElementEntity.getId() + "/adjacents";
    }

    @Test
    public void itMustAdjTwoConstructionElements() throws Exception {
        List<AdjacentElementsEntityTable> beforeCreate
                = TestUtils.adjacentElementsFromJson(TestUtils.GETobjects(mockMvc, adjUrl));
        Assert.assertTrue("There should be no adjacency after creation", beforeCreate.isEmpty());
        ConstructionElementEntity adj = projectCreator.createConstructionElementEntity();
        TestUtils.CREATEobject(mockMvc, adjUrl + "/" + adj.getId(), null, AdjacentElementsEntityTable.class);
        List<AdjacentElementsEntityTable> afterCreate
                = TestUtils.adjacentElementsFromJson(TestUtils.GETobjects(mockMvc, adjUrl));
        Assert.assertEquals("There should be exactly one adjacency file after creation", 1, afterCreate.size());
        Assert.assertEquals(adj.getId(), afterCreate.get(0).getAdjacentElementId());
        Assert.assertEquals(constructionElementEntity.getId(), afterCreate.get(0).getConstructionElementId());
    }


    @Test
    public void itMustDeleteAdjConstructionElements() throws Exception {
        List<AdjacentElementsEntityTable> beforeCreate
                = TestUtils.adjacentElementsFromJson(TestUtils.GETobjects(mockMvc, adjUrl));
        Assert.assertTrue("There should be no adjacency file after creation", beforeCreate.isEmpty());
        ConstructionElementEntity adj = projectCreator.createConstructionElementEntity();
        TestUtils.CREATEobject(mockMvc, adjUrl + "/" + adj.getId(), null, AdjacentElementsEntityTable.class);
        TestUtils.DELETEobject(mockMvc, adjUrl + "/" + adj.getId());
        List<ConstructionElementDocsEntity> afterDelete
                = TestUtils.constructionElementDocsFromJson(TestUtils.GETobjects(mockMvc, adjUrl));
        Assert.assertTrue("There should be no adjacency file after deletion", afterDelete.isEmpty());
    }
}
