package ru.crp.lvms.construction.element;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import ru.crp.lvms.BaseIT;
import ru.crp.lvms.model.entities.FileEntity;
import ru.crp.lvms.model.entities.construction.elements.ConstructionElementEntity;
import ru.crp.lvms.utils.FilesTester;
import ru.crp.lvms.utils.TestUtils;
import ru.crp.lvms.web.dto.pagination.DataRequest;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static ru.crp.lvms.utils.TestUtils.*;
import static ru.crp.lvms.utils.URLs.CONSTRUCTION_ELEMENTS_URL;
import static ru.crp.lvms.utils.URLs.OPTIMIZATIONS_URL;

public class ConstructionElementFilesIT extends BaseIT {

    @Autowired
    private FilesTester filesTester;

    private ConstructionElementEntity constructionElementEntity;

    private static final String fileContent = unique("fileContent");

    @Before
    public void init() {
        constructionElementEntity = projectCreator.createConstructionElementEntity();
    }

    @Test
    public void itMustCreateCADFiles() throws Exception {
        assertSuccefulCreation("CADFiles");
    }

    @Test
    public void itMustGetCADFiles() throws Exception {
        assertFileFromUrlWithSameContent("CADFiles");
    }

    @Test
    public void itMustGetCADFilesPage() throws Exception {
        assertFilePageFromUrlWithSameContent("CADFiles");
    }

    @Test
    public void itMustDeleteCADFiles() throws Exception {
        assertSuccefulDeletion("CADFiles");
    }

    @Test
    public void itMustNotDeleteCADFilesUsedInOptimisation() throws Exception {
        assertFileNotDeleteIfUsed("CADFiles");
    }

    @Test
    public void itMustCreateCAEFiles() throws Exception {
        assertSuccefulCreation("CAEFiles");
    }

    @Test
    public void itMustGetCAEFiles() throws Exception {
        assertFileFromUrlWithSameContent("CAEFiles");
    }

    @Test
    public void itMustGetCAEFilesPage() throws Exception {
        assertFilePageFromUrlWithSameContent("CAEFiles");
    }

    @Test
    public void itMustDeleteCAEFiles() throws Exception {
        assertSuccefulDeletion("CAEFiles");
    }

    @Test
    public void itMustNotDeleteCAEFilesUsedInOptimisation() throws Exception {
        assertFileNotDeleteIfUsed("CAEFiles");
    }

    @Test
    public void itMustCreateCAOFiles() throws Exception {
        assertSuccefulCreation("CAOFiles");
    }

    @Test
    public void itMustGetCAOFiles() throws Exception {
        assertFileFromUrlWithSameContent("CAOFiles");
    }

    @Test
    public void itMustGetCAOFilesPage() throws Exception {
        assertFilePageFromUrlWithSameContent("CAOFiles");
    }

    @Test
    public void itMustDeleteCAOFiles() throws Exception {
        assertSuccefulDeletion("CAOFiles");
    }

    @Test
    public void itMustNotDeleteCAOFilesUsedInOptimisation() throws Exception {
        assertFileNotDeleteIfUsed("CAOFiles");
    }

    @Test
    public void itMustCreateDesignSolutions() throws Exception {
        assertSuccefulCreation("designSolutions");
    }

    @Test
    public void itMustGetDesignSolutions() throws Exception {
        assertFileFromUrlWithSameContent("designSolutions");
    }

    @Test
    public void itMustGetDesignSolutionsPage() throws Exception {
        assertFilePageFromUrlWithSameContent("designSolutions");
    }

    @Test
    public void itMustDeleteDesignSolutions() throws Exception {
        assertSuccefulDeletion("designSolutions");
    }

    @Test
    public void itMustNotDeleteDesignSolutionsUsedInOptimisation() throws Exception {
        assertFileNotDeleteIfUsed("designSolutions");
    }

    @Test
    public void itMustCreateLoadCondition() throws Exception {
        assertSuccefulCreation("loadConditions");
    }

    @Test
    public void itMustGetLoadCondition() throws Exception {
        assertFileFromUrlWithSameContent("loadConditions");
    }

    @Test
    public void itMustGetLoadConditionPage() throws Exception {
        assertFilePageFromUrlWithSameContent("loadConditions");
    }

    @Test
    public void itMustDeleteLoadCondition() throws Exception {
        assertSuccefulDeletion("loadConditions");
    }

    @Test
    public void itMustNotDeleteLoadConditionUsedInOptimisation() throws Exception {
        assertFileNotDeleteIfUsed("loadConditions");
    }

    private int assertSuccefulCreation(String subUrl) throws Exception {
        String url = CONSTRUCTION_ELEMENTS_URL + "/" + constructionElementEntity.getId() + "/" + subUrl;
        MockMultipartFile cad = new MockMultipartFile("file", unique("fileContent"), null, fileContent.getBytes());
        FileEntity result = TestUtils.performRequestWithFiles(url, mockMvc, Collections.emptyMap(), null,
                Collections.singletonList(cad), status().isOk(), FileEntity.class);
        filesTester.assertFileSaved(result.getId(), fileContent.getBytes());
        return result.getId();
    }

    private void assertFileFromUrlWithSameContent(String subUrl) throws Exception {
        String url = CONSTRUCTION_ELEMENTS_URL + "/" + constructionElementEntity.getId() + "/" + subUrl;
        List<FileEntity> filesBefore = filesFromJson(GETobjects(mockMvc, url));
        int fileId = assertSuccefulCreation(subUrl);
        List<FileEntity> filesAfter = filesFromJson(GETobjects(mockMvc, url));
        Assert.assertEquals(filesBefore.size() + 1, filesAfter.size());
        Assert.assertTrue(filesAfter.stream().map(FileEntity::getId).collect(Collectors.toList()).contains(fileId));
        filesTester.assertFileSaved(fileId, fileContent.getBytes());
    }

    private void assertFilePageFromUrlWithSameContent(String subUrl) throws Exception {
        String url = CONSTRUCTION_ELEMENTS_URL + "/" + constructionElementEntity.getId() + "/" + subUrl + "/list";
        DataRequest dataRequest = projectCreator.newDataRequest();
        List<FileEntity> filesBefore = filesPageFromJson(GETPage(mockMvc, url, dataRequest));
        int fileId = assertSuccefulCreation(subUrl);
        List<FileEntity> filesAfter = filesPageFromJson(GETPage(mockMvc, url, dataRequest));
        Assert.assertEquals(filesBefore.size() + 1, filesAfter.size());
        Assert.assertTrue(filesAfter.stream().map(FileEntity::getId).collect(Collectors.toList()).contains(fileId));
        filesTester.assertFileSaved(fileId, fileContent.getBytes());
    }

    private void assertSuccefulDeletion(String subUrl) throws Exception {
        String getUrl = CONSTRUCTION_ELEMENTS_URL + "/" + constructionElementEntity.getId() + "/" + subUrl;
        List<FileEntity> filesBefore = filesFromJson(GETobjects(mockMvc, getUrl));
        int fileId = assertSuccefulCreation(subUrl);
        String deleteUrl = CONSTRUCTION_ELEMENTS_URL + "/" + constructionElementEntity.getId() + "/" + subUrl + "/" + fileId;
        DELETEobject(mockMvc, deleteUrl);
        List<FileEntity> filesAfter = filesFromJson(GETobjects(mockMvc, getUrl));
        Assert.assertEquals(filesBefore.size(), filesAfter.size());
        Assert.assertEquals(filesBefore.stream().map(FileEntity::getId).collect(Collectors.toList()),
                filesAfter.stream().map(FileEntity::getId).collect(Collectors.toList()));
    }

    private void assertFileNotDeleteIfUsed(String subUrl) throws Exception {
        Integer optId = projectCreator.createOptimization().getId();
        int fileId = assertSuccefulCreation(subUrl);
        String optUrl = OPTIMIZATIONS_URL + "/" + optId + "/models/replace";
        TestUtils.CREATEobject(mockMvc, optUrl, Collections.singletonList(fileId));
        String url = CONSTRUCTION_ELEMENTS_URL + "/" + constructionElementEntity.getId() + "/" + subUrl + "/" + fileId;
        DELETEAndExpectConflict(mockMvc, url);
    }
}
