package ru.crp.lvms.construction.element;

import com.fasterxml.jackson.databind.JsonNode;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.web.servlet.MvcResult;
import ru.crp.cmlb.model.entities.base.Entity;
import ru.crp.lvms.CRUDIT;
import ru.crp.lvms.model.entities.construction.elements.ConstructionElementEntity;
import ru.crp.lvms.model.entities.misc.FileType;
import ru.crp.lvms.utils.FilesTester;
import ru.crp.lvms.utils.TestUtils;

import java.io.IOException;
import java.util.List;
import java.util.function.Function;
import java.util.function.Supplier;

import static org.junit.Assert.assertNull;
import static ru.crp.lvms.utils.URLs.CONSTRUCTION_ELEMENTS_URL;

public class ConstructionElementsEndpointIT extends CRUDIT<ConstructionElementEntity, ConstructionElementEntity> {

    @Autowired
    private FilesTester filesTester;

    @Override
    public String setUrl() {
        return CONSTRUCTION_ELEMENTS_URL;
    }

    @Override
    public Supplier<ConstructionElementEntity> setEntityCreator() {
        return projectCreator::createConstructionElementEntity;
    }

    @Override
    public Supplier<ConstructionElementEntity> setEntityInitialisation() {
        return projectCreator::newConstructionElementEntity;
    }

    @Override
    public Function<JsonNode, List<? extends Entity>> setJsonFromList() {
        return TestUtils::constructionElementFromJson;
    }

    @Override
    public Function<JsonNode, List<? extends Entity>> setJsonFromPage() {
        return TestUtils::constructionElementPageFromJson;
    }


    @Override
    public void testUpdate() throws Exception {
        super.testUpdate();
    }

    @Override
    public void update(ConstructionElementEntity toUpdate) {
        toUpdate.setName("testName");
    }

    @Override
    public void validateAfterUpdate(ConstructionElementEntity before, ConstructionElementEntity after) {
        Assert.assertEquals(before.getName(), after.getName());
    }

    private ConstructionElementEntity constructionElementEntity;

    @Before
    public void initTests() {
        constructionElementEntity = projectCreator.createConstructionElementEntity();
        filesTester.setMockMvc(mockMvc);
    }


    @Test
    public void itShouldBeAbleToChangeImageFile() throws Exception {
        String url = getUrlForFile(FileType.ImageFile);
        filesTester.testUpdateFile(FileType.ImageFile, url);
    }


    @Test
    public void itShouldBeAbleToChangeCadFile() throws Exception {
        String url = getUrlForFile(FileType.CADFile);
        filesTester.testUpdateFile(FileType.CADFile, url);
    }


    private String getUrlForFile(FileType fileType) {
        if (fileType == FileType.ImageFile) {
            return url = CONSTRUCTION_ELEMENTS_URL + "/" + constructionElementEntity.getId() + "/imageFile";
        } else if (fileType == FileType.CADFile) {
            return url = CONSTRUCTION_ELEMENTS_URL + "/" + constructionElementEntity.getId() + "/initialCadModel";
        } else {
            return url = CONSTRUCTION_ELEMENTS_URL + "/" + constructionElementEntity.getId() + "/descriptionFiles";
        }
    }


    @Test
    public void itShouldBeAbleToDeleteImageFile() throws Exception {
        String url = getUrlForFile(FileType.ImageFile);
        filesTester.testDeleteFile(FileType.ImageFile, false, url, this::fileRemovedChecker);
    }


    @Test
    public void itShouldBeAbleToDeleteDescriptionFile() throws Exception {
        String url = getUrlForFile(FileType.DescriptionFile);
        filesTester.testDeleteFile(FileType.DescriptionFile, false, url, (unused1, unused2) -> {
        });
    }


    private void fileRemovedChecker(FileType fileType, MvcResult result) {
        try {
            ConstructionElementEntity constructionElement = jacksonMapper
                    .readValue(result.getResponse().getContentAsString(), ConstructionElementEntity.class);
            if (fileType == FileType.ImageFile) {
                assertNull(constructionElement.getImageFileId());
            } else if (fileType == FileType.CADFile) {
                assertNull(constructionElement.getInitialCadModelId());
            }
        } catch (IOException e) {
            throw new RuntimeException();
        }
    }
}
