package ru.crp.lvms.material;

import com.fasterxml.jackson.databind.JsonNode;
import org.junit.Assert;
import org.junit.Test;
import ru.crp.cmlb.model.entities.base.Entity;
import ru.crp.lvms.CRUDIT;
import ru.crp.lvms.model.entities.compositeMaterials.CompositeMaterialEntity;
import ru.crp.lvms.model.entities.EquipmentEntity;
import ru.crp.lvms.model.entities.technology.TechnologyEntityTable;
import ru.crp.lvms.utils.TestUtils;
import ru.crp.lvms.web.dto.CompositeMaterialIn;

import java.util.List;
import java.util.function.Function;
import java.util.function.Supplier;

import static ru.crp.lvms.utils.TestUtils.CREATEAndExpectBadRequest;
import static ru.crp.lvms.utils.URLs.COMPOSITE_MATERIALS_URL;

public class CompositeMaterialIT extends CRUDIT<CompositeMaterialIn, CompositeMaterialEntity> {
    @Override
    public String setUrl() {
        return COMPOSITE_MATERIALS_URL;
    }

    @Override
    public Supplier<CompositeMaterialEntity> setEntityCreator() {
        return projectCreator::createCompositeMaterial;
    }

    @Override
    public Supplier<CompositeMaterialIn> setEntityInitialisation() {
        return projectCreator::newCompositeMaterial;
    }

    @Override
    public Function<JsonNode, List<? extends Entity>> setJsonFromList() {
        return TestUtils::compositeMaterialsFromJson;
    }

    @Override
    public Function<JsonNode, List<? extends Entity>> setJsonFromPage() {
        return TestUtils::compositeMaterialsPageFromJson;
    }

    @Override
    public void update(CompositeMaterialIn toUpdate) {
        toUpdate.setDescription("testDescription");
        toUpdate.setName("testName");
    }

    @Override
    public void validateAfterUpdate(CompositeMaterialIn before, CompositeMaterialEntity after) {
        Assert.assertEquals(before.getDescription(), after.getDescription());
        Assert.assertEquals(before.getName(), after.getName());
    }


    // ticket http://youtrack.crplab.ru/issue/LVMS-58
    @Test
    public void testTechnologyEquipmentConsistency() throws Exception {
        CompositeMaterialIn compositeMaterialIn = projectCreator.newCompositeMaterial();
        TechnologyEntityTable technology = projectCreator.createTechnologyEntity();
        EquipmentEntity equipment = projectCreator.createEquipment();
        compositeMaterialIn.setTechnologyId(technology.getId());
        compositeMaterialIn.setEquipmentId(equipment.getId());
        CREATEAndExpectBadRequest(mockMvc, COMPOSITE_MATERIALS_URL, compositeMaterialIn);
    }
}
