package ru.crp.lvms.material;

import com.fasterxml.jackson.databind.JsonNode;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MvcResult;
import ru.crp.lvms.BaseIT;
import ru.crp.lvms.managers.ParametersManager;
import ru.crp.lvms.model.entities.compositeMaterials.CompositeMaterialEntity;
import ru.crp.lvms.model.entities.MaterialEntity;
import ru.crp.lvms.model.entities.ParameterSample;
import ru.crp.lvms.model.entities.ParametersEntityTable;
import ru.crp.lvms.web.dto.CompositeMaterialIn;
import ru.crp.lvms.web.dto.DefaultParameterValue;
import ru.crp.lvms.web.dto.MaterialIn;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.BiConsumer;

import static java.util.Arrays.asList;
import static java.util.stream.Collectors.toList;
import static org.junit.Assert.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static ru.crp.lvms.utils.TestUtils.*;
import static ru.crp.lvms.utils.URLs.COMPOSITE_MATERIALS_URL;
import static ru.crp.lvms.utils.URLs.MATERIALS_URL;

// todo need refactoring
public class MaterialParametersEndpointIT extends BaseIT {
    @Autowired
    private ParametersManager parametersManager;
    private List<Integer> allParametersForMaterial;
    private List<Integer> allParametersForCompositeMaterial;
    private int testParamId;

    @Override
    public void setup() throws Exception {
        super.setup();
        testParamId = projectCreator.createParameter().getId();
        projectCreator.createParameter();
        List<ParametersEntityTable> allParameters = parametersManager.findAll(null);
        allParametersForMaterial = allParameters.stream()
                .filter(param -> param.getMaterialRelation().equals(0) || param.getMaterialRelation().equals(2))
                .map(ParametersEntityTable::getId)
                .collect(toList());
        allParametersForCompositeMaterial = allParameters.stream()
                        .filter(param -> param.getMaterialRelation().equals(1) || param.getMaterialRelation().equals(2))
                        .map(ParametersEntityTable::getId)
                        .collect(toList());
    }


    @Test
    public void itShouldBeAbleToGetDefaultParameterValues() throws Exception {
        testBothMaterials(this::testGetDefaultParameters);
    }

    private void testGetDefaultParameters(boolean isComposite, int materialId) {
        List<DefaultParameterValue> defaultParameters = projectCreator.getDefaultParameters(isComposite, materialId);
        defaultParameters.forEach(pv -> assertNull(pv.getParamValue()));
        List<Integer> paramIds = defaultParameters.stream()
                .map(DefaultParameterValue::getId)
                .collect(toList());
        if (isComposite) {
            allParametersForCompositeMaterial.forEach(id -> assertTrue(paramIds.contains(id)));

        } else {
            allParametersForMaterial.forEach(id -> assertTrue(paramIds.contains(id)));
        }
    }


    @Test
    public void itShouldBeAbleToGetSingleParameterValues() throws Exception {
        testBothMaterials(this::testGetSingleParameterValues);
    }

    private void testGetSingleParameterValues(boolean isComposite, int materialId) {
        List<ParameterSample> expected = projectCreator.getParameterPoints(isComposite, materialId, testParamId);
        ParameterSample point1 = projectCreator.addParameterToMaterial(testParamId, materialId);
        ParameterSample point2 = projectCreator.addParameterToMaterial(testParamId, materialId);
        expected.addAll(asList(point1, point2));

        List<ParameterSample> actual = projectCreator.getParameterPoints(isComposite, materialId, testParamId);

        assertParameterPointsEquals(expected, actual);
    }


    @Test
    public void itShouldBeAbleToSetParameterValues() throws Exception {
        testBothMaterials(this::testSetValues);
    }

    private void testSetValues(boolean isComposite, int materialId) {
        ParameterSample point1 = projectCreator.newParameterPoint(null, null);
        ParameterSample point2 = projectCreator.newParameterPoint(null, null);
        List<ParameterSample> expected = new ArrayList<>(asList(point1, point2));
        String url = createMaterialParametersUrl(isComposite, materialId, testParamId, null);
        try {
            MvcResult result = mockMvc.perform(put(url)
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(toJsonNoProxy(expected)))
                    .andExpect(status().isOk())
                    .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                    .andReturn();
            JsonNode responseJson = jacksonMapper.readTree(result.getResponse().getContentAsString());
            List<ParameterSample> actual = parameterSamplesFromJson(responseJson);
            assertParameterPointsEquals(expected, actual);
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }


    @Test
    public void itShouldBeAbleToDeleteAllParameterValues() throws Exception {
        testBothMaterials(this::testDeleteAllValues);
    }

    private void testDeleteAllValues(boolean isComposite, int materialId) {
        projectCreator.addParameterToMaterial(testParamId, materialId);
        String url = createMaterialParametersUrl(isComposite, materialId, testParamId, null);
        DELETEobject(mockMvc, url);

        JsonNode responseJson = GETobjects(mockMvc, url);
        List<ParameterSample> parameterValues = parameterSamplesFromJson(responseJson);
        assertTrue(parameterValues.isEmpty());
    }


    @Test
    public void itShouldBeAbleToAddParameterValue() throws Exception {
        testBothMaterials(this::testCreatePoint);
    }

    private void testCreatePoint(boolean isComposite, int materialId) {
        ParameterSample newPoint = projectCreator.newParameterPoint(null, null);
        String url = createMaterialParametersUrl(isComposite, materialId, testParamId, null) + "/point";
        try {
            MvcResult result = mockMvc.perform(post(url)
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(toJson(newPoint)))
                    .andExpect(status().isOk())
                    .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                    .andReturn();
            JsonNode responseJson = jacksonMapper.readTree(result.getResponse().getContentAsString());
            List<ParameterSample> parameterValues = parameterSamplesFromJson(responseJson);
            assertFalse(parameterValues.isEmpty());
            assertParamPointsEquals(newPoint, parameterValues.get(0));
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }


    @Test
    public void itShouldBeAbleToUpdateParameterValue() throws Exception {
        testBothMaterials(this::testUpdatePoint);

    }

    private void testUpdatePoint(boolean isComposite, int materialId) {
        List<ParameterSample> parameterValuesBefore = projectCreator.getParameterPoints(isComposite, materialId, testParamId);
        projectCreator.addParameterToMaterial(testParamId, materialId);
        List<ParameterSample> parameterValues = projectCreator.getParameterPoints(isComposite, materialId, testParamId);
        assertEquals(parameterValuesBefore.size() + 1, parameterValues.size());

        ParameterSample paramSample = parameterValues.get(0);
        paramSample.setFuncParamValue(BigDecimal.valueOf(123d));
        String url = createMaterialParametersUrl(isComposite, materialId, testParamId, paramSample.getId());
        try {
            MvcResult result = mockMvc.perform(put(url)
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(toJsonNoProxy(paramSample)))
                    .andExpect(status().isOk())
                    .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                    .andReturn();
            JsonNode responseJson = jacksonMapper.readTree(result.getResponse().getContentAsString());
            List<ParameterSample> parameterSamples = parameterSamplesFromJson(responseJson);
            assertEquals(parameterValuesBefore.size() + 1, parameterSamples.size());
            Optional<ParameterSample> updatedParameter =
                    parameterSamples.stream().filter(p -> Objects.equals(p.getId(), paramSample.getId())).findAny();
            assertTrue(updatedParameter.isPresent());
            assertParamPointsEquals(paramSample, updatedParameter.get());
        } catch (Exception ex) {
            throw new RuntimeException(ex);
        }
    }


    @Test
    public void itShouldBeAbleToDeleteParameterValues() throws Exception {
        testBothMaterials(this::itShouldBeAbleToDeleteParameterValues);
    }

    private void itShouldBeAbleToDeleteParameterValues(boolean isComposite, int materialId) {
        List<ParameterSample> parameterPointsBeforeTest =
                projectCreator.getParameterPoints(isComposite, materialId, testParamId);
        projectCreator.addParameterToMaterial(testParamId, materialId);
        List<ParameterSample> parameterPoints = projectCreator.getParameterPoints(isComposite, materialId, testParamId);
        assertEquals(parameterPointsBeforeTest.size() + 1, parameterPoints.size());

        String url = createMaterialParametersUrl(isComposite, materialId, testParamId, parameterPoints.get(0).getId());
        DELETEobject(mockMvc, url);

        parameterPoints = projectCreator.getParameterPoints(isComposite, materialId, testParamId);
        assertEquals(parameterPointsBeforeTest.size(), parameterPoints.size());
    }


    @Test
    public void defaultParametersShouldContainValuesAtDefaultTemp() throws Exception {
        testBothMaterials(this::defaultParametersShouldContainValuesAtDefaultTemp);
    }

    private void defaultParametersShouldContainValuesAtDefaultTemp(boolean isComposite, int materialId) {
        BigDecimal expected = BigDecimal.valueOf(100d);
        projectCreator.addParameterToMaterial(testParamId, materialId,
                projectCreator.newParameterPoint(ParametersManager.DEFAULT_TEMPERATURE, expected));

        List<DefaultParameterValue> defaultParameters = projectCreator.getDefaultParameters(isComposite, materialId);
        BigDecimal actual = defaultParameters.stream()
                .filter(pv -> pv.getId() == testParamId)
                .map(DefaultParameterValue::getParamValue)
                .filter(Objects::nonNull)
                .findAny().orElse(null);
        assertEquals(expected, actual);
    }


    private void assertParameterPointsEquals(List<ParameterSample> expected, List<ParameterSample> actual) {
        assertEquals(expected.size(), actual.size());
        Collections.sort(expected, Comparator.comparing(ParameterSample::getTemperature));
        Collections.sort(actual, Comparator.comparing(ParameterSample::getTemperature));
        Collections.sort(expected, Comparator.comparing(ParameterSample::getTemperature));
        Collections.sort(actual, Comparator.comparing(ParameterSample::getTemperature));
        for (int i = 0; i < expected.size(); ++i) {
            assertParamPointsEquals(expected.get(i), actual.get(i));
        }
    }


    private void assertParamPointsEquals(ParameterSample expected, ParameterSample actual) {
        assertEquals(expected.getTemperature(), actual.getTemperature());
        assertEquals(expected.getFuncParamValue(), actual.getFuncParamValue());
        assertEquals(expected.getSimParamValue(), actual.getSimParamValue());
        assertEquals(expected.getParamValue(), actual.getParamValue());
    }


    private void testBothMaterials(BiConsumer<Boolean, Integer> testMethod) throws Exception {
        MaterialIn newMaterial = projectCreator.newMaterial();
        MaterialEntity materialEntity = CREATEobject(mockMvc, MATERIALS_URL, newMaterial, MaterialEntity.class);
        testMethod.accept(false, materialEntity.getId());

        CompositeMaterialIn newCopositeMaterial = projectCreator.newCompositeMaterial();
        CompositeMaterialEntity compositeMaterialEntity =
                CREATEobject(mockMvc, COMPOSITE_MATERIALS_URL, newCopositeMaterial, CompositeMaterialEntity.class);
        testMethod.accept(true, compositeMaterialEntity.getId());
    }
}
